<?php

//  Copyright (C) 2005-2013 Solomon Peachy (pizza@shaftnet.org)

//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.

//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

$auth_ldap_url = "";
$auth_ldap_dn = "";

class po_auth_ldap {
  private $handle;
  private $dn;
  private $bound = -1; // 0 = anon, 1 = user, 2 = superuser

  function __construct() {    
    global $auth_ldap_url;
    global $auth_ldap_dn;

    $this->handle = ldap_connect($auth_ldap_url);

    if (!$this->handle) 
      return FALSE;

    $this->dn = $auth_ldap_dn;

    ldap_set_option($this->handle, LDAP_OPT_PROTOCOL_VERSION, 3);

#    ldap_set_option(NULL, LDAP_OPT_DEBUG_LEVEL, 7);
    if (!ldap_bind($this->handle))
      return FALSE;

    $this->bound = 1;
  }

  /* Clean up any resources obtained in the constructor */
  function __destruct() {
    ldap_unbind($this->handle); // closes too.
    $this->bound = -1;
    $this->handle = FALSE;
  }

  /* Arguments:
   $username = "some_plaintext_username";
   $password = "some_plaintext_password";
   
   Returns:
   FALSE if unsuccessful
   username if successful
  */

  function auth_user($username, $password) {
    if (empty($password))
	return FALSE;

    $r = ldap_search($this->handle, $this->dn, "uid=$username");
    
    $info = ldap_get_entries($this->handle, $r);
    if (!$info || !$info['count'])
      return FALSE;

    $info = $info[0];

    if (ldap_bind($this->handle, $info['dn'], $password)) {
      $this->bound = 1;
      return $username;
    } 

    return FALSE;
  }

  /* Arguments:
    $username = "some_plaintext_username";

   Returns:  array()

   THESE ARE REQUIRED FIELDS:

    'first_name' => 'firstname'
    'last_name' => 'lastname'
    'email' => 'email_addr'
    'type' => PO_USER_TYPE

    OPTIONAL FIELDS

    address1, address2, city, zipcode, state, country, phone, url

  */
  function user_info($username) {
    $res = array();

    if ($this->bound == -1) {  // bind anonymously...
      if (!ldap_bind($this->handle))
	return FALSE;
    }

    $r = ldap_search($this->handle, $this->dn, "uid=$username");
    
    $info = ldap_get_entries($this->handle, $r);
    if (!$info || !$info['count'])
      return FALSE;

    $info = $info[0];

    $res['last_name'] = $info['sn'][0];
    $res['first_name'] = $info['givenname'][0];
    $res['email'] = $info['mail'][0];
    $res['type'] = 4; // XXX -- ADMIN!
    
    return $res;
  }
  
  /* Mangles the password for storage in local database */
  function passwd_transform($password, $salt) {
    return $password;
  }
  
  public $can_change_pass = FALSE;
  public $username_is_email = FALSE; 
  public $local_register = FALSE;
  public $local_userinfo = TRUE;
}

?>
