<?php

//  Copyright (C) 2002-2006 Balint Kis (balint@k-i-s.net)
//  Copyright (C) 2005-2013 Solomon Peachy (pizza@shaftnet.org)

//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.

//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

include_once "include/config.php";
include_once "include/import.php";
include_once "include/site.php";
include_once "include/common.php";

/* Handle a search request */
$search_request     = isset($_REQUEST['search']);
if ($search_request) {
  $keyword_list = "";
  foreach ($_REQUEST['keywords'] as $keyword) {
    $keyword_list .= "$keyword%20";
 }
  if ($keyword_list != "") {
    header("Location: search.text.advanced.php?keyword=$keyword_list");
    exit();
  }
 }

$database = site_prolog(PO_USER_TYPE_USER);

$photo_id           = pg_escape_string($database, $_REQUEST['photo_id']);
$owner_id           = pg_escape_string($database, $_REQUEST['owner_id']);

$version_id         = pg_escape_string($database, $_REQUEST['version']);
$destination_folder = pg_escape_string($database, $_REQUEST['destination_folder']);
$destination_album  = pg_escape_string($database, $_REQUEST['destination_album']);
$source_album       = isset($_REQUEST['source_album']) ? pg_escape_string($database, $_REQUEST['source_album']) : "";
$source_folder      = pg_escape_string($database, $_REQUEST['source_folder']);

$detail_info        = pg_escape_string($database, $_REQUEST['detail_info']);
$transform          = pg_escape_string($database, $_REQUEST['transform']);

$transform_request  = isset($_REQUEST['rotate_req']);
$export_request     = isset($_REQUEST['export_req']);
$print_request      = isset($_REQUEST['print_req']);
$printer_request    = isset($_REQUEST['printer_req']);
$spool_request      = isset($_REQUEST['spool_req']);
$watermark_request  = isset($_REQUEST['watermark_req']);

$thumb_f_request    = isset($_REQUEST['thumb_f']);
$thumb_a_request    = isset($_REQUEST['thumb_a']);

$goto_f_request     = isset($_REQUEST['go_to_f']);
$goto_a_request     = isset($_REQUEST['go_to_a']);
$move_f_request     = isset($_REQUEST['move_to_f']);
$link_a_request     = isset($_REQUEST['link_to_a']);
$move_a_request     = isset($_REQUEST['move_to_a']);
$delete_request     = isset($_REQUEST['delete_req']);

$album_id = isset($_REQUEST['album']) ? pg_escape_string($database, $_REQUEST['album']) : FALSE;

/* Doesn't take arguments */
if ($export_request) {
  site_epilog($database);
  header("Location: export.php?export_type=$_REQUEST[export_type]");
  exit();
 }
 
/* Doesn't take arguments */
if ($print_request) {
  site_epilog($database);
  header("Location: print.php?print_type=$_REQUEST[print_type]");
  exit();
}

/* Doesn't take arguments */
if ($printer_request) {
  site_epilog($database);
  header("Location: printer.php?printer=$_REQUEST[printer]&photo_id=$_REQUEST[photo_id]&version_id=$_REQUEST[version]");
  exit();
}

/* Doesn't take arguments */
if ($goto_f_request && ($destination_folder != "")) {
  site_epilog($database);
  header("Location: ".generate_link('folder', $destination_folder, array(), TRUE));
  exit();
}

/* Doesn't take arguments */
if ($goto_a_request && ($destination_album != "")) {
  site_epilog($database);
  header("Location: ".generate_link('album', $destination_album, array(), TRUE));
  exit();
}

/* Redirect a trash request appropriately */
if ($delete_request) {
  if ($source_album != "") {
    $move_a_request = TRUE;
    $destination_album = "trash";
  } else {
    $move_f_request = TRUE;
    if ($po_user['type'] == PO_USER_TYPE_ADMIN) {
      $destination_folder = "(select identifier from folder where users = $owner_id and caption = 'Trash')";
    } else {
      $destination_folder = $po_user['trash_folder'];
    }
  }
 }

if ($spool_request) {
  $link_a_request = TRUE;
  $destination_album = $po_user['spool_album'];
 }

if ($delete_request) {
  if ($album_id) {
    $return_path = generate_link('album', $album_id);
  } else {
    $return_path = generate_link('folder', $source_folder);
  }
} else {
  $return_path = generate_link('photo', $photo_id, array('ver'=>$version_id, 'detail_info'=>$detail_info,'album'=>$album_id), TRUE);
}

/* Establish whether or not the viewer has appropriate rights */
$owns = FALSE; 
$can_link = FALSE;

if ($po_user['type'] == PO_USER_TYPE_ADMIN) {
  $owns = TRUE; 
  $can_link = TRUE;
 } else {
  $result = pg_query($database, "select users, can_access_photo(photo.identifier, $po_user[id], '{".$passwords."}') from photo where identifier='$photo_id'");
  $data = pg_fetch_row($result);
  if ($data[0] == $po_user['id']) {
    $owns = TRUE;
    $can_link = TRUE;
  } elseif ($data[1] == 't') {
    $can_link = TRUE;
  }
 }

/* Verify parameters and destination permissions */
if ($po_user['type'] != PO_USER_TYPE_ADMIN) {
  $can_access_dest = FALSE;
  $can_access_source = FALSE;
  $skip = FALSE;

  if ($move_f_request || $thumb_f_request) {
    $can_access_source = $owns;
    if ($destination_folder != "") {
      $result = pg_query($database, "select identifier from folder where identifier='$destination_folder' and users = $po_user[id]");
      $can_access_dest = (pg_num_rows($result) > 0);
    }
  } else if ($move_a_request || $thumb_a_request) {
    if ($destination_album != "") {
      $result = pg_query($database, "select identifier from album where identifier='$source_album' and users = $po_user[id]");      
      $can_access_source = (pg_num_rows($result) > 0);
      
      if ($destination_album == "trash") {
	$can_access_dest = TRUE;
      } elseif ($can_link) {
	$result = pg_query($database, "select identifier from album where identifier='$destination_album' and users = $po_user[id]");      
	$can_access_dest = (pg_num_rows($result) > 0);
      }
    }
  } else if ($link_a_request || $thumb_a_request || $watermark_request) {
    $can_access_source = $can_link;
    if (($destination_album != "") &&
	($destination_album != "trash")) {
      $result = pg_query($database, "select identifier from album where identifier='$destination_album' and users = $po_user[id]");      
      $can_access_dest = (pg_num_rows($result) > 0);
    }
  } else if ($spool_request && $can_link)
    $can_access_dest = TRUE;
  if ($watermark_request && $can_link)
    $can_access_dest = TRUE;
  if ($transform_request && $owns) {
    $can_access_dest = TRUE;
    $can_access_source = TRUE;
  } else {
    $skip = TRUE;
  }

  if (!$skip && (!$can_access_dest || !$can_access_source)) {
    site_epilog($database);
    site_push_error($strings['errors_not_owner']);
    header("Location: $return_path");
    exit();
  }
}

/* ===  If we got here, we can perform whatever action was requested === */

/* Perform the actual work! */
if ($move_f_request && ($destination_folder != "")) {
   pg_query($database, "update photo set folder = $destination_folder where identifier='$photo_id'");
 } elseif (($link_a_request || $move_a_request) && ($destination_album != "")) {
   pg_query($database, "begin");

   /* Delete from the originating album (for moves/deletes) */
   if (($source_album != "") && $move_a_request) {
     $result = pg_query($database, "delete from album_content where album=$source_album and photo = $photo_id and version = $version_id ");
   }
   
   /* Add to the destination album (for linkings) */
   if ($destination_album != "trash") {
     /* Check for duplicates */
     $result = pg_fetch_row(pg_query($database, "select count(*) from album_content where photo = $photo_id and version = $version_id and album = $destination_album"));
     if ($result[0] == 0) {
       /* Add to album */
       $result = pg_query($database, "insert into album_content (photo, version, album)
                             values ($photo_id, $version_id, $destination_album)");
     }
   }
   
  if (!$result) {
    site_push_error($strings['errors_db_insert_failed']);
    pg_query($database, "rollback");
  } else {
    pg_query($database, "commit");
  }

 } elseif ($watermark_request) {
   $row = pg_fetch_row(pg_query($database, "select path from files where version=$version_id and size = 0"));
   store_user_pref($database, $po_user['id'], 'watermark_photo', $version_id);
   store_user_pref($database, $po_user['id'], 'watermark_path', $row[0]);
} elseif ($transform_request && ($transform != "")) {

  /* Rotate, etc.. This gets complicated as we have to worry about
   volume allocation */

  if (($current_volume = get_current_volume($database, FALSE)) == FALSE) {
    site_push_error($strings['errors_failed_volume']);
    site_epilog($database);
    header("Location: $return_path");
  }

  $cleanup_files = array();

  pg_query($database, "begin");

  $version_selector = $version_id ? "photo_version.identifier=$version_id" : "photo_version.master='t'";

  $next_index = pg_fetch_row(pg_query($database, "select nextval('photo_id_sequence')"));

  $photo_data = pg_fetch_assoc(pg_query($database, "
    select get_image_path(photo_version.identifier, 0) as original_path,
           colorspace, orientation, photo_version.identifier as version, camera  
      from photo, photo_version, photo_tech
     where photo.identifier = photo_version.photo
       and photo.identifier = photo_tech.photo
       and $version_selector
       and photo.identifier = '$photo_id'"));

  if ($photo_data['camera']) {
    $camera_profile = pg_fetch_row(pg_query($database, "
    select raw_icc_profile 
      from camera, camera_type
       where camera.identifier = $photo_data[camera]
       and camera.type = camera_type.identifier"));
  } else {
    $camera_profile[0] = FALSE;
  }

  $old_files = array();
  $res = pg_query($database, "select identifier, path from files where version = $version_id and size != 0");
  for ($i = 0 ; $i < pg_num_rows($res) ; $i++) {
    $row = pg_fetch_row($res);
    $old_files[$row[0]] = $row[1];
  }

  $original = $original_name = $image_repository_path ."/". $photo_data['original_path'];
  $original_file_type   = strtolower(substr($original_name, strrpos($original_name, ".") + 1));

  $colorspace = $photo_data['colorspace'];
  $orientation = $photo_data['orientation'];
  $version_id = $photo_data['version'];

  $orientation = map_transform($orientation, $transform);

  $retval = pg_query($database, "update photo_version set orientation = $orientation where $version_selector");

  $orientation = orientation_to_xform($database, $orientation);

  $multi_page_parameter = $mime_type[$original_file_type]['page'];
  $decoder              = $mime_type[$original_file_type]['decoder'];

  $index = 0;
  $image_data = array();
  $image_data['jpgfromraw'] = FALSE;
  $image_data['po_options'] = $po_options;
  $image_data['po_user'] = $po_user;
  $image_data['file'][$index]['orientation_xform'] = $orientation;
  $image_data['file'][$index]['colorspace'] = $photo_data['colorspace'];

  // XXX preserve comments on new files!

  $output = ""; // We ignore it anyway...

  /* RAW Decode */
  if ($retval) {
    $temporary_ppm_file = import_decode($original_name, $decoder, $camera_profile[0], $image_data, $index, $output);
    if ($temporary_ppm_file) {
      $original_name = $temporary_ppm_file;
    } 
  }

  /* Generate scaled images */
  if ($retval) {
    $retval = import_generate_all_scaled($original_name, $multi_page_parameter, $original, $database, $version_id, '', '', $current_volume, $next_index[0], $cleanup_files, $original, $image_data, $index, $output);
  }

  if ($temporary_ppm_file) {
    unlink($temporary_ppm_file);
  }

  /* Delete the old files! */
  if ($retval) {
    foreach ($old_files as $file => $path) {
      $retval = pg_query($database, "delete from files where identifier = $file");
    }
  }
  if ($retval) {
      $err = error_reporting(0);
      foreach ($old_files as $file => $path) {
	unlink("$image_repository_path/$path");
      }
      error_reporting($err);
  }

  if ($retval) {
    pg_query($database, "commit");
  } else {
    pg_query($database, "rollback");
    $err = error_reporting(0);
    foreach ($cleanup_files as $file) {
      unlink($file);
    }
    error_reporting($err);
  }
} elseif ($thumb_a_request && $destination_album) {
  $result = pg_query($database, "update album set thumb_ver = $version_id where identifier = $destination_album");
  if (!$result) {
    site_push_error($strings['errors_db_insert_failed']);
  }
} elseif ($thumb_f_request && $destination_folder) {
  $result = pg_query($database, "update folder set thumb_ver = $version_id where identifier = $destination_folder");
  if (!$result) {
    site_push_error($strings['errors_db_insert_failed']);
  }
}


site_epilog($database);
header("Location: $return_path");
exit();

?>
