--
-- PostgreSQL database dump
--

SET client_encoding = 'UTF8';
SET check_function_bodies = false;
SET client_min_messages = warning;

SET search_path = public, pg_catalog;

--
-- Name: folder_parents; Type: TYPE; Schema: public; Owner: po_user
--

CREATE TYPE folder_parents AS (
	identifier integer,
	caption text
);


--
-- Name: album_and_subalbums(integer); Type: FUNCTION; Schema: public; Owner: po_user
--

CREATE FUNCTION album_and_subalbums(integer) RETURNS SETOF integer
    AS ' DECLARE ident record; ident2 record; BEGIN RETURN NEXT $1; FOR ident IN SELECT identifier FROM album f WHERE f.parent_album = $1 LOOP FOR ident2 IN SELECT * FROM album_and_subalbums(ident.identifier) LOOP RETURN NEXT ident2.album_and_subalbums; END LOOP; END LOOP;  RETURN; END; '
    LANGUAGE plpgsql;


--
-- Name: album_content_date_added_trig(); Type: FUNCTION; Schema: public; Owner: po_user
--

CREATE FUNCTION album_content_date_added_trig() RETURNS "trigger"
    AS ' BEGIN  IF TG_OP = ''DELETE'' THEN UPDATE album SET date_changed = now() WHERE album.identifier = OLD.album; RETURN OLD; END IF;  NEW.date_added = now();  UPDATE album SET date_changed = now() WHERE album.identifier = NEW.album;  RETURN NEW;  END '
    LANGUAGE plpgsql;


--
-- Name: can_access_album(integer, integer, character varying[]); Type: FUNCTION; Schema: public; Owner: po_user
--

CREATE FUNCTION can_access_album(integer, integer, character varying[]) RETURNS boolean
    AS ' DECLARE res_id alias for $1; user_id alias for $2; passwd_list alias for $3; res_access integer; owns_id integer; can_access boolean; admin_type boolean; passwd varchar; BEGIN     SELECT INTO res_access, owns_id, passwd access_rights, users, password FROM album WHERE identifier = res_id;  IF res_access = 1 THEN RETURN true; END IF;  IF owns_id = user_id THEN RETURN true; END IF;  SELECT INTO admin_type true FROM users WHERE identifier = user_id AND type = 4; IF admin_type is true THEN RETURN true; END IF;  IF res_access = 2 THEN SELECT INTO can_access true FROM album, calendar WHERE album.event = calendar.identifier AND calendar.client = user_id;  IF can_access is true THEN RETURN true; END IF;  select into can_access true WHERE passwd = ANY(passwd_list); IF can_access is true THEN RETURN true; END IF; END IF;  return false; END; '
    LANGUAGE plpgsql;


--
-- Name: can_access_folder(integer, integer, character varying[]); Type: FUNCTION; Schema: public; Owner: po_user
--

CREATE FUNCTION can_access_folder(integer, integer, character varying[]) RETURNS boolean
    AS ' DECLARE res_id alias for $1; user_id alias for $2; passwd_list alias for $3; res_access integer; owns_id integer; can_access boolean; admin_type boolean; passwd varchar; BEGIN     SELECT INTO res_access, owns_id, passwd access_rights, users, password FROM folder WHERE identifier = res_id;  IF res_access = 1 THEN RETURN true; END IF;  IF owns_id = user_id THEN RETURN true; END IF;  SELECT INTO admin_type true FROM users WHERE identifier = user_id AND type = 4; IF admin_type is true THEN RETURN true; END IF;  IF res_access = 2 THEN SELECT INTO can_access true FROM folder, calendar WHERE folder.event = calendar.identifier AND calendar.client = user_id;  IF can_access is true THEN RETURN true; END IF;  select into can_access true WHERE passwd = ANY(passwd_list); IF can_access is true THEN RETURN true; END IF; END IF;  return false; END; '
    LANGUAGE plpgsql;


--
-- Name: can_access_photo(integer, integer, character varying[]); Type: FUNCTION; Schema: public; Owner: po_user
--

CREATE FUNCTION can_access_photo(integer, integer, character varying[]) RETURNS boolean
    AS ' DECLARE res_id alias for $1; user_id alias for $2; passwd_list alias for $3; res_access integer; owns_id integer; can_access boolean; admin_type boolean; BEGIN     SELECT INTO res_access, owns_id access_rights, users FROM photo WHERE identifier = res_id;  IF res_access = 1 THEN RETURN true; END IF;  IF owns_id = user_id THEN RETURN true; END IF;  SELECT INTO admin_type true FROM users WHERE identifier = user_id AND type = 4; IF admin_type is true THEN RETURN true; END IF;  IF res_access = 2 THEN SELECT INTO can_access true FROM photo, folder, calendar WHERE folder.identifier = photo.folder AND folder.event = calendar.identifier AND calendar.client = user_id; IF can_access is true THEN RETURN true; END IF;  SELECT INTO can_access true FROM photo, folder WHERE photo.identifier = res_id AND photo.folder = folder.identifier AND folder.password = ANY(passwd_list); IF can_access is true THEN RETURN true; END IF;  SELECT INTO can_access true FROM album_content, album, calendar WHERE album_content.photo = res_id AND album.identifier = album_content.album AND album.event = calendar.identifier AND calendar.client = user_id; IF can_access is true THEN RETURN true; END IF;  SELECT INTO can_access true FROM album, album_content WHERE album_content.photo = res_id AND album_content.album = album.identifier AND album.password = ANY(passwd_list); IF can_access is true THEN RETURN true; END IF;  END IF;  return false; END; '
    LANGUAGE plpgsql;


--
-- Name: concat_ignore_null(character varying, character varying); Type: FUNCTION; Schema: public; Owner: po_user
--

CREATE FUNCTION concat_ignore_null(character varying, character varying) RETURNS character varying
    AS ' declare string_1         alias for $1; string_2         alias for $2; begin if string_1 is null then return string_2; end if;  if string_2 is null then return string_1; end if;  return string_1||string_2; end; '
    LANGUAGE plpgsql;


--
-- Name: count_albums_by_user(integer); Type: FUNCTION; Schema: public; Owner: po_user
--

CREATE FUNCTION count_albums_by_user(integer) RETURNS bigint
    AS 'select count(identifier) from album where users = $1'
    LANGUAGE sql;


--
-- Name: count_albums_by_user(integer, integer, character varying[]); Type: FUNCTION; Schema: public; Owner: po_user
--

CREATE FUNCTION count_albums_by_user(integer, integer, character varying[]) RETURNS bigint
    AS ' select count(identifier) from album where users = $1 and can_access_album(identifier, $2, $3) '
    LANGUAGE sql;


--
-- Name: count_equipment_by_user(integer); Type: FUNCTION; Schema: public; Owner: po_user
--

CREATE FUNCTION count_equipment_by_user(integer) RETURNS bigint
    AS ' select ((select count(identifier) from lens where users = $1) + (select count(identifier) from camera where users = $1) + (select count(identifier) from scanner where users = $1) + (select count(identifier) from film where users = $1) + (select count(identifier) from support where users = $1) + (select count(identifier) from flash where users = $1)) '
    LANGUAGE sql;


--
-- Name: count_folders_by_user(integer); Type: FUNCTION; Schema: public; Owner: po_user
--

CREATE FUNCTION count_folders_by_user(integer) RETURNS bigint
    AS 'select count(identifier) from folder where users = $1'
    LANGUAGE sql;


--
-- Name: count_folders_by_user(integer, integer, character varying[]); Type: FUNCTION; Schema: public; Owner: po_user
--

CREATE FUNCTION count_folders_by_user(integer, integer, character varying[]) RETURNS bigint
    AS ' select count(identifier) from folder where users = $1 and can_access_folder(identifier, $2, $3) '
    LANGUAGE sql;


--
-- Name: count_photo_versions(); Type: FUNCTION; Schema: public; Owner: po_user
--

CREATE FUNCTION count_photo_versions() RETURNS bigint
    AS 'select count(identifier) from photo_version'
    LANGUAGE sql;


--
-- Name: count_photo_versions_by_user(integer); Type: FUNCTION; Schema: public; Owner: po_user
--

CREATE FUNCTION count_photo_versions_by_user(integer) RETURNS bigint
    AS 'select count(photo_version.identifier) from photo_version, photo where photo_version.photo=photo.identifier and photo.users = $1'
    LANGUAGE sql;


--
-- Name: count_photos(); Type: FUNCTION; Schema: public; Owner: po_user
--

CREATE FUNCTION count_photos() RETURNS bigint
    AS 'select count(identifier) from photo'
    LANGUAGE sql;


--
-- Name: count_photos(integer); Type: FUNCTION; Schema: public; Owner: po_user
--

CREATE FUNCTION count_photos(integer) RETURNS bigint
    AS 'select count(identifier) from photo where access_rights <= $1'
    LANGUAGE sql;


--
-- Name: count_photos_by_album(integer, integer, character varying[]); Type: FUNCTION; Schema: public; Owner: po_user
--

CREATE FUNCTION count_photos_by_album(integer, integer, character varying[]) RETURNS bigint
    AS 'select count(photo) from album_content where album = $1 and can_access_photo(photo, $2, $3); '
    LANGUAGE sql;


--
-- Name: count_photos_by_folder(integer, integer, character varying[]); Type: FUNCTION; Schema: public; Owner: po_user
--

CREATE FUNCTION count_photos_by_folder(integer, integer, character varying[]) RETURNS bigint
    AS ' select count(identifier) from photo where folder = $1 and can_access_photo(identifier, $2, $3); '
    LANGUAGE sql;


--
-- Name: count_photos_by_user(integer); Type: FUNCTION; Schema: public; Owner: po_user
--

CREATE FUNCTION count_photos_by_user(integer) RETURNS bigint
    AS 'select count(identifier) from photo where users = $1'
    LANGUAGE sql;


--
-- Name: count_photos_by_user(integer, integer, character varying[]); Type: FUNCTION; Schema: public; Owner: po_user
--

CREATE FUNCTION count_photos_by_user(integer, integer, character varying[]) RETURNS bigint
    AS ' select count(identifier) from photo where users = $1 and can_access_photo(identifier, $2, $3) '
    LANGUAGE sql;


--
-- Name: count_subalbums_by_album(integer, integer, character varying[]); Type: FUNCTION; Schema: public; Owner: po_user
--

CREATE FUNCTION count_subalbums_by_album(integer, integer, character varying[]) RETURNS bigint
    AS ' select count(identifier) from album where parent_album = $1 and can_access_album(identifier, $2, $3); '
    LANGUAGE sql;


--
-- Name: count_subfolders_by_folder(integer, integer, character varying[]); Type: FUNCTION; Schema: public; Owner: po_user
--

CREATE FUNCTION count_subfolders_by_folder(integer, integer, character varying[]) RETURNS bigint
    AS ' select count(identifier) from folder where parent_folder = $1 and can_access_folder(identifier, $2, $3); '
    LANGUAGE sql;


--
-- Name: count_users(); Type: FUNCTION; Schema: public; Owner: po_user
--

CREATE FUNCTION count_users() RETURNS bigint
    AS 'select count(identifier) from users'
    LANGUAGE sql;


--
-- Name: date_added_trig(); Type: FUNCTION; Schema: public; Owner: po_user
--

CREATE FUNCTION date_added_trig() RETURNS "trigger"
    AS ' BEGIN  IF TG_OP = ''INSERT'' THEN NEW.date_of_creation = now(); END IF;  RETURN NEW;  END; '
    LANGUAGE plpgsql;


--
-- Name: date_changed_album(); Type: FUNCTION; Schema: public; Owner: po_user
--

CREATE FUNCTION date_changed_album() RETURNS "trigger"
    AS ' BEGIN  IF TG_OP = ''DELETE'' THEN UPDATE album SET date_changed = now() where identifier = OLD.parent_album; RETURN OLD; END IF;  IF TG_OP = ''INSERT'' THEN NEW.date_of_creation = now(); UPDATE album SET date_changed = now() WHERE identifier = NEW.parent_album; END IF;  NEW.date_changed = now(); RETURN NEW;  END; '
    LANGUAGE plpgsql;


--
-- Name: date_changed_album_content(); Type: FUNCTION; Schema: public; Owner: po_user
--

CREATE FUNCTION date_changed_album_content() RETURNS "trigger"
    AS ' BEGIN  IF TG_OP = ''DELETE'' THEN UPDATE album SET date_changed = now() WHERE identifier = OLD.album; RETURN OLD; END IF;  IF TG_OP = ''INSERT'' THEN UPDATE album SET date_changed = now() WHERE identifier = NEW.album; END IF;  RETURN NEW;  END; '
    LANGUAGE plpgsql;


--
-- Name: date_changed_files(); Type: FUNCTION; Schema: public; Owner: po_user
--

CREATE FUNCTION date_changed_files() RETURNS "trigger"
    AS ' BEGIN  IF TG_OP = ''INSERT'' THEN NEW.created = now(); END IF;  RETURN NEW;  END; '
    LANGUAGE plpgsql;


--
-- Name: date_changed_folder(); Type: FUNCTION; Schema: public; Owner: po_user
--

CREATE FUNCTION date_changed_folder() RETURNS "trigger"
    AS ' BEGIN  IF TG_OP = ''DELETE'' THEN UPDATE folder SET date_changed = now() where identifier = OLD.parent_folder; RETURN OLD; END IF;  IF TG_OP = ''INSERT'' THEN NEW.date_of_creation = now(); UPDATE folder SET date_changed = now() WHERE identifier = NEW.parent_folder; END IF;  NEW.date_changed = now(); RETURN NEW;  END; '
    LANGUAGE plpgsql;


--
-- Name: folder_and_subfolders(integer); Type: FUNCTION; Schema: public; Owner: po_user
--

CREATE FUNCTION folder_and_subfolders(integer) RETURNS SETOF integer
    AS ' DECLARE ident record; ident2 record; BEGIN RETURN NEXT $1; FOR ident IN SELECT identifier FROM folder f WHERE f.parent_folder = $1 LOOP FOR ident2 IN SELECT * FROM folder_and_subfolders(ident.identifier) LOOP RETURN NEXT ident2.folder_and_subfolders; END LOOP; END LOOP;  RETURN; END; '
    LANGUAGE plpgsql;


--
-- Name: get_album_with_parents(integer); Type: FUNCTION; Schema: public; Owner: po_user
--

CREATE FUNCTION get_album_with_parents(integer) RETURNS SETOF folder_parents
    AS ' DECLARE album_id INT; row RECORD; retval RECORD; BEGIN album_id := $1; WHILE album_id IS NOT NULL LOOP SELECT INTO row identifier, parent_album, caption FROM album WHERE identifier = album_id LIMIT 1; SELECT INTO retval row.identifier, row.caption; RETURN NEXT retval; album_id := row.parent_album; END LOOP;  RETURN; END; '
    LANGUAGE plpgsql;


--
-- Name: get_folder_with_parents(integer); Type: FUNCTION; Schema: public; Owner: po_user
--

CREATE FUNCTION get_folder_with_parents(integer) RETURNS SETOF folder_parents
    AS ' DECLARE folder_id INT; row RECORD; retval RECORD; BEGIN folder_id := $1; WHILE folder_id IS NOT NULL LOOP SELECT INTO row identifier, parent_folder, caption FROM folder WHERE identifier = folder_id LIMIT 1; SELECT INTO retval row.identifier, row.caption; RETURN NEXT retval; folder_id := row.parent_folder; END LOOP;  RETURN; END; '
    LANGUAGE plpgsql;


--
-- Name: get_image_path(integer, integer); Type: FUNCTION; Schema: public; Owner: po_user
--

CREATE FUNCTION get_image_path(integer, integer) RETURNS text
    AS ' SELECT path FROM files WHERE version = $1 AND size = $2; '
    LANGUAGE sql;


--
-- Name: get_user_pref(integer, text); Type: FUNCTION; Schema: public; Owner: po_user
--

CREATE FUNCTION get_user_pref(integer, text) RETURNS text
    AS ' SELECT value FROM user_preferences WHERE owner = $1 AND key = $2; '
    LANGUAGE sql;


--
-- Name: number_of_camera_references(integer); Type: FUNCTION; Schema: public; Owner: po_user
--

CREATE FUNCTION number_of_camera_references(integer) RETURNS bigint
    AS 'select count(photo_tech.identifier) from photo_tech, camera where photo_tech.camera=camera.identifier and camera.identifier = $1'
    LANGUAGE sql;


--
-- Name: number_of_dupes(integer); Type: FUNCTION; Schema: public; Owner: po_user
--

CREATE FUNCTION number_of_dupes(integer) RETURNS bigint
    AS 'select count(identifier) from photo_dupe where photo = $1'
    LANGUAGE sql;


--
-- Name: number_of_film_references(integer); Type: FUNCTION; Schema: public; Owner: po_user
--

CREATE FUNCTION number_of_film_references(integer) RETURNS bigint
    AS 'select count(photo_tech.identifier) from photo_tech, film where photo_tech.film=film.identifier and film.identifier = $1'
    LANGUAGE sql;


--
-- Name: number_of_filter_references(integer); Type: FUNCTION; Schema: public; Owner: po_user
--

CREATE FUNCTION number_of_filter_references(integer) RETURNS bigint
    AS 'select count(photo_tech.identifier) from photo_tech, filter where photo_tech.filter=filter.identifier and filter.identifier = $1'
    LANGUAGE sql;


--
-- Name: number_of_flash_references(integer); Type: FUNCTION; Schema: public; Owner: po_user
--

CREATE FUNCTION number_of_flash_references(integer) RETURNS bigint
    AS 'select count(photo_tech.identifier) from photo_tech, flash where photo_tech.flash=flash.identifier and flash.identifier = $1'
    LANGUAGE sql;


--
-- Name: number_of_lens_references(integer); Type: FUNCTION; Schema: public; Owner: po_user
--

CREATE FUNCTION number_of_lens_references(integer) RETURNS bigint
    AS 'select count(photo_tech.identifier) from photo_tech, lens where photo_tech.lens=lens.identifier and lens.identifier = $1'
    LANGUAGE sql;


--
-- Name: number_of_location_references(integer); Type: FUNCTION; Schema: public; Owner: po_user
--

CREATE FUNCTION number_of_location_references(integer) RETURNS bigint
    AS 'select count(photo.identifier) from photo, location where photo.location=location.identifier and location.identifier = $1'
    LANGUAGE sql;


--
-- Name: number_of_manufacturer_ref(integer); Type: FUNCTION; Schema: public; Owner: po_user
--

CREATE FUNCTION number_of_manufacturer_ref(integer) RETURNS bigint
    AS ' select ((select count(identifier) from lens_type where manufacturer = $1) + (select count(identifier) from camera_type where manufacturer = $1) + (select count(identifier) from scanner_type where manufacturer = $1) + (select count(identifier) from support_type where manufacturer = $1) + (select count(identifier) from film_type where manufacturer = $1) + (select count(identifier) from flash_type where manufacturer = $1)) '
    LANGUAGE sql;


--
-- Name: number_of_scanner_references(integer); Type: FUNCTION; Schema: public; Owner: po_user
--

CREATE FUNCTION number_of_scanner_references(integer) RETURNS bigint
    AS 'select count(photo_tech.identifier) from photo_tech, scanner where photo_tech.scanner=scanner.identifier and scanner.identifier = $1'
    LANGUAGE sql;


--
-- Name: number_of_support_references(integer); Type: FUNCTION; Schema: public; Owner: po_user
--

CREATE FUNCTION number_of_support_references(integer) RETURNS bigint
    AS 'select count(photo_tech.identifier) from photo_tech, support where photo_tech.support=support.identifier and support.identifier = $1'
    LANGUAGE sql;


--
-- Name: photo_date_changed_trig(); Type: FUNCTION; Schema: public; Owner: po_user
--

CREATE FUNCTION photo_date_changed_trig() RETURNS "trigger"
    AS ' BEGIN  IF TG_OP = ''DELETE'' THEN UPDATE folder SET date_changed = now() where identifier = OLD.folder; RETURN OLD; END IF;  IF TG_OP = ''INSERT'' THEN UPDATE folder SET date_changed = now() where identifier = NEW.folder; NEW.date_changed = now(); NEW.date_added = now(); RETURN NEW; END IF;  IF NEW.views = OLD.views THEN NEW.date_changed = now(); END IF;  IF NEW.folder != OLD.folder THEN UPDATE folder SET date_changed = now() where identifier = NEW.folder; UPDATE folder SET date_changed = now() where identifier = OLD.folder; END IF;  RETURN NEW;  END; '
    LANGUAGE plpgsql;


--
-- Name: photo_shop_transaction_references(integer); Type: FUNCTION; Schema: public; Owner: po_user
--

CREATE FUNCTION photo_shop_transaction_references(integer) RETURNS bigint
    AS 'select count(shop_order.transaction) from photo_item, shop_order where photo_item.photo = $1 and photo_item.identifier=shop_order.item'
    LANGUAGE sql;


--
-- Name: photo_tech_date_changed_trig(); Type: FUNCTION; Schema: public; Owner: po_user
--

CREATE FUNCTION photo_tech_date_changed_trig() RETURNS "trigger"
    AS ' BEGIN  IF TG_OP = ''DELETE'' THEN UPDATE photo SET date_changed = now() WHERE photo.identifier = OLD.photo; RETURN OLD; END IF;  UPDATE photo SET date_changed = now() WHERE photo.identifier = NEW.photo;  RETURN NEW;  END; '
    LANGUAGE plpgsql;


--
-- Name: photo_version_date_changed_trig(); Type: FUNCTION; Schema: public; Owner: po_user
--

CREATE FUNCTION photo_version_date_changed_trig() RETURNS "trigger"
    AS ' BEGIN  IF TG_OP = ''DELETE'' THEN UPDATE photo SET date_changed = now() WHERE photo.identifier = OLD.photo;  RETURN OLD; END IF;  IF TG_OP = ''INSERT'' THEN NEW.date_of_creation = now(); END IF;  UPDATE photo SET date_changed = now() WHERE photo.identifier = NEW.photo;  NEW.date_changed = now();  RETURN NEW;  END; '
    LANGUAGE plpgsql;


--
-- Name: tokenized_ilike(character varying, character varying); Type: FUNCTION; Schema: public; Owner: po_user
--

CREATE FUNCTION tokenized_ilike(character varying, character varying) RETURNS boolean
    AS ' declare string alias for $1; keyword alias for $2; begin return tokenized_like(lower(string), lower(keyword)); end; '
    LANGUAGE plpgsql;


--
-- Name: tokenized_ilike(integer, character varying); Type: FUNCTION; Schema: public; Owner: po_user
--

CREATE FUNCTION tokenized_ilike(integer, character varying) RETURNS boolean
    AS ' declare identifier alias for $1; keyword alias for $2; begin return tokenized_ilike(text(identifier), keyword); end; '
    LANGUAGE plpgsql;


--
-- Name: tokenized_like(character varying, character varying); Type: FUNCTION; Schema: public; Owner: po_user
--

CREATE FUNCTION tokenized_like(character varying, character varying) RETURNS boolean
    AS ' declare string alias for $1; keyword alias for $2;  token varchar; character char; string_length integer; token_length integer; token_begin integer; index_cursor integer; head_cursor integer; begin head_cursor := 1; string_length := char_length(string);  while head_cursor < string_length loop token_length := 0; index_cursor := head_cursor; token_begin := head_cursor; character := substring(string from index_cursor for 1); while index_cursor <= string_length loop if ((character = '' '') or (character = '';'') or (index_cursor = string_length)) then if (index_cursor = string_length) then token_length := token_length + 1; end if; token := substring(string from token_begin for token_length); if (token = keyword) then return true; end if; end if; token_length := token_length + 1; index_cursor := index_cursor + 1; character := substring(string from index_cursor for 1); end loop;  index_cursor := head_cursor; while index_cursor <= string_length loop character := substring(string from index_cursor for 1); if ((character = '' '') or (character = '';'') or (index_cursor = string_length)) then head_cursor := index_cursor + 1; exit; end if; index_cursor := index_cursor + 1; end loop; end loop;  return false; end; '
    LANGUAGE plpgsql;


--
-- Name: tokenized_like(integer, character varying); Type: FUNCTION; Schema: public; Owner: po_user
--

CREATE FUNCTION tokenized_like(integer, character varying) RETURNS boolean
    AS ' declare identifier alias for $1; keyword alias for $2; begin return tokenized_like(text(identifier), keyword); end; '
    LANGUAGE plpgsql;


--
-- Name: =##=; Type: OPERATOR; Schema: public; Owner: po_user
--

CREATE OPERATOR =##= (
    PROCEDURE = tokenized_ilike,
    LEFTARG = character varying,
    RIGHTARG = character varying,
    COMMUTATOR = =##=
);


--
-- Name: =##=; Type: OPERATOR; Schema: public; Owner: po_user
--

CREATE OPERATOR =##= (
    PROCEDURE = tokenized_ilike,
    LEFTARG = integer,
    RIGHTARG = character varying,
    COMMUTATOR = =##=
);


--
-- Name: ====; Type: OPERATOR; Schema: public; Owner: po_user
--

CREATE OPERATOR ==== (
    PROCEDURE = tokenized_like,
    LEFTARG = character varying,
    RIGHTARG = character varying,
    COMMUTATOR = ====
);


--
-- Name: ====; Type: OPERATOR; Schema: public; Owner: po_user
--

CREATE OPERATOR ==== (
    PROCEDURE = tokenized_like,
    LEFTARG = integer,
    RIGHTARG = character varying,
    COMMUTATOR = ====
);


--
-- Name: |||; Type: OPERATOR; Schema: public; Owner: po_user
--

CREATE OPERATOR ||| (
    PROCEDURE = concat_ignore_null,
    LEFTARG = character varying,
    RIGHTARG = character varying
);


SET default_tablespace = '';

SET default_with_oids = false;

--
-- Name: access_type; Type: TABLE; Schema: public; Owner: po_user; Tablespace: 
--

CREATE TABLE access_type (
    identifier integer NOT NULL,
    value character varying(10)
);


--
-- Name: access_type_id_sequence; Type: SEQUENCE; Schema: public; Owner: po_user
--

CREATE SEQUENCE access_type_id_sequence
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: access_type_id_sequence; Type: SEQUENCE SET; Schema: public; Owner: po_user
--

SELECT pg_catalog.setval('access_type_id_sequence', 3, true);


--
-- Name: album; Type: TABLE; Schema: public; Owner: po_user; Tablespace: 
--

CREATE TABLE album (
    identifier integer NOT NULL,
    users integer NOT NULL,
    caption text,
    date_of_creation timestamp without time zone,
    access_rights integer NOT NULL,
    description text,
    parent_album integer,
    "password" text,
    date_changed timestamp without time zone NOT NULL,
    event integer
);


--
-- Name: album_content; Type: TABLE; Schema: public; Owner: po_user; Tablespace: 
--

CREATE TABLE album_content (
    identifier integer NOT NULL,
    photo integer NOT NULL,
    album integer NOT NULL,
    version integer NOT NULL,
    date_added timestamp without time zone NOT NULL
);


--
-- Name: album_content_id_sequence; Type: SEQUENCE; Schema: public; Owner: po_user
--

CREATE SEQUENCE album_content_id_sequence
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: album_content_id_sequence; Type: SEQUENCE SET; Schema: public; Owner: po_user
--

SELECT pg_catalog.setval('album_content_id_sequence', 1, false);


--
-- Name: album_id_sequence; Type: SEQUENCE; Schema: public; Owner: po_user
--

CREATE SEQUENCE album_id_sequence
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: album_id_sequence; Type: SEQUENCE SET; Schema: public; Owner: po_user
--

SELECT pg_catalog.setval('album_id_sequence', 1, true);


--
-- Name: calendar; Type: TABLE; Schema: public; Owner: po_user; Tablespace: 
--

CREATE TABLE calendar (
    identifier integer NOT NULL,
    users integer NOT NULL,
    start_date timestamp without time zone,
    end_date timestamp without time zone,
    date_of_creation timestamp without time zone,
    remark text,
    client integer
);


--
-- Name: calendar_id_sequence; Type: SEQUENCE; Schema: public; Owner: po_user
--

CREATE SEQUENCE calendar_id_sequence
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: calendar_id_sequence; Type: SEQUENCE SET; Schema: public; Owner: po_user
--

SELECT pg_catalog.setval('calendar_id_sequence', 1, false);


--
-- Name: camera; Type: TABLE; Schema: public; Owner: po_user; Tablespace: 
--

CREATE TABLE camera (
    identifier integer NOT NULL,
    "type" integer,
    users integer,
    serial_number character varying(20),
    date_of_purchase timestamp without time zone,
    purchased_new character(1),
    access_rights integer NOT NULL,
    ignore_comment boolean NOT NULL,
    CONSTRAINT camera_purchased_new_check CHECK (((purchased_new = 't'::bpchar) OR (purchased_new = 'f'::bpchar)))
);


--
-- Name: camera_id_sequence; Type: SEQUENCE; Schema: public; Owner: po_user
--

CREATE SEQUENCE camera_id_sequence
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: camera_id_sequence; Type: SEQUENCE SET; Schema: public; Owner: po_user
--

SELECT pg_catalog.setval('camera_id_sequence', 1, false);


--
-- Name: camera_metering; Type: TABLE; Schema: public; Owner: po_user; Tablespace: 
--

CREATE TABLE camera_metering (
    identifier integer NOT NULL,
    value character varying(40)
);


--
-- Name: camera_metering_id_sequence; Type: SEQUENCE; Schema: public; Owner: po_user
--

CREATE SEQUENCE camera_metering_id_sequence
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: camera_metering_id_sequence; Type: SEQUENCE SET; Schema: public; Owner: po_user
--

SELECT pg_catalog.setval('camera_metering_id_sequence', 5, true);


--
-- Name: camera_program; Type: TABLE; Schema: public; Owner: po_user; Tablespace: 
--

CREATE TABLE camera_program (
    identifier integer NOT NULL,
    value character varying(40)
);


--
-- Name: camera_program_id_sequence; Type: SEQUENCE; Schema: public; Owner: po_user
--

CREATE SEQUENCE camera_program_id_sequence
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: camera_program_id_sequence; Type: SEQUENCE SET; Schema: public; Owner: po_user
--

SELECT pg_catalog.setval('camera_program_id_sequence', 4, true);


--
-- Name: camera_type; Type: TABLE; Schema: public; Owner: po_user; Tablespace: 
--

CREATE TABLE camera_type (
    identifier integer NOT NULL,
    manufacturer integer NOT NULL,
    model character varying(100),
    variation character varying(100),
    last_modified_date timestamp without time zone,
    last_modifying_users integer,
    raw_icc_profile integer
);


--
-- Name: camera_type_id_sequence; Type: SEQUENCE; Schema: public; Owner: po_user
--

CREATE SEQUENCE camera_type_id_sequence
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: camera_type_id_sequence; Type: SEQUENCE SET; Schema: public; Owner: po_user
--

SELECT pg_catalog.setval('camera_type_id_sequence', 1, false);


--
-- Name: client; Type: TABLE; Schema: public; Owner: po_user; Tablespace: 
--

CREATE TABLE client (
    identifier integer NOT NULL,
    users integer NOT NULL,
    client integer NOT NULL,
    date_of_creation timestamp without time zone,
    status integer NOT NULL,
    "trusted" character(1),
    CONSTRAINT client_trusted_check CHECK ((("trusted" = 't'::bpchar) OR ("trusted" = 'f'::bpchar)))
);


--
-- Name: client_id_sequence; Type: SEQUENCE; Schema: public; Owner: po_user
--

CREATE SEQUENCE client_id_sequence
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: client_id_sequence; Type: SEQUENCE SET; Schema: public; Owner: po_user
--

SELECT pg_catalog.setval('client_id_sequence', 1, false);


--
-- Name: client_status; Type: TABLE; Schema: public; Owner: po_user; Tablespace: 
--

CREATE TABLE client_status (
    identifier integer NOT NULL,
    value character varying(12)
);


--
-- Name: client_status_id_sequence; Type: SEQUENCE; Schema: public; Owner: po_user
--

CREATE SEQUENCE client_status_id_sequence
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: client_status_id_sequence; Type: SEQUENCE SET; Schema: public; Owner: po_user
--

SELECT pg_catalog.setval('client_status_id_sequence', 3, true);


--
-- Name: country; Type: TABLE; Schema: public; Owner: po_user; Tablespace: 
--

CREATE TABLE country (
    identifier integer NOT NULL,
    code character varying(5),
    value character varying(40)
);


--
-- Name: country_id_sequence; Type: SEQUENCE; Schema: public; Owner: po_user
--

CREATE SEQUENCE country_id_sequence
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: country_id_sequence; Type: SEQUENCE SET; Schema: public; Owner: po_user
--

SELECT pg_catalog.setval('country_id_sequence', 240, true);


--
-- Name: ev_comp; Type: TABLE; Schema: public; Owner: po_user; Tablespace: 
--

CREATE TABLE ev_comp (
    identifier integer NOT NULL,
    value character varying(10)
);


--
-- Name: ev_comp_id_sequence; Type: SEQUENCE; Schema: public; Owner: po_user
--

CREATE SEQUENCE ev_comp_id_sequence
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: ev_comp_id_sequence; Type: SEQUENCE SET; Schema: public; Owner: po_user
--

SELECT pg_catalog.setval('ev_comp_id_sequence', 41, true);


--
-- Name: files; Type: TABLE; Schema: public; Owner: po_user; Tablespace: 
--

CREATE TABLE files (
    identifier integer NOT NULL,
    version integer NOT NULL,
    size integer NOT NULL,
    x_res integer NOT NULL,
    y_res integer NOT NULL,
    path text NOT NULL,
    created timestamp without time zone NOT NULL,
    filesize integer NOT NULL,
    params text NOT NULL,
    comments text,
    watermark integer,
    sha1sum text,
    backup_volume text
);


--
-- Name: files_id_sequence; Type: SEQUENCE; Schema: public; Owner: po_user
--

CREATE SEQUENCE files_id_sequence
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: files_id_sequence; Type: SEQUENCE SET; Schema: public; Owner: po_user
--

SELECT pg_catalog.setval('files_id_sequence', 1, false);


--
-- Name: film; Type: TABLE; Schema: public; Owner: po_user; Tablespace: 
--

CREATE TABLE film (
    identifier integer NOT NULL,
    "type" integer NOT NULL,
    users integer NOT NULL,
    access_rights integer NOT NULL
);


--
-- Name: film_format; Type: TABLE; Schema: public; Owner: po_user; Tablespace: 
--

CREATE TABLE film_format (
    identifier integer NOT NULL,
    value character varying(10)
);


--
-- Name: film_format_id_sequence; Type: SEQUENCE; Schema: public; Owner: po_user
--

CREATE SEQUENCE film_format_id_sequence
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: film_format_id_sequence; Type: SEQUENCE SET; Schema: public; Owner: po_user
--

SELECT pg_catalog.setval('film_format_id_sequence', 26, true);


--
-- Name: film_id_sequence; Type: SEQUENCE; Schema: public; Owner: po_user
--

CREATE SEQUENCE film_id_sequence
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: film_id_sequence; Type: SEQUENCE SET; Schema: public; Owner: po_user
--

SELECT pg_catalog.setval('film_id_sequence', 1, false);


--
-- Name: film_iso; Type: TABLE; Schema: public; Owner: po_user; Tablespace: 
--

CREATE TABLE film_iso (
    identifier integer NOT NULL,
    value integer
);


--
-- Name: film_iso_id_sequence; Type: SEQUENCE; Schema: public; Owner: po_user
--

CREATE SEQUENCE film_iso_id_sequence
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: film_iso_id_sequence; Type: SEQUENCE SET; Schema: public; Owner: po_user
--

SELECT pg_catalog.setval('film_iso_id_sequence', 31, true);


--
-- Name: film_type; Type: TABLE; Schema: public; Owner: po_user; Tablespace: 
--

CREATE TABLE film_type (
    identifier integer NOT NULL,
    manufacturer integer NOT NULL,
    model character varying(100),
    iso integer NOT NULL,
    format integer NOT NULL,
    last_modified_date timestamp without time zone,
    last_modifying_users integer
);


--
-- Name: film_type_id_sequence; Type: SEQUENCE; Schema: public; Owner: po_user
--

CREATE SEQUENCE film_type_id_sequence
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: film_type_id_sequence; Type: SEQUENCE SET; Schema: public; Owner: po_user
--

SELECT pg_catalog.setval('film_type_id_sequence', 1, false);


--
-- Name: filter; Type: TABLE; Schema: public; Owner: po_user; Tablespace: 
--

CREATE TABLE filter (
    identifier integer NOT NULL,
    "type" integer NOT NULL,
    users integer NOT NULL,
    date_of_purchase timestamp without time zone,
    purchased_new character(1),
    access_rights integer NOT NULL,
    serial_number character varying(20),
    CONSTRAINT filter_purchased_new_check CHECK (((purchased_new = 't'::bpchar) OR (purchased_new = 'f'::bpchar)))
);


--
-- Name: filter_id_sequence; Type: SEQUENCE; Schema: public; Owner: po_user
--

CREATE SEQUENCE filter_id_sequence
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: filter_id_sequence; Type: SEQUENCE SET; Schema: public; Owner: po_user
--

SELECT pg_catalog.setval('filter_id_sequence', 1, false);


--
-- Name: filter_type; Type: TABLE; Schema: public; Owner: po_user; Tablespace: 
--

CREATE TABLE filter_type (
    identifier integer NOT NULL,
    manufacturer integer NOT NULL,
    model character varying(100),
    variation character varying(100),
    last_modified_date timestamp without time zone,
    last_modifying_users integer
);


--
-- Name: filter_type_id_sequence; Type: SEQUENCE; Schema: public; Owner: po_user
--

CREATE SEQUENCE filter_type_id_sequence
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: filter_type_id_sequence; Type: SEQUENCE SET; Schema: public; Owner: po_user
--

SELECT pg_catalog.setval('filter_type_id_sequence', 1, false);


--
-- Name: flash; Type: TABLE; Schema: public; Owner: po_user; Tablespace: 
--

CREATE TABLE flash (
    identifier integer NOT NULL,
    "type" integer NOT NULL,
    users integer NOT NULL,
    serial_number character varying(20),
    date_of_purchase timestamp without time zone,
    purchased_new character(1),
    access_rights integer NOT NULL,
    CONSTRAINT flash_purchased_new_check CHECK (((purchased_new = 't'::bpchar) OR (purchased_new = 'f'::bpchar)))
);


--
-- Name: flash_id_sequence; Type: SEQUENCE; Schema: public; Owner: po_user
--

CREATE SEQUENCE flash_id_sequence
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: flash_id_sequence; Type: SEQUENCE SET; Schema: public; Owner: po_user
--

SELECT pg_catalog.setval('flash_id_sequence', 1, false);


--
-- Name: flash_mode; Type: TABLE; Schema: public; Owner: po_user; Tablespace: 
--

CREATE TABLE flash_mode (
    identifier integer NOT NULL,
    value character varying(40)
);


--
-- Name: flash_mode_id_sequence; Type: SEQUENCE; Schema: public; Owner: po_user
--

CREATE SEQUENCE flash_mode_id_sequence
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: flash_mode_id_sequence; Type: SEQUENCE SET; Schema: public; Owner: po_user
--

SELECT pg_catalog.setval('flash_mode_id_sequence', 3, true);


--
-- Name: flash_type; Type: TABLE; Schema: public; Owner: po_user; Tablespace: 
--

CREATE TABLE flash_type (
    identifier integer NOT NULL,
    manufacturer integer NOT NULL,
    model character varying(100),
    variation character varying(100),
    last_modified_date timestamp without time zone,
    last_modifying_users integer
);


--
-- Name: flash_type_id_sequence; Type: SEQUENCE; Schema: public; Owner: po_user
--

CREATE SEQUENCE flash_type_id_sequence
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: flash_type_id_sequence; Type: SEQUENCE SET; Schema: public; Owner: po_user
--

SELECT pg_catalog.setval('flash_type_id_sequence', 1, false);


--
-- Name: folder; Type: TABLE; Schema: public; Owner: po_user; Tablespace: 
--

CREATE TABLE folder (
    identifier integer NOT NULL,
    parent_folder integer,
    users integer NOT NULL,
    caption text,
    date_of_creation timestamp without time zone,
    access_rights integer NOT NULL,
    description text,
    "password" text,
    date_changed timestamp without time zone NOT NULL,
    event integer
);


--
-- Name: folder_id_sequence; Type: SEQUENCE; Schema: public; Owner: po_user
--

CREATE SEQUENCE folder_id_sequence
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: folder_id_sequence; Type: SEQUENCE SET; Schema: public; Owner: po_user
--

SELECT pg_catalog.setval('folder_id_sequence', 2, true);


--
-- Name: label; Type: TABLE; Schema: public; Owner: po_user; Tablespace: 
--

CREATE TABLE label (
    identifier integer NOT NULL,
    users integer,
    manufacturer integer,
    name character varying(300),
    space_x double precision,
    space_y double precision,
    size_x double precision,
    size_y double precision,
    offset_x double precision,
    offset_y double precision,
    columns double precision,
    "rows" double precision,
    paper integer NOT NULL,
    date_of_creation timestamp without time zone,
    last_modified_date timestamp without time zone
);


--
-- Name: label_id_sequence; Type: SEQUENCE; Schema: public; Owner: po_user
--

CREATE SEQUENCE label_id_sequence
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: label_id_sequence; Type: SEQUENCE SET; Schema: public; Owner: po_user
--

SELECT pg_catalog.setval('label_id_sequence', 134, true);


--
-- Name: lens; Type: TABLE; Schema: public; Owner: po_user; Tablespace: 
--

CREATE TABLE lens (
    identifier integer NOT NULL,
    "type" integer NOT NULL,
    users integer NOT NULL,
    serial_number character varying(20),
    date_of_purchase timestamp without time zone,
    purchased_new character(1),
    access_rights integer NOT NULL,
    CONSTRAINT lens_purchased_new_check CHECK (((purchased_new = 't'::bpchar) OR (purchased_new = 'f'::bpchar)))
);


--
-- Name: lens_id_sequence; Type: SEQUENCE; Schema: public; Owner: po_user
--

CREATE SEQUENCE lens_id_sequence
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: lens_id_sequence; Type: SEQUENCE SET; Schema: public; Owner: po_user
--

SELECT pg_catalog.setval('lens_id_sequence', 1, false);


--
-- Name: lens_type; Type: TABLE; Schema: public; Owner: po_user; Tablespace: 
--

CREATE TABLE lens_type (
    identifier integer NOT NULL,
    manufacturer integer NOT NULL,
    model character varying(100),
    variation character varying(100),
    last_modified_date timestamp without time zone,
    last_modifying_users integer,
    exiftool_lens_id text,
    min_focal_length numeric,
    max_focal_length numeric,
    min_aperture numeric,
    max_aperture numeric
);


--
-- Name: lens_type_id_sequence; Type: SEQUENCE; Schema: public; Owner: po_user
--

CREATE SEQUENCE lens_type_id_sequence
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: lens_type_id_sequence; Type: SEQUENCE SET; Schema: public; Owner: po_user
--

SELECT pg_catalog.setval('lens_type_id_sequence', 1, false);


--
-- Name: location; Type: TABLE; Schema: public; Owner: po_user; Tablespace: 
--

CREATE TABLE "location" (
    identifier integer NOT NULL,
    place character varying(300),
    state integer,
    country integer,
    last_modified_date timestamp without time zone,
    users integer,
    city character varying(300),
    latitude character varying(10),
    longitude character varying(10),
    private boolean NOT NULL
);


--
-- Name: location_id_sequence; Type: SEQUENCE; Schema: public; Owner: po_user
--

CREATE SEQUENCE location_id_sequence
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: location_id_sequence; Type: SEQUENCE SET; Schema: public; Owner: po_user
--

SELECT pg_catalog.setval('location_id_sequence', 1, false);


--
-- Name: manufacturer; Type: TABLE; Schema: public; Owner: po_user; Tablespace: 
--

CREATE TABLE manufacturer (
    identifier integer NOT NULL,
    name character varying(100),
    url character varying(100),
    last_modified_date timestamp without time zone,
    last_modifying_users integer
);


--
-- Name: manufacturer_id_sequence; Type: SEQUENCE; Schema: public; Owner: po_user
--

CREATE SEQUENCE manufacturer_id_sequence
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: manufacturer_id_sequence; Type: SEQUENCE SET; Schema: public; Owner: po_user
--

SELECT pg_catalog.setval('manufacturer_id_sequence', 13, true);


--
-- Name: orientations; Type: TABLE; Schema: public; Owner: po_user; Tablespace: 
--

CREATE TABLE orientations (
    identifier integer NOT NULL,
    name character varying(100),
    im_transform character varying(100)
);


--
-- Name: orientations_id_sequence; Type: SEQUENCE; Schema: public; Owner: po_user
--

CREATE SEQUENCE orientations_id_sequence
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: orientations_id_sequence; Type: SEQUENCE SET; Schema: public; Owner: po_user
--

SELECT pg_catalog.setval('orientations_id_sequence', 8, true);


--
-- Name: paper; Type: TABLE; Schema: public; Owner: po_user; Tablespace: 
--

CREATE TABLE paper (
    identifier integer NOT NULL,
    name character varying(300),
    size_x double precision,
    size_y double precision,
    last_modified_date timestamp without time zone,
    last_modifying_users integer
);


--
-- Name: paper_id_sequence; Type: SEQUENCE; Schema: public; Owner: po_user
--

CREATE SEQUENCE paper_id_sequence
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: paper_id_sequence; Type: SEQUENCE SET; Schema: public; Owner: po_user
--

SELECT pg_catalog.setval('paper_id_sequence', 54, true);


--
-- Name: photo; Type: TABLE; Schema: public; Owner: po_user; Tablespace: 
--

CREATE TABLE photo (
    identifier integer NOT NULL,
    users integer NOT NULL,
    folder integer NOT NULL,
    "location" integer NOT NULL,
    caption text,
    date_of_exposure timestamp without time zone,
    access_rights integer NOT NULL,
    views integer,
    copyright_statement text,
    hide_original character(1),
    author text,
    title text,
    caption_writer text,
    category character varying(3),
    credit text,
    source text,
    headline text,
    instructions text,
    transmission_reference text,
    supplemental_category text,
    web_statement text,
    date_added timestamp without time zone,
    date_changed timestamp without time zone NOT NULL,
    store_url text,
    comments text,
    CONSTRAINT photo_hide_original_check CHECK (((hide_original = 't'::bpchar) OR (hide_original = 'f'::bpchar)))
);


--
-- Name: photo_dupe; Type: TABLE; Schema: public; Owner: po_user; Tablespace: 
--

CREATE TABLE photo_dupe (
    identifier integer NOT NULL,
    "key" integer NOT NULL,
    photo integer NOT NULL,
    date_of_creation timestamp without time zone,
    storage_location character varying(50),
    storage_index character varying(10),
    remark text,
    version integer NOT NULL
);


--
-- Name: photo_dupe_id_sequence; Type: SEQUENCE; Schema: public; Owner: po_user
--

CREATE SEQUENCE photo_dupe_id_sequence
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: photo_dupe_id_sequence; Type: SEQUENCE SET; Schema: public; Owner: po_user
--

SELECT pg_catalog.setval('photo_dupe_id_sequence', 1, false);


--
-- Name: photo_id_sequence; Type: SEQUENCE; Schema: public; Owner: po_user
--

CREATE SEQUENCE photo_id_sequence
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: photo_id_sequence; Type: SEQUENCE SET; Schema: public; Owner: po_user
--

SELECT pg_catalog.setval('photo_id_sequence', 1, false);


--
-- Name: photo_keywords; Type: TABLE; Schema: public; Owner: po_user; Tablespace: 
--

CREATE TABLE photo_keywords (
    photo integer NOT NULL,
    keyword text NOT NULL
);


--
-- Name: photo_tech; Type: TABLE; Schema: public; Owner: po_user; Tablespace: 
--

CREATE TABLE photo_tech (
    identifier integer NOT NULL,
    photo integer NOT NULL,
    camera integer,
    lens integer,
    film integer,
    flash integer,
    scanner integer,
    filter integer,
    shutter integer,
    camera_metering integer,
    camera_program integer,
    flash_mode integer,
    exposure_comp integer,
    flash_comp integer,
    ev_difference integer,
    iso_override integer,
    support integer,
    scan_multiscan integer,
    exif text,
    iptc text,
    latitude numeric,
    longitude numeric,
    altitude numeric,
    img_direction numeric,
    rdf text,
    scan_bitdepth integer,
    scan_resolution integer,
    focal_length numeric,
    aperture numeric
);


--
-- Name: photo_tech_id_sequence; Type: SEQUENCE; Schema: public; Owner: po_user
--

CREATE SEQUENCE photo_tech_id_sequence
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: photo_tech_id_sequence; Type: SEQUENCE SET; Schema: public; Owner: po_user
--

SELECT pg_catalog.setval('photo_tech_id_sequence', 1, false);


--
-- Name: photo_version; Type: TABLE; Schema: public; Owner: po_user; Tablespace: 
--

CREATE TABLE photo_version (
    identifier integer NOT NULL,
    "key" integer NOT NULL,
    photo integer NOT NULL,
    master character(1) NOT NULL,
    date_of_creation timestamp without time zone NOT NULL,
    "comment" text,
    original_image_name character varying(500) NOT NULL,
    colorspace integer NOT NULL,
    orientation integer NOT NULL,
    date_changed timestamp without time zone NOT NULL,
    CONSTRAINT photo_version_master_check CHECK (((master = 't'::bpchar) OR (master = 'f'::bpchar)))
);


--
-- Name: photo_version_id_sequence; Type: SEQUENCE; Schema: public; Owner: po_user
--

CREATE SEQUENCE photo_version_id_sequence
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: photo_version_id_sequence; Type: SEQUENCE SET; Schema: public; Owner: po_user
--

SELECT pg_catalog.setval('photo_version_id_sequence', 1, false);


--
-- Name: print_id_sequence; Type: SEQUENCE; Schema: public; Owner: po_user
--

CREATE SEQUENCE print_id_sequence
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: print_id_sequence; Type: SEQUENCE SET; Schema: public; Owner: po_user
--

SELECT pg_catalog.setval('print_id_sequence', 1, false);


--
-- Name: rating; Type: TABLE; Schema: public; Owner: po_user; Tablespace: 
--

CREATE TABLE rating (
    identifier integer NOT NULL,
    photo integer NOT NULL,
    users integer NOT NULL,
    date_of_creation timestamp without time zone,
    value integer NOT NULL,
    "comment" text,
    version integer NOT NULL
);


--
-- Name: rating_id_sequence; Type: SEQUENCE; Schema: public; Owner: po_user
--

CREATE SEQUENCE rating_id_sequence
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: rating_id_sequence; Type: SEQUENCE SET; Schema: public; Owner: po_user
--

SELECT pg_catalog.setval('rating_id_sequence', 1, false);


--
-- Name: scanner; Type: TABLE; Schema: public; Owner: po_user; Tablespace: 
--

CREATE TABLE scanner (
    identifier integer NOT NULL,
    "type" integer NOT NULL,
    users integer NOT NULL,
    serial_number character varying(20),
    date_of_purchase timestamp without time zone,
    purchased_new character(1),
    access_rights integer NOT NULL,
    CONSTRAINT scanner_purchased_new_check CHECK (((purchased_new = 't'::bpchar) OR (purchased_new = 'f'::bpchar)))
);


--
-- Name: scanner_id_sequence; Type: SEQUENCE; Schema: public; Owner: po_user
--

CREATE SEQUENCE scanner_id_sequence
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: scanner_id_sequence; Type: SEQUENCE SET; Schema: public; Owner: po_user
--

SELECT pg_catalog.setval('scanner_id_sequence', 1, false);


--
-- Name: scanner_type; Type: TABLE; Schema: public; Owner: po_user; Tablespace: 
--

CREATE TABLE scanner_type (
    identifier integer NOT NULL,
    manufacturer integer NOT NULL,
    model character varying(100),
    variation character varying(100),
    last_modified_date timestamp without time zone,
    last_modifying_users integer
);


--
-- Name: scanner_type_id_sequence; Type: SEQUENCE; Schema: public; Owner: po_user
--

CREATE SEQUENCE scanner_type_id_sequence
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: scanner_type_id_sequence; Type: SEQUENCE SET; Schema: public; Owner: po_user
--

SELECT pg_catalog.setval('scanner_type_id_sequence', 1, false);


--
-- Name: shutter; Type: TABLE; Schema: public; Owner: po_user; Tablespace: 
--

CREATE TABLE shutter (
    identifier integer NOT NULL,
    value character varying(10)
);


--
-- Name: shutter_id_sequence; Type: SEQUENCE; Schema: public; Owner: po_user
--

CREATE SEQUENCE shutter_id_sequence
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: shutter_id_sequence; Type: SEQUENCE SET; Schema: public; Owner: po_user
--

SELECT pg_catalog.setval('shutter_id_sequence', 56, true);


--
-- Name: state; Type: TABLE; Schema: public; Owner: po_user; Tablespace: 
--

CREATE TABLE state (
    identifier integer NOT NULL,
    code character varying(5),
    value character varying(40)
);


--
-- Name: state_id_sequence; Type: SEQUENCE; Schema: public; Owner: po_user
--

CREATE SEQUENCE state_id_sequence
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: state_id_sequence; Type: SEQUENCE SET; Schema: public; Owner: po_user
--

SELECT pg_catalog.setval('state_id_sequence', 69, true);


--
-- Name: support; Type: TABLE; Schema: public; Owner: po_user; Tablespace: 
--

CREATE TABLE support (
    identifier integer NOT NULL,
    "type" integer NOT NULL,
    users integer NOT NULL,
    serial_number character varying(20),
    date_of_purchase timestamp without time zone,
    purchased_new character(1),
    access_rights integer NOT NULL,
    CONSTRAINT support_purchased_new_check CHECK (((purchased_new = 't'::bpchar) OR (purchased_new = 'f'::bpchar)))
);


--
-- Name: support_id_sequence; Type: SEQUENCE; Schema: public; Owner: po_user
--

CREATE SEQUENCE support_id_sequence
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: support_id_sequence; Type: SEQUENCE SET; Schema: public; Owner: po_user
--

SELECT pg_catalog.setval('support_id_sequence', 1, false);


--
-- Name: support_type; Type: TABLE; Schema: public; Owner: po_user; Tablespace: 
--

CREATE TABLE support_type (
    identifier integer NOT NULL,
    manufacturer integer NOT NULL,
    model character varying(100),
    variation character varying(100),
    last_modified_date timestamp without time zone,
    last_modifying_users integer
);


--
-- Name: support_type_id_sequence; Type: SEQUENCE; Schema: public; Owner: po_user
--

CREATE SEQUENCE support_type_id_sequence
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: support_type_id_sequence; Type: SEQUENCE SET; Schema: public; Owner: po_user
--

SELECT pg_catalog.setval('support_type_id_sequence', 1, false);


--
-- Name: user_preferences; Type: TABLE; Schema: public; Owner: po_user; Tablespace: 
--

CREATE TABLE user_preferences (
    "owner" integer NOT NULL,
    "key" character varying(100) NOT NULL,
    value text
);


--
-- Name: user_type; Type: TABLE; Schema: public; Owner: po_user; Tablespace: 
--

CREATE TABLE user_type (
    identifier integer NOT NULL,
    value character varying(15)
);


--
-- Name: user_type_id_sequence; Type: SEQUENCE; Schema: public; Owner: po_user
--

CREATE SEQUENCE user_type_id_sequence
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: user_type_id_sequence; Type: SEQUENCE SET; Schema: public; Owner: po_user
--

SELECT pg_catalog.setval('user_type_id_sequence', 4, true);


--
-- Name: users; Type: TABLE; Schema: public; Owner: po_user; Tablespace: 
--

CREATE TABLE users (
    identifier integer NOT NULL,
    first_name character varying(50),
    last_name character varying(50),
    company character varying(50),
    member_since timestamp without time zone,
    "type" integer NOT NULL,
    hide boolean DEFAULT false NOT NULL,
    username character varying(255) NOT NULL,
    "password" character varying(64),
    address1 character varying(100),
    address2 character varying(100),
    city character varying(100),
    zipcode character varying(10),
    state integer,
    country integer,
    phone character varying(40),
    email character varying(150),
    url character varying(150)
);


--
-- Name: users_id_sequence; Type: SEQUENCE; Schema: public; Owner: po_user
--

CREATE SEQUENCE users_id_sequence
    START WITH 2
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: users_id_sequence; Type: SEQUENCE SET; Schema: public; Owner: po_user
--

SELECT pg_catalog.setval('users_id_sequence', 2, false);


--
-- Name: version; Type: TABLE; Schema: public; Owner: po_user; Tablespace: 
--

CREATE TABLE version (
    identifier integer NOT NULL,
    description character varying(300),
    major smallint,
    minor smallint,
    variation character varying(1)
);


--
-- Name: view_camera; Type: VIEW; Schema: public; Owner: po_user
--

CREATE VIEW view_camera AS
    SELECT camera.identifier, manufacturer.name, camera_type.model, camera_type.variation, camera.serial_number, camera.users, camera.date_of_purchase, camera.purchased_new, manufacturer.url, access_type.value AS access_rights, camera_type.raw_icc_profile, camera.ignore_comment FROM camera_type, camera, manufacturer, access_type WHERE (((camera_type.manufacturer = manufacturer.identifier) AND (camera_type.identifier = camera."type")) AND (camera.access_rights = access_type.identifier)) ORDER BY manufacturer.name, camera_type.model, camera_type.variation;


--
-- Name: view_camera_type; Type: VIEW; Schema: public; Owner: po_user
--

CREATE VIEW view_camera_type AS
    SELECT camera_type.identifier, manufacturer.name, camera_type.model, camera_type.variation, manufacturer.url, camera_type.raw_icc_profile FROM camera_type, manufacturer WHERE (camera_type.manufacturer = manufacturer.identifier) ORDER BY manufacturer.name, camera_type.model, camera_type.variation;


--
-- Name: view_client; Type: VIEW; Schema: public; Owner: po_user
--

CREATE VIEW view_client AS
    SELECT client.identifier, client.users, client.client, users.last_name, users.first_name, client.date_of_creation, client_status.value, client.status, users."type", client."trusted" FROM client, users, client_status WHERE ((client.client = users.identifier) AND (client.status = client_status.identifier)) ORDER BY users.last_name, users.first_name;


--
-- Name: view_contact; Type: VIEW; Schema: public; Owner: po_user
--

CREATE VIEW view_contact AS
    SELECT client.identifier, client.users, client.client, users.last_name, users.first_name, client.date_of_creation, client_status.value, client.status FROM client, users, client_status WHERE ((client.client = users.identifier) AND (client.status = client_status.identifier)) ORDER BY users.last_name, users.first_name;


--
-- Name: view_contact_info; Type: VIEW; Schema: public; Owner: po_user
--

CREATE VIEW view_contact_info AS
    SELECT c.identifier, c.username, c.first_name, c.last_name, c.company, c.member_since, c."type", c.address1, c.address2, c.city, c.zipcode, (SELECT state.value FROM state WHERE (state.identifier = c.state)) AS state, (SELECT country.value FROM country WHERE (country.identifier = c.country)) AS country, c.phone, c.email, c.url FROM users c;


--
-- Name: view_film; Type: VIEW; Schema: public; Owner: po_user
--

CREATE VIEW view_film AS
    SELECT film.identifier, manufacturer.name, film_type.model, film_iso.value AS iso, film_format.value AS format, film.users, manufacturer.url, access_type.value AS access_rights FROM film_type, film, manufacturer, film_iso, film_format, access_type WHERE (((((film_type.manufacturer = manufacturer.identifier) AND (film_type.identifier = film."type")) AND (film_type.iso = film_iso.identifier)) AND (film_type.format = film_format.identifier)) AND (film.access_rights = access_type.identifier)) ORDER BY manufacturer.name, film_type.model, film_iso.value;


--
-- Name: view_film_type; Type: VIEW; Schema: public; Owner: po_user
--

CREATE VIEW view_film_type AS
    SELECT film_type.identifier, manufacturer.name, film_type.model, film_iso.value AS iso, film_format.value AS format, manufacturer.url FROM film_type, manufacturer, film_iso, film_format WHERE (((film_type.manufacturer = manufacturer.identifier) AND (film_type.iso = film_iso.identifier)) AND (film_type.format = film_format.identifier)) ORDER BY manufacturer.name, film_type.model, film_iso.value;


--
-- Name: view_filter; Type: VIEW; Schema: public; Owner: po_user
--

CREATE VIEW view_filter AS
    SELECT filter.identifier, manufacturer.name, filter_type.model, filter_type.variation, filter.users, filter.date_of_purchase, filter.purchased_new, manufacturer.url, access_type.value AS access_rights, filter.serial_number FROM filter_type, filter, manufacturer, access_type WHERE (((filter_type.manufacturer = manufacturer.identifier) AND (filter_type.identifier = filter."type")) AND (filter.access_rights = access_type.identifier)) ORDER BY manufacturer.name, filter_type.model, filter_type.variation;


--
-- Name: view_filter_type; Type: VIEW; Schema: public; Owner: po_user
--

CREATE VIEW view_filter_type AS
    SELECT filter_type.identifier, manufacturer.name, filter_type.model, filter_type.variation, manufacturer.url FROM filter_type, manufacturer WHERE (filter_type.manufacturer = manufacturer.identifier) ORDER BY manufacturer.name, filter_type.model, filter_type.variation;


--
-- Name: view_flash; Type: VIEW; Schema: public; Owner: po_user
--

CREATE VIEW view_flash AS
    SELECT flash.identifier, manufacturer.name, flash_type.model, flash_type.variation, flash.serial_number, flash.users, flash.date_of_purchase, flash.purchased_new, manufacturer.url, access_type.value AS access_rights FROM flash_type, flash, manufacturer, access_type WHERE (((flash_type.manufacturer = manufacturer.identifier) AND (flash_type.identifier = flash."type")) AND (flash.access_rights = access_type.identifier)) ORDER BY manufacturer.name, flash_type.model, flash_type.variation;


--
-- Name: view_flash_type; Type: VIEW; Schema: public; Owner: po_user
--

CREATE VIEW view_flash_type AS
    SELECT flash_type.identifier, manufacturer.name, flash_type.model, flash_type.variation, manufacturer.url FROM flash_type, manufacturer WHERE (flash_type.manufacturer = manufacturer.identifier) ORDER BY manufacturer.name, flash_type.model, flash_type.variation;


--
-- Name: view_lens; Type: VIEW; Schema: public; Owner: po_user
--

CREATE VIEW view_lens AS
    SELECT lens.identifier, manufacturer.name, lens_type.model, lens_type.variation, lens_type.min_aperture AS s_min_aperture, lens_type.max_aperture AS s_max_aperture, lens_type.min_focal_length AS s_min_focal_length, lens_type.max_focal_length AS s_max_focal_length, lens.serial_number, lens.users, lens.date_of_purchase, lens.purchased_new, manufacturer.url, access_type.value AS access_rights, lens_type.exiftool_lens_id FROM lens_type, lens, manufacturer, access_type WHERE (((lens_type.manufacturer = manufacturer.identifier) AND (lens_type.identifier = lens."type")) AND (lens.access_rights = access_type.identifier)) ORDER BY manufacturer.name, lens_type.min_focal_length, lens_type.model, lens_type.variation;


--
-- Name: view_lens_type; Type: VIEW; Schema: public; Owner: po_user
--

CREATE VIEW view_lens_type AS
    SELECT lens_type.identifier, manufacturer.name, lens_type.model, lens_type.variation, lens_type.min_aperture AS s_min_aperture, lens_type.max_aperture AS s_max_aperture, lens_type.min_focal_length AS s_min_focal_length, lens_type.max_focal_length AS s_max_focal_length, manufacturer.url, lens_type.exiftool_lens_id FROM lens_type, manufacturer WHERE (lens_type.manufacturer = manufacturer.identifier) ORDER BY manufacturer.name, lens_type.min_focal_length, lens_type.model, lens_type.variation;


--
-- Name: view_location; Type: VIEW; Schema: public; Owner: po_user
--

CREATE VIEW view_location AS
    SELECT "location".identifier, "location".place, "location".city, (SELECT state.value FROM state WHERE (state.identifier = "location".state)) AS state, (SELECT country.value FROM country WHERE (country.identifier = "location".country)) AS country, "location".users, "location".last_modified_date, "location".latitude, "location".longitude, "location".private FROM "location";


--
-- Name: view_rating; Type: VIEW; Schema: public; Owner: po_user
--

CREATE VIEW view_rating AS
    SELECT rating.identifier, rating.photo, rating.users, rating.date_of_creation, rating.value, users.first_name, users.last_name, rating."comment", rating.version FROM rating, users WHERE (rating.users = users.identifier);


--
-- Name: view_scanner; Type: VIEW; Schema: public; Owner: po_user
--

CREATE VIEW view_scanner AS
    SELECT scanner.identifier, manufacturer.name, scanner_type.model, scanner_type.variation, scanner.serial_number, scanner.users, scanner.date_of_purchase, scanner.purchased_new, manufacturer.url, access_type.value AS access_rights FROM scanner_type, scanner, manufacturer, access_type WHERE (((scanner_type.manufacturer = manufacturer.identifier) AND (scanner_type.identifier = scanner."type")) AND (scanner.access_rights = access_type.identifier)) ORDER BY manufacturer.name, scanner_type.model, scanner_type.variation;


--
-- Name: view_scanner_type; Type: VIEW; Schema: public; Owner: po_user
--

CREATE VIEW view_scanner_type AS
    SELECT scanner_type.identifier, manufacturer.name, scanner_type.model, scanner_type.variation, manufacturer.url FROM scanner_type, manufacturer WHERE (scanner_type.manufacturer = manufacturer.identifier) ORDER BY manufacturer.name, scanner_type.model, scanner_type.variation;


--
-- Name: view_support; Type: VIEW; Schema: public; Owner: po_user
--

CREATE VIEW view_support AS
    SELECT support.identifier, manufacturer.name, support_type.model, support_type.variation, support.serial_number, support.users, support.date_of_purchase, support.purchased_new, manufacturer.url, access_type.value AS access_rights FROM support_type, support, manufacturer, access_type WHERE (((support_type.manufacturer = manufacturer.identifier) AND (support_type.identifier = support."type")) AND (support.access_rights = access_type.identifier)) ORDER BY manufacturer.name, support_type.model, support_type.variation;


--
-- Name: view_support_type; Type: VIEW; Schema: public; Owner: po_user
--

CREATE VIEW view_support_type AS
    SELECT support_type.identifier, manufacturer.name, support_type.model, support_type.variation, manufacturer.url FROM support_type, manufacturer WHERE (support_type.manufacturer = manufacturer.identifier) ORDER BY manufacturer.name, support_type.model, support_type.variation;


--
-- Name: view_users_subscribed_to; Type: VIEW; Schema: public; Owner: po_user
--

CREATE VIEW view_users_subscribed_to AS
    SELECT client.identifier, client.users, client.client, users.last_name, users.first_name, client.date_of_creation, client_status.value, client.status FROM client, users, client_status WHERE ((client.users = users.identifier) AND (client.status = client_status.identifier)) ORDER BY users.last_name, users.first_name;


--
-- Data for Name: access_type; Type: TABLE DATA; Schema: public; Owner: po_user
--

INSERT INTO access_type (identifier, value) VALUES (1, 'Public');
INSERT INTO access_type (identifier, value) VALUES (2, 'Protected');
INSERT INTO access_type (identifier, value) VALUES (3, 'Private');


--
-- Data for Name: album; Type: TABLE DATA; Schema: public; Owner: po_user
--

INSERT INTO album (identifier, users, caption, date_of_creation, access_rights, description, parent_album, "password", date_changed, event) VALUES (1, 1, 'Spool', now(), 3, NULL, NULL, NULL, now(), NULL);


--
-- Data for Name: album_content; Type: TABLE DATA; Schema: public; Owner: po_user
--



--
-- Data for Name: calendar; Type: TABLE DATA; Schema: public; Owner: po_user
--



--
-- Data for Name: camera; Type: TABLE DATA; Schema: public; Owner: po_user
--



--
-- Data for Name: camera_metering; Type: TABLE DATA; Schema: public; Owner: po_user
--

INSERT INTO camera_metering (identifier, value) VALUES (1, 'Spot');
INSERT INTO camera_metering (identifier, value) VALUES (2, 'Center');
INSERT INTO camera_metering (identifier, value) VALUES (3, '3D Matrix');
INSERT INTO camera_metering (identifier, value) VALUES (4, '3D Color Matrix');
INSERT INTO camera_metering (identifier, value) VALUES (5, 'Zone');


--
-- Data for Name: camera_program; Type: TABLE DATA; Schema: public; Owner: po_user
--

INSERT INTO camera_program (identifier, value) VALUES (1, 'Camera Program Automatic');
INSERT INTO camera_program (identifier, value) VALUES (2, 'Aperture Priority');
INSERT INTO camera_program (identifier, value) VALUES (3, 'Shutter Priority');
INSERT INTO camera_program (identifier, value) VALUES (4, 'Manual');


--
-- Data for Name: camera_type; Type: TABLE DATA; Schema: public; Owner: po_user
--



--
-- Data for Name: client; Type: TABLE DATA; Schema: public; Owner: po_user
--



--
-- Data for Name: client_status; Type: TABLE DATA; Schema: public; Owner: po_user
--

INSERT INTO client_status (identifier, value) VALUES (1, 'Pending');
INSERT INTO client_status (identifier, value) VALUES (2, 'Accepted');
INSERT INTO client_status (identifier, value) VALUES (3, 'Rejected');


--
-- Data for Name: country; Type: TABLE DATA; Schema: public; Owner: po_user
--

INSERT INTO country (identifier, code, value) VALUES (1, 'ad', 'Andorra');
INSERT INTO country (identifier, code, value) VALUES (2, 'ae', 'United Arab Emirates');
INSERT INTO country (identifier, code, value) VALUES (3, 'af', 'Afghanistan');
INSERT INTO country (identifier, code, value) VALUES (4, 'ag', 'Antigua and Barbuda');
INSERT INTO country (identifier, code, value) VALUES (5, 'ai', 'Anguilla');
INSERT INTO country (identifier, code, value) VALUES (6, 'al', 'Albania');
INSERT INTO country (identifier, code, value) VALUES (7, 'am', 'Armenia');
INSERT INTO country (identifier, code, value) VALUES (8, 'an', 'Netherlands Antilles');
INSERT INTO country (identifier, code, value) VALUES (9, 'ao', 'Angola');
INSERT INTO country (identifier, code, value) VALUES (10, 'aq', 'Antarctica');
INSERT INTO country (identifier, code, value) VALUES (11, 'ar', 'Argentina');
INSERT INTO country (identifier, code, value) VALUES (12, 'as', 'American Samoa');
INSERT INTO country (identifier, code, value) VALUES (13, 'at', 'Austria');
INSERT INTO country (identifier, code, value) VALUES (14, 'au', 'Australia');
INSERT INTO country (identifier, code, value) VALUES (15, 'aw', 'Aruba');
INSERT INTO country (identifier, code, value) VALUES (16, 'az', 'Azerbaijan');
INSERT INTO country (identifier, code, value) VALUES (17, 'ba', 'Bosnia and Herzegovina');
INSERT INTO country (identifier, code, value) VALUES (18, 'bb', 'Barbados');
INSERT INTO country (identifier, code, value) VALUES (19, 'bd', 'Bangladesh');
INSERT INTO country (identifier, code, value) VALUES (20, 'be', 'Belgium');
INSERT INTO country (identifier, code, value) VALUES (21, 'bf', 'Burkina Faso');
INSERT INTO country (identifier, code, value) VALUES (22, 'bg', 'Bulgaria');
INSERT INTO country (identifier, code, value) VALUES (23, 'bh', 'Bahrain');
INSERT INTO country (identifier, code, value) VALUES (24, 'bi', 'Burundi');
INSERT INTO country (identifier, code, value) VALUES (25, 'bj', 'Benin');
INSERT INTO country (identifier, code, value) VALUES (26, 'bm', 'Bermuda');
INSERT INTO country (identifier, code, value) VALUES (27, 'bn', 'Brunei Darussalam');
INSERT INTO country (identifier, code, value) VALUES (28, 'bo', 'Bolivia');
INSERT INTO country (identifier, code, value) VALUES (29, 'br', 'Brazil');
INSERT INTO country (identifier, code, value) VALUES (30, 'bs', 'Bahamas');
INSERT INTO country (identifier, code, value) VALUES (31, 'bt', 'Bhutan');
INSERT INTO country (identifier, code, value) VALUES (32, 'bv', 'Bouvet Island');
INSERT INTO country (identifier, code, value) VALUES (33, 'bw', 'Botswana');
INSERT INTO country (identifier, code, value) VALUES (34, 'by', 'Belarus');
INSERT INTO country (identifier, code, value) VALUES (35, 'bz', 'Belize');
INSERT INTO country (identifier, code, value) VALUES (36, 'ca', 'Canada');
INSERT INTO country (identifier, code, value) VALUES (37, 'cc', 'Cocos (Keeling) Islands');
INSERT INTO country (identifier, code, value) VALUES (38, 'cf', 'Central African Republic');
INSERT INTO country (identifier, code, value) VALUES (39, 'cg', 'Congo');
INSERT INTO country (identifier, code, value) VALUES (40, 'ch', 'Switzerland');
INSERT INTO country (identifier, code, value) VALUES (41, 'ck', 'Cook Islands');
INSERT INTO country (identifier, code, value) VALUES (42, 'cl', 'Chile');
INSERT INTO country (identifier, code, value) VALUES (43, 'cm', 'Cameroon');
INSERT INTO country (identifier, code, value) VALUES (44, 'cn', 'China');
INSERT INTO country (identifier, code, value) VALUES (45, 'co', 'Colombia');
INSERT INTO country (identifier, code, value) VALUES (46, 'cr', 'Costa Rica');
INSERT INTO country (identifier, code, value) VALUES (47, 'cu', 'Cuba');
INSERT INTO country (identifier, code, value) VALUES (48, 'cv', 'Cape Verde');
INSERT INTO country (identifier, code, value) VALUES (49, 'cx', 'Christmas Island');
INSERT INTO country (identifier, code, value) VALUES (50, 'cy', 'Cyprus');
INSERT INTO country (identifier, code, value) VALUES (51, 'cz', 'Czech Republic');
INSERT INTO country (identifier, code, value) VALUES (52, 'de', 'Germany');
INSERT INTO country (identifier, code, value) VALUES (53, 'dj', 'Djibouti');
INSERT INTO country (identifier, code, value) VALUES (54, 'dk', 'Denmark');
INSERT INTO country (identifier, code, value) VALUES (55, 'dm', 'Dominica');
INSERT INTO country (identifier, code, value) VALUES (56, 'do', 'Dominican Republic');
INSERT INTO country (identifier, code, value) VALUES (57, 'dz', 'Algeria');
INSERT INTO country (identifier, code, value) VALUES (58, 'ec', 'Ecuador');
INSERT INTO country (identifier, code, value) VALUES (59, 'ee', 'Estonia');
INSERT INTO country (identifier, code, value) VALUES (60, 'eg', 'Egypt');
INSERT INTO country (identifier, code, value) VALUES (61, 'eh', 'Western Sahara');
INSERT INTO country (identifier, code, value) VALUES (62, 'er', 'Eritrea');
INSERT INTO country (identifier, code, value) VALUES (63, 'es', 'Spain');
INSERT INTO country (identifier, code, value) VALUES (64, 'et', 'Ethiopia');
INSERT INTO country (identifier, code, value) VALUES (65, 'fi', 'Finland');
INSERT INTO country (identifier, code, value) VALUES (66, 'fj', 'Fiji');
INSERT INTO country (identifier, code, value) VALUES (67, 'fk', 'Falkland Islands (Malvinas)');
INSERT INTO country (identifier, code, value) VALUES (68, 'fm', 'Micronesia');
INSERT INTO country (identifier, code, value) VALUES (69, 'fo', 'Faroe Islands');
INSERT INTO country (identifier, code, value) VALUES (70, 'fr', 'France');
INSERT INTO country (identifier, code, value) VALUES (71, 'fx', 'France Metropolitan');
INSERT INTO country (identifier, code, value) VALUES (72, 'ga', 'Gabon');
INSERT INTO country (identifier, code, value) VALUES (73, 'gb', 'Great Britain');
INSERT INTO country (identifier, code, value) VALUES (74, 'gd', 'Grenada');
INSERT INTO country (identifier, code, value) VALUES (75, 'ge', 'Georgia');
INSERT INTO country (identifier, code, value) VALUES (76, 'gf', 'French Guiana');
INSERT INTO country (identifier, code, value) VALUES (77, 'gh', 'Ghana');
INSERT INTO country (identifier, code, value) VALUES (78, 'gi', 'Gibraltar');
INSERT INTO country (identifier, code, value) VALUES (79, 'gl', 'Greenland');
INSERT INTO country (identifier, code, value) VALUES (80, 'gm', 'Gambia');
INSERT INTO country (identifier, code, value) VALUES (81, 'gn', 'Guinea');
INSERT INTO country (identifier, code, value) VALUES (82, 'gp', 'Guadeloupe');
INSERT INTO country (identifier, code, value) VALUES (83, 'gq', 'Equatorial Guinea');
INSERT INTO country (identifier, code, value) VALUES (84, 'gr', 'Greece');
INSERT INTO country (identifier, code, value) VALUES (85, 'gs', 'S. Georgia and S. Sandwich Isls.');
INSERT INTO country (identifier, code, value) VALUES (86, 'gt', 'Guatemala');
INSERT INTO country (identifier, code, value) VALUES (87, 'gu', 'Guam');
INSERT INTO country (identifier, code, value) VALUES (88, 'gw', 'Guinea-Bissau');
INSERT INTO country (identifier, code, value) VALUES (89, 'gy', 'Guyana');
INSERT INTO country (identifier, code, value) VALUES (90, 'hk', 'Hong Kong');
INSERT INTO country (identifier, code, value) VALUES (91, 'hm', 'Heard and McDonald Islands');
INSERT INTO country (identifier, code, value) VALUES (92, 'hn', 'Honduras');
INSERT INTO country (identifier, code, value) VALUES (93, 'hr', 'Croatia (Hrvatska)');
INSERT INTO country (identifier, code, value) VALUES (94, 'ht', 'Haiti');
INSERT INTO country (identifier, code, value) VALUES (95, 'hu', 'Hungary');
INSERT INTO country (identifier, code, value) VALUES (96, 'id', 'Indonesia');
INSERT INTO country (identifier, code, value) VALUES (97, 'ie', 'Ireland');
INSERT INTO country (identifier, code, value) VALUES (98, 'il', 'Israel');
INSERT INTO country (identifier, code, value) VALUES (99, 'in', 'India');
INSERT INTO country (identifier, code, value) VALUES (100, 'io', 'British Indian Ocean Territory');
INSERT INTO country (identifier, code, value) VALUES (101, 'iq', 'Iraq');
INSERT INTO country (identifier, code, value) VALUES (102, 'ir', 'Iran');
INSERT INTO country (identifier, code, value) VALUES (103, 'is', 'Iceland');
INSERT INTO country (identifier, code, value) VALUES (104, 'it', 'Italy');
INSERT INTO country (identifier, code, value) VALUES (105, 'jm', 'Jamaica');
INSERT INTO country (identifier, code, value) VALUES (106, 'jo', 'Jordan');
INSERT INTO country (identifier, code, value) VALUES (107, 'jp', 'Japan');
INSERT INTO country (identifier, code, value) VALUES (108, 'ke', 'Kenya');
INSERT INTO country (identifier, code, value) VALUES (109, 'kg', 'Kyrgyzstan');
INSERT INTO country (identifier, code, value) VALUES (110, 'kh', 'Cambodia');
INSERT INTO country (identifier, code, value) VALUES (111, 'ki', 'Kiribati');
INSERT INTO country (identifier, code, value) VALUES (112, 'km', 'Comoros');
INSERT INTO country (identifier, code, value) VALUES (113, 'kn', 'Saint Kitts and Nevis');
INSERT INTO country (identifier, code, value) VALUES (114, 'kp', 'Korea (North)');
INSERT INTO country (identifier, code, value) VALUES (115, 'kr', 'Korea (South)');
INSERT INTO country (identifier, code, value) VALUES (116, 'kw', 'Kuwait');
INSERT INTO country (identifier, code, value) VALUES (117, 'ky', 'Cayman Islands');
INSERT INTO country (identifier, code, value) VALUES (118, 'kz', 'Kazakhstan');
INSERT INTO country (identifier, code, value) VALUES (119, 'la', 'Laos');
INSERT INTO country (identifier, code, value) VALUES (120, 'lb', 'Lebanon');
INSERT INTO country (identifier, code, value) VALUES (121, 'lc', 'Saint Lucia');
INSERT INTO country (identifier, code, value) VALUES (122, 'li', 'Liechtenstein');
INSERT INTO country (identifier, code, value) VALUES (123, 'lk', 'Sri Lanka');
INSERT INTO country (identifier, code, value) VALUES (124, 'lr', 'Liberia');
INSERT INTO country (identifier, code, value) VALUES (125, 'ls', 'Lesotho');
INSERT INTO country (identifier, code, value) VALUES (126, 'lt', 'Lithuania');
INSERT INTO country (identifier, code, value) VALUES (127, 'lu', 'Luxembourg');
INSERT INTO country (identifier, code, value) VALUES (128, 'lv', 'Latvia');
INSERT INTO country (identifier, code, value) VALUES (129, 'ly', 'Libya');
INSERT INTO country (identifier, code, value) VALUES (130, 'ma', 'Morocco');
INSERT INTO country (identifier, code, value) VALUES (131, 'mc', 'Monaco');
INSERT INTO country (identifier, code, value) VALUES (132, 'md', 'Moldova');
INSERT INTO country (identifier, code, value) VALUES (133, 'mg', 'Madagascar');
INSERT INTO country (identifier, code, value) VALUES (134, 'mh', 'Marshall Islands');
INSERT INTO country (identifier, code, value) VALUES (135, 'mk', 'Macedonia');
INSERT INTO country (identifier, code, value) VALUES (136, 'ml', 'Mali');
INSERT INTO country (identifier, code, value) VALUES (137, 'mm', 'Myanmar');
INSERT INTO country (identifier, code, value) VALUES (138, 'mn', 'Mongolia');
INSERT INTO country (identifier, code, value) VALUES (139, 'mo', 'Macau');
INSERT INTO country (identifier, code, value) VALUES (140, 'mp', 'Northern Mariana Islands');
INSERT INTO country (identifier, code, value) VALUES (141, 'mq', 'Martinique');
INSERT INTO country (identifier, code, value) VALUES (142, 'mr', 'Mauritania');
INSERT INTO country (identifier, code, value) VALUES (143, 'ms', 'Montserrat');
INSERT INTO country (identifier, code, value) VALUES (144, 'mt', 'Malta');
INSERT INTO country (identifier, code, value) VALUES (145, 'mu', 'Mauritius');
INSERT INTO country (identifier, code, value) VALUES (146, 'mv', 'Maldives');
INSERT INTO country (identifier, code, value) VALUES (147, 'mw', 'Malawi');
INSERT INTO country (identifier, code, value) VALUES (148, 'mx', 'Mexico');
INSERT INTO country (identifier, code, value) VALUES (149, 'my', 'Malaysia');
INSERT INTO country (identifier, code, value) VALUES (150, 'mz', 'Mozambique');
INSERT INTO country (identifier, code, value) VALUES (151, 'na', 'Namibia');
INSERT INTO country (identifier, code, value) VALUES (152, 'nc', 'New Caledonia');
INSERT INTO country (identifier, code, value) VALUES (153, 'ne', 'Niger');
INSERT INTO country (identifier, code, value) VALUES (154, 'nf', 'Norfolk Island');
INSERT INTO country (identifier, code, value) VALUES (155, 'ng', 'Nigeria');
INSERT INTO country (identifier, code, value) VALUES (156, 'ni', 'Nicaragua');
INSERT INTO country (identifier, code, value) VALUES (157, 'nl', 'Netherlands');
INSERT INTO country (identifier, code, value) VALUES (158, 'no', 'Norway');
INSERT INTO country (identifier, code, value) VALUES (159, 'np', 'Nepal');
INSERT INTO country (identifier, code, value) VALUES (160, 'nr', 'Nauru');
INSERT INTO country (identifier, code, value) VALUES (161, 'nt', 'Neutral Zone');
INSERT INTO country (identifier, code, value) VALUES (162, 'nu', 'Niue');
INSERT INTO country (identifier, code, value) VALUES (163, 'nz', 'New Zealand (Aotearoa)');
INSERT INTO country (identifier, code, value) VALUES (164, 'om', 'Oman');
INSERT INTO country (identifier, code, value) VALUES (165, 'pa', 'Panama');
INSERT INTO country (identifier, code, value) VALUES (166, 'pe', 'Peru');
INSERT INTO country (identifier, code, value) VALUES (167, 'pf', 'French Polynesia');
INSERT INTO country (identifier, code, value) VALUES (168, 'pg', 'Papua New Guinea');
INSERT INTO country (identifier, code, value) VALUES (169, 'ph', 'Philippines');
INSERT INTO country (identifier, code, value) VALUES (170, 'pk', 'Pakistan');
INSERT INTO country (identifier, code, value) VALUES (171, 'pl', 'Poland');
INSERT INTO country (identifier, code, value) VALUES (172, 'pm', 'St. Pierre and Miquelon');
INSERT INTO country (identifier, code, value) VALUES (173, 'pn', 'Pitcairn');
INSERT INTO country (identifier, code, value) VALUES (174, 'pr', 'Puerto Rico');
INSERT INTO country (identifier, code, value) VALUES (175, 'pt', 'Portugal');
INSERT INTO country (identifier, code, value) VALUES (176, 'pw', 'Palau');
INSERT INTO country (identifier, code, value) VALUES (177, 'py', 'Paraguay');
INSERT INTO country (identifier, code, value) VALUES (178, 'qa', 'Qatar');
INSERT INTO country (identifier, code, value) VALUES (179, 're', 'Reunion');
INSERT INTO country (identifier, code, value) VALUES (180, 'ro', 'Romania');
INSERT INTO country (identifier, code, value) VALUES (181, 'ru', 'Russian Federation');
INSERT INTO country (identifier, code, value) VALUES (182, 'rw', 'Rwanda');
INSERT INTO country (identifier, code, value) VALUES (183, 'sa', 'Saudi Arabia');
INSERT INTO country (identifier, code, value) VALUES (184, 'sb', 'Solomon Islands');
INSERT INTO country (identifier, code, value) VALUES (185, 'sc', 'Seychelles');
INSERT INTO country (identifier, code, value) VALUES (186, 'sd', 'Sudan');
INSERT INTO country (identifier, code, value) VALUES (187, 'se', 'Sweden');
INSERT INTO country (identifier, code, value) VALUES (188, 'sg', 'Singapore');
INSERT INTO country (identifier, code, value) VALUES (189, 'sh', 'St. Helena');
INSERT INTO country (identifier, code, value) VALUES (190, 'si', 'Slovenia');
INSERT INTO country (identifier, code, value) VALUES (191, 'sj', 'Svalbard and Jan Mayen Islands');
INSERT INTO country (identifier, code, value) VALUES (192, 'sk', 'Slovak Republic');
INSERT INTO country (identifier, code, value) VALUES (193, 'sl', 'Sierra Leone');
INSERT INTO country (identifier, code, value) VALUES (194, 'sm', 'San Marino');
INSERT INTO country (identifier, code, value) VALUES (195, 'sn', 'Senegal');
INSERT INTO country (identifier, code, value) VALUES (196, 'so', 'Somalia');
INSERT INTO country (identifier, code, value) VALUES (197, 'sr', 'Suriname');
INSERT INTO country (identifier, code, value) VALUES (198, 'st', 'Sao Tome and Principe');
INSERT INTO country (identifier, code, value) VALUES (199, 'sv', 'El Salvador');
INSERT INTO country (identifier, code, value) VALUES (200, 'sy', 'Syria');
INSERT INTO country (identifier, code, value) VALUES (201, 'sz', 'Swaziland');
INSERT INTO country (identifier, code, value) VALUES (202, 'tc', 'Turks and Caicos Islands');
INSERT INTO country (identifier, code, value) VALUES (203, 'td', 'Chad');
INSERT INTO country (identifier, code, value) VALUES (204, 'tf', 'French Southern Territories');
INSERT INTO country (identifier, code, value) VALUES (205, 'tg', 'Togo');
INSERT INTO country (identifier, code, value) VALUES (206, 'th', 'Thailand');
INSERT INTO country (identifier, code, value) VALUES (207, 'tj', 'Tajikistan');
INSERT INTO country (identifier, code, value) VALUES (208, 'tk', 'Tokelau');
INSERT INTO country (identifier, code, value) VALUES (209, 'tm', 'Turkmenistan');
INSERT INTO country (identifier, code, value) VALUES (210, 'tn', 'Tunisia');
INSERT INTO country (identifier, code, value) VALUES (211, 'to', 'Tonga');
INSERT INTO country (identifier, code, value) VALUES (212, 'tp', 'East Timor');
INSERT INTO country (identifier, code, value) VALUES (213, 'tr', 'Turkey');
INSERT INTO country (identifier, code, value) VALUES (214, 'tt', 'Trinidad and Tobago');
INSERT INTO country (identifier, code, value) VALUES (215, 'tv', 'Tuvalu');
INSERT INTO country (identifier, code, value) VALUES (216, 'tw', 'Taiwan');
INSERT INTO country (identifier, code, value) VALUES (217, 'tz', 'Tanzania');
INSERT INTO country (identifier, code, value) VALUES (218, 'ua', 'Ukraine');
INSERT INTO country (identifier, code, value) VALUES (219, 'ug', 'Uganda');
INSERT INTO country (identifier, code, value) VALUES (220, 'uk', 'United Kingdom');
INSERT INTO country (identifier, code, value) VALUES (221, 'um', 'US Minor Outlying Islands');
INSERT INTO country (identifier, code, value) VALUES (222, 'us', 'United States');
INSERT INTO country (identifier, code, value) VALUES (223, 'uy', 'Uruguay');
INSERT INTO country (identifier, code, value) VALUES (224, 'uz', 'Uzbekistan');
INSERT INTO country (identifier, code, value) VALUES (225, 'va', 'Vatican City State (Holy See)');
INSERT INTO country (identifier, code, value) VALUES (226, 'vc', 'Saint Vincent and the Grenadines');
INSERT INTO country (identifier, code, value) VALUES (227, 've', 'Venezuela');
INSERT INTO country (identifier, code, value) VALUES (228, 'vg', 'Virgin Islands (British)');
INSERT INTO country (identifier, code, value) VALUES (229, 'vi', 'Virgin Islands (U.S.)');
INSERT INTO country (identifier, code, value) VALUES (230, 'vn', 'Viet Nam');
INSERT INTO country (identifier, code, value) VALUES (231, 'vu', 'Vanuatu');
INSERT INTO country (identifier, code, value) VALUES (232, 'wf', 'Wallis and Futuna Islands');
INSERT INTO country (identifier, code, value) VALUES (233, 'ws', 'Samoa');
INSERT INTO country (identifier, code, value) VALUES (234, 'ye', 'Yemen');
INSERT INTO country (identifier, code, value) VALUES (235, 'yt', 'Mayotte');
INSERT INTO country (identifier, code, value) VALUES (236, 'yu', 'Yugoslavia');
INSERT INTO country (identifier, code, value) VALUES (237, 'za', 'South Africa');
INSERT INTO country (identifier, code, value) VALUES (238, 'zm', 'Zambia');
INSERT INTO country (identifier, code, value) VALUES (239, 'zr', 'Zaire');
INSERT INTO country (identifier, code, value) VALUES (240, 'zw', 'Zimbabwe');


--
-- Data for Name: ev_comp; Type: TABLE DATA; Schema: public; Owner: po_user
--

INSERT INTO ev_comp (identifier, value) VALUES (1, '-5');
INSERT INTO ev_comp (identifier, value) VALUES (2, '-4.7');
INSERT INTO ev_comp (identifier, value) VALUES (3, '-4.5');
INSERT INTO ev_comp (identifier, value) VALUES (4, '-4.3');
INSERT INTO ev_comp (identifier, value) VALUES (5, '-4');
INSERT INTO ev_comp (identifier, value) VALUES (6, '-3.7');
INSERT INTO ev_comp (identifier, value) VALUES (7, '-3.5');
INSERT INTO ev_comp (identifier, value) VALUES (8, '-3.3');
INSERT INTO ev_comp (identifier, value) VALUES (9, '-3');
INSERT INTO ev_comp (identifier, value) VALUES (10, '-2.7');
INSERT INTO ev_comp (identifier, value) VALUES (11, '-2.5');
INSERT INTO ev_comp (identifier, value) VALUES (12, '-2.3');
INSERT INTO ev_comp (identifier, value) VALUES (13, '-2');
INSERT INTO ev_comp (identifier, value) VALUES (14, '-1.7');
INSERT INTO ev_comp (identifier, value) VALUES (15, '-1.5');
INSERT INTO ev_comp (identifier, value) VALUES (16, '-1.3');
INSERT INTO ev_comp (identifier, value) VALUES (17, '-1');
INSERT INTO ev_comp (identifier, value) VALUES (18, '-0.7');
INSERT INTO ev_comp (identifier, value) VALUES (19, '-0.5');
INSERT INTO ev_comp (identifier, value) VALUES (20, '-0.3');
INSERT INTO ev_comp (identifier, value) VALUES (21, '0');
INSERT INTO ev_comp (identifier, value) VALUES (22, '0.3');
INSERT INTO ev_comp (identifier, value) VALUES (23, '0.5');
INSERT INTO ev_comp (identifier, value) VALUES (24, '0.7');
INSERT INTO ev_comp (identifier, value) VALUES (25, '1');
INSERT INTO ev_comp (identifier, value) VALUES (26, '1.3');
INSERT INTO ev_comp (identifier, value) VALUES (27, '1.5');
INSERT INTO ev_comp (identifier, value) VALUES (28, '1.7');
INSERT INTO ev_comp (identifier, value) VALUES (29, '2');
INSERT INTO ev_comp (identifier, value) VALUES (30, '2.3');
INSERT INTO ev_comp (identifier, value) VALUES (31, '2.5');
INSERT INTO ev_comp (identifier, value) VALUES (32, '2.7');
INSERT INTO ev_comp (identifier, value) VALUES (33, '3');
INSERT INTO ev_comp (identifier, value) VALUES (34, '3.3');
INSERT INTO ev_comp (identifier, value) VALUES (35, '3.5');
INSERT INTO ev_comp (identifier, value) VALUES (36, '3.7');
INSERT INTO ev_comp (identifier, value) VALUES (37, '4');
INSERT INTO ev_comp (identifier, value) VALUES (38, '4.3');
INSERT INTO ev_comp (identifier, value) VALUES (39, '4.5');
INSERT INTO ev_comp (identifier, value) VALUES (40, '4.7');
INSERT INTO ev_comp (identifier, value) VALUES (41, '5');


--
-- Data for Name: files; Type: TABLE DATA; Schema: public; Owner: po_user
--



--
-- Data for Name: film; Type: TABLE DATA; Schema: public; Owner: po_user
--



--
-- Data for Name: film_format; Type: TABLE DATA; Schema: public; Owner: po_user
--

INSERT INTO film_format (identifier, value) VALUES (3, '24x65 mm');
INSERT INTO film_format (identifier, value) VALUES (4, '40x40 mm');
INSERT INTO film_format (identifier, value) VALUES (6, '60x60 mm');
INSERT INTO film_format (identifier, value) VALUES (7, '60x70 mm');
INSERT INTO film_format (identifier, value) VALUES (8, '4x5 inch');
INSERT INTO film_format (identifier, value) VALUES (9, '8x10 inch');
INSERT INTO film_format (identifier, value) VALUES (10, '20x24 inch');
INSERT INTO film_format (identifier, value) VALUES (11, 'Digital');
INSERT INTO film_format (identifier, value) VALUES (2, '35 mm');
INSERT INTO film_format (identifier, value) VALUES (5, '60x45 mm');
INSERT INTO film_format (identifier, value) VALUES (1, 'APS-C');
INSERT INTO film_format (identifier, value) VALUES (12, '110-C');
INSERT INTO film_format (identifier, value) VALUES (13, '126-C');
INSERT INTO film_format (identifier, value) VALUES (14, '60x80 mm');
INSERT INTO film_format (identifier, value) VALUES (15, '60x90 mm');
INSERT INTO film_format (identifier, value) VALUES (16, '60x120 mm');
INSERT INTO film_format (identifier, value) VALUES (17, '60x170 mm');
INSERT INTO film_format (identifier, value) VALUES (18, '60x240 mm');
INSERT INTO film_format (identifier, value) VALUES (19, '65x90 mm');
INSERT INTO film_format (identifier, value) VALUES (20, '90x120 mm');
INSERT INTO film_format (identifier, value) VALUES (21, '100x150 mm');
INSERT INTO film_format (identifier, value) VALUES (22, '130x180 mm');
INSERT INTO film_format (identifier, value) VALUES (23, '180x240 mm');
INSERT INTO film_format (identifier, value) VALUES (24, '240x300 mm');
INSERT INTO film_format (identifier, value) VALUES (25, '5x7 inch');
INSERT INTO film_format (identifier, value) VALUES (26, '11x14 inch');


--
-- Data for Name: film_iso; Type: TABLE DATA; Schema: public; Owner: po_user
--

INSERT INTO film_iso (identifier, value) VALUES (1, 6);
INSERT INTO film_iso (identifier, value) VALUES (2, 8);
INSERT INTO film_iso (identifier, value) VALUES (3, 10);
INSERT INTO film_iso (identifier, value) VALUES (4, 12);
INSERT INTO film_iso (identifier, value) VALUES (5, 16);
INSERT INTO film_iso (identifier, value) VALUES (6, 20);
INSERT INTO film_iso (identifier, value) VALUES (7, 25);
INSERT INTO film_iso (identifier, value) VALUES (8, 32);
INSERT INTO film_iso (identifier, value) VALUES (9, 40);
INSERT INTO film_iso (identifier, value) VALUES (10, 50);
INSERT INTO film_iso (identifier, value) VALUES (11, 80);
INSERT INTO film_iso (identifier, value) VALUES (12, 100);
INSERT INTO film_iso (identifier, value) VALUES (13, 125);
INSERT INTO film_iso (identifier, value) VALUES (14, 160);
INSERT INTO film_iso (identifier, value) VALUES (15, 200);
INSERT INTO film_iso (identifier, value) VALUES (16, 250);
INSERT INTO film_iso (identifier, value) VALUES (17, 320);
INSERT INTO film_iso (identifier, value) VALUES (18, 400);
INSERT INTO film_iso (identifier, value) VALUES (19, 500);
INSERT INTO film_iso (identifier, value) VALUES (20, 640);
INSERT INTO film_iso (identifier, value) VALUES (21, 800);
INSERT INTO film_iso (identifier, value) VALUES (22, 1000);
INSERT INTO film_iso (identifier, value) VALUES (23, 1250);
INSERT INTO film_iso (identifier, value) VALUES (24, 1600);
INSERT INTO film_iso (identifier, value) VALUES (25, 2000);
INSERT INTO film_iso (identifier, value) VALUES (26, 2500);
INSERT INTO film_iso (identifier, value) VALUES (27, 3200);
INSERT INTO film_iso (identifier, value) VALUES (28, 4000);
INSERT INTO film_iso (identifier, value) VALUES (29, 5000);
INSERT INTO film_iso (identifier, value) VALUES (30, 6400);
INSERT INTO film_iso (identifier, value) VALUES (31, 64);


--
-- Data for Name: film_type; Type: TABLE DATA; Schema: public; Owner: po_user
--



--
-- Data for Name: filter; Type: TABLE DATA; Schema: public; Owner: po_user
--



--
-- Data for Name: filter_type; Type: TABLE DATA; Schema: public; Owner: po_user
--



--
-- Data for Name: flash; Type: TABLE DATA; Schema: public; Owner: po_user
--



--
-- Data for Name: flash_mode; Type: TABLE DATA; Schema: public; Owner: po_user
--

INSERT INTO flash_mode (identifier, value) VALUES (1, 'Normal Sync');
INSERT INTO flash_mode (identifier, value) VALUES (2, 'Slow Sync');
INSERT INTO flash_mode (identifier, value) VALUES (3, 'Rear Sync');


--
-- Data for Name: flash_type; Type: TABLE DATA; Schema: public; Owner: po_user
--



--
-- Data for Name: folder; Type: TABLE DATA; Schema: public; Owner: po_user
--

INSERT INTO folder (identifier, parent_folder, users, caption, date_of_creation, access_rights, description, "password", date_changed, event) VALUES (1, NULL, 1, 'Trash', now(), 3, NULL, NULL, now(), NULL);


--
-- Data for Name: label; Type: TABLE DATA; Schema: public; Owner: po_user
--

INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (1, NULL, 1, '5267 - Return Address', 52.400199999999998, 12.699999999999999, 44.450000000000003, 12.699999999999999, 7.1374000000000004, 12.699999999999999, 4, 20, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (2, NULL, 1, '5960 - Address', 69.849999999999994, 25.399999999999999, 66.674999999999997, 25.399999999999999, 4.7751999999999999, 12.699999999999999, 3, 10, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (3, NULL, 1, '5961 - Address', 106.3625, 25.399999999999999, 101.59999999999999, 25.399999999999999, 3.9624000000000001, 12.699999999999999, 2, 10, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (4, NULL, 1, '5962 - Address', 106.3625, 33.865819999999999, 101.59999999999999, 33.865819999999999, 3.9624000000000001, 21.16582, 2, 7, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (5, NULL, 1, '5963 - Address', 106.3625, 50.799999999999997, 101.59999999999999, 50.799999999999997, 3.9624000000000001, 12.699999999999999, 2, 5, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (6, NULL, 1, '5264 - Address', 106.3625, 84.665819999999997, 101.59999999999999, 84.665819999999997, 3.9624000000000001, 12.699999999999999, 2, 3, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (7, NULL, 1, '5667 - Clear Return Address', 52.07, 12.699999999999999, 44.450000000000003, 12.699999999999999, 7.6200000000000001, 12.699999999999999, 4, 20, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (8, NULL, 1, '5930 - Clear Address', 71.965819999999994, 25.399999999999999, 71.958200000000005, 25.399999999999999, 0, 12.699999999999999, 3, 10, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (9, NULL, 1, '5932 - Clear Address', 107.95, 33.858199999999997, 107.95, 33.858199999999997, 0, 21.209, 2, 7, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (10, NULL, 1, '5664 - Clear Address', 107.95, 84.665819999999997, 107.95, 84.658199999999994, 0, 12.699999999999999, 2, 3, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (11, NULL, 1, '5265 - Full Sheet', 0, 0, 215.90000000000001, 279.39999999999998, 0, 0, 1, 1, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (12, NULL, 1, '5266 - File Folder-Assorted', 101.59999999999999, 17.145, 87.325199999999995, 13.741400000000001, 13.487399999999999, 12.699999999999999, 2, 15, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (13, NULL, 1, '5388 - Index Card', 127, 76.200000000000003, 127, 76.200000000000003, 44.450000000000003, 25.399999999999999, 1, 3, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (14, NULL, 1, '5305 - Laser Tent Cards Med.', 215.90000000000001, 127, 196.84999999999999, 107.95, 9.5250000000000004, 22.225000000000001, 1, 2, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (15, NULL, 1, '5309 - Laser Tent Cards Large', 279.39999999999998, 177.80000000000001, 260.35000000000002, 158.75, 9.5250000000000004, 28.574999999999999, 1, 1, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (16, NULL, 1, '5315 - Laser Note Cards', 139.69999999999999, 0, 139.69999999999999, 215.90000000000001, 0, 0, 2, 1, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (17, NULL, 1, '6020 - 35mm Slide', 2.54, 0, 43.18, 8.4700000000000006, 14.83, 12.91, 4, 32, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (18, NULL, 1, '8920 - Ink Jet Address', 69.849999999999994, 25.399999999999999, 66.674999999999997, 25.399999999999999, 4.7751999999999999, 12.699999999999999, 3, 10, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (19, NULL, 1, '8161 - Ink Jet Address', 106.3625, 25.399999999999999, 101.59999999999999, 25.399999999999999, 3.9624000000000001, 12.699999999999999, 2, 10, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (20, NULL, 1, '8922 - Ink Jet Address', 106.3625, 33.865819999999999, 101.59999999999999, 33.865819999999999, 3.9624000000000001, 21.16582, 2, 7, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (21, NULL, 1, '8923 - Ink Jet Address', 106.3625, 50.799999999999997, 101.59999999999999, 50.799999999999997, 3.9624000000000001, 12.699999999999999, 2, 5, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (22, NULL, 1, '8164 - Ink Jet Address', 106.3625, 84.665819999999997, 101.59999999999999, 84.665819999999997, 3.9624000000000001, 12.699999999999999, 2, 3, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (23, NULL, 1, '8165 - Ink Jet Full Sheet', 215.90000000000001, 279.39999999999998, 215.90000000000001, 279.39999999999998, 0, 0, 1, 1, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (24, NULL, 1, '8166 - Ink Jet File Folder-Assorted', 101.59999999999999, 17.145, 87.325199999999995, 13.741400000000001, 13.487399999999999, 12.699999999999999, 2, 15, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (25, NULL, 1, '8927 - Ink Jet Return Address', 52.400199999999998, 12.699999999999999, 44.450000000000003, 12.699999999999999, 7.1374000000000004, 12.699999999999999, 4, 20, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (26, NULL, 1, '8315 - Ink Jet Note Cards', 139.69999999999999, 0, 139.69999999999999, 215.90000000000001, 0, 0, 2, 1, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (27, NULL, 1, '8316 - Ink Jet Greeting Card(Portrait)', 215.90000000000001, 139.69999999999999, 215.90000000000001, 139.69999999999999, 0, 0, 1, 2, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (28, NULL, 1, '8316 - Ink Jet Greeting Card(Landscape)', 139.69999999999999, 215.90000000000001, 139.69999999999999, 215.90000000000001, 0, 0, 2, 1, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (29, NULL, 1, '8371 - Ink Jet Business Card', 88.900000000000006, 50.799999999999997, 88.900000000000006, 50.799999999999997, 19.050000000000001, 12.699999999999999, 2, 5, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (30, NULL, 1, '8372 - Ink Jet Business Card', 88.900000000000006, 50.799999999999997, 88.900000000000006, 50.799999999999997, 19.050000000000001, 12.699999999999999, 2, 5, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (31, NULL, 1, '8660 - Ink Jet Clear Address', 69.849999999999994, 25.399999999999999, 66.674999999999997, 25.399999999999999, 4.7751999999999999, 12.699999999999999, 3, 10, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (32, NULL, 1, '8930 - Ink Jet Clear Address', 69.849999999999994, 25.399999999999999, 66.674999999999997, 25.399999999999999, 4.7751999999999999, 12.699999999999999, 3, 10, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (33, NULL, 1, '8662 - Ink Jet Clear Address', 106.3625, 33.865819999999999, 101.59999999999999, 33.865819999999999, 3.9624000000000001, 21.16582, 2, 7, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (34, NULL, 1, '8932 - Ink Jet Clear Address', 106.3625, 33.865819999999999, 101.59999999999999, 33.865819999999999, 3.9624000000000001, 21.16582, 2, 7, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (35, NULL, 1, '8663 - Ink Jet Clear Address/Shipping', 106.3625, 50.799999999999997, 101.59999999999999, 50.799999999999997, 3.9624000000000001, 12.699999999999999, 2, 5, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (36, NULL, 1, '2160 - Addressing', 66.802000000000007, 25.399999999999999, 66.802000000000007, 25.399999999999999, 20.574000000000002, 12.699999999999999, 1, 8, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (37, NULL, 1, '2162 - Addressing', 101.59999999999999, 33.865819999999999, 101.59999999999999, 33.865819999999999, 3.1749999999999998, 12.699999999999999, 1, 6, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (38, NULL, 1, '2163 - Address/Shipping', 101.59999999999999, 50.799999999999997, 101.59999999999999, 50.799999999999997, 3.1749999999999998, 12.699999999999999, 1, 4, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (39, NULL, 1, '2164 - Shipping', 101.59999999999999, 127, 101.59999999999999, 84.137500000000003, 3.1749999999999998, 21.431249999999999, 1, 2, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (40, NULL, 1, '2660 - Clear Address', 66.802000000000007, 25.399999999999999, 66.802000000000007, 25.399999999999999, 20.574000000000002, 12.699999999999999, 1, 8, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (41, NULL, 1, '2662 - Clear Address', 107.95, 33.865819999999999, 107.95, 33.865819999999999, 0, 12.699999999999999, 1, 6, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (42, NULL, 1, '2663 - Clear Address/Shipping', 107.95, 50.799999999999997, 107.95, 50.799999999999997, 0, 12.699999999999999, 1, 4, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (43, NULL, 1, '2180 - File Folder-Assorted', 87.376000000000005, 17.145, 87.376000000000005, 13.741400000000001, 10.287000000000001, 12.699999999999999, 1, 12, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (44, NULL, 1, '2181 - File Folder-White', 87.376000000000005, 17.145, 87.376000000000005, 16.916399999999999, 10.287000000000001, 12.699999999999999, 1, 12, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (45, NULL, 1, '5824 - CD-Recordable Labels', 114.3, 139.69999999999999, 114.3, 114.3, 50.799999999999997, 12.699999999999999, 1, 2, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (46, NULL, 1, '5836 - MtgCreator Portfolio Label', 88.900000000000006, 50.799999999999997, 88.900000000000006, 50.799999999999997, 19.050000000000001, 12.699999999999999, 2, 5, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (47, NULL, 1, '5836 - MtgCreator Divider 5 Tab', 52.07, 12.699999999999999, 44.450000000000003, 12.699999999999999, 7.6200000000000001, 12.699999999999999, 4, 20, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (48, NULL, 1, '5841 - MtgCreator Portfolio Label', 88.900000000000006, 50.799999999999997, 88.900000000000006, 50.799999999999997, 19.050000000000001, 12.699999999999999, 2, 5, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (49, NULL, 1, '5843 - MtgCreator Name Badge', 88.900000000000006, 55.033417999999998, 88.900000000000006, 55.033417999999998, 19.050000000000001, 29.633417999999999, 2, 4, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (50, NULL, 1, '5844 - MtgCreator Tent Card', 215.90000000000001, 127, 196.84999999999999, 107.95, 9.5250000000000004, 22.225000000000001, 1, 2, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (51, NULL, 1, '5845 - MtgCreator Clear Address', 71.958200000000005, 25.399999999999999, 71.958200000000005, 25.399999999999999, 0, 12.699999999999999, 3, 10, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (52, NULL, 1, '5845 - MtgCreator Note Card', 139.69999999999999, 0, 139.69999999999999, 215.90000000000001, 0, 0, 2, 1, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (53, NULL, 1, '3110 - Rectangle Stickers', 68.262500000000003, 38.100000000000001, 60.325000000000003, 31.75, 9.5250000000000004, 28.574999999999999, 3, 6, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (54, NULL, 1, '3111 - Square Stickers', 68.262500000000003, 76.200000000000003, 63.5, 63.5, 7.9501999999999997, 31.75, 3, 3, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (55, NULL, 1, '3112 - Small Round Stickers', 50.799999999999997, 50.799999999999997, 38.100000000000001, 38.100000000000001, 12.699999999999999, 19.050000000000001, 4, 5, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (56, NULL, 1, '3113 - Big Round Stickers', 68.262500000000003, 76.200000000000003, 63.5, 63.5, 7.9375, 31.75, 3, 3, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (57, NULL, 1, '3114 - Full Page Stickers', 215.90000000000001, 279.39999999999998, 215.90000000000001, 279.39999999999998, 0, 0, 1, 1, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (58, NULL, 1, '3610 - Greeting Cards', 177.80000000000001, 127, 177.80000000000001, 114.3, 19.050000000000001, 19.050000000000001, 1, 2, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (59, NULL, 1, '3611 - Postcards', 152.40000000000001, 114.3, 152.40000000000001, 101.59999999999999, 31.75, 31.75, 1, 2, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (60, NULL, 1, '3612 - Biz Cards', 101.59999999999999, 63.5, 88.900000000000006, 50.799999999999997, 12.699999999999999, 19.050000000000001, 2, 4, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (61, NULL, 1, '3613 - Laminated ID Cards', 82.549999999999997, 93.131640000000004, 82.549999999999997, 50.799999999999997, 109.53749999999999, 21.16582, 1, 3, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (62, NULL, 1, 'L7159 - Address', 66.540126000000001, 33.859977999999998, 64.000125999999995, 33.859977999999998, 6.4599820000000001, 13.785088, 3, 8, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (63, NULL, 1, 'L7160 - Address', 66.049905999999993, 38.100000000000001, 63.5, 38.100000000000001, 7.199884, 15.875, 3, 7, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (64, NULL, 1, 'L7161 - Address', 66.040000000000006, 46.559978000000001, 63.5, 46.559978000000001, 7.2100439999999999, 9.5450660000000003, 3, 6, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (65, NULL, 1, 'L7162 - Address', 101.59999999999999, 33.869883999999999, 99.060000000000002, 33.869883999999999, 4.6700439999999999, 13.744956, 2, 8, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (66, NULL, 1, 'L7163 - Address', 101.59999999999999, 38.100000000000001, 99.060000000000002, 38.100000000000001, 4.6700439999999999, 15.875, 2, 7, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (67, NULL, 1, 'L7164 - Address', 66.040000000000006, 71.967089999999999, 63.5, 71.967089999999999, 7.2100439999999999, 5.2900580000000001, 3, 4, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (68, NULL, 1, 'L7165 - Parcel', 101.59999999999999, 67.730115999999995, 99.060000000000002, 67.730115999999995, 4.6700439999999999, 13.759942000000001, 2, 4, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (69, NULL, 1, 'L7166 - Parcel', 101.59999999999999, 93.130116000000001, 99.060000000000002, 93.130116000000001, 4.6700439999999999, 9.5250000000000004, 2, 3, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (70, NULL, 1, 'L7167 - Shipping', 199.60005799999999, 289.04996799999998, 199.60005799999999, 289.04996799999998, 5.1998879999999996, 4.7000159999999997, 1, 1, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (71, NULL, 1, 'L7168 - Shipping', 199.60005799999999, 143.53006600000001, 199.60005799999999, 143.53006600000001, 4.7000159999999997, 5.6949339999999999, 1, 2, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (72, NULL, 1, 'L7169 - Parcel', 138.999976, 101.639878, 138.999976, 99.099878000000004, 10.219944, 4.629912, 2, 2, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (73, NULL, 1, 'L7173 - Shipping', 101.59999999999999, 56.999885999999996, 99.060000000000002, 56.999885999999996, 4.6700439999999999, 6.7249040000000004, 2, 5, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (74, NULL, 1, 'L7551 - Clear Mini Address', 40.640000000000001, 21.169884, 38.100000000000001, 21.169884, 4.6700439999999999, 11.640058, 5, 13, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (75, NULL, 1, 'L7552 - Petite Clear', 104.18292599999999, 21.166073999999998, 55.015892000000001, 12.699999999999999, 25.399999999999999, 47.625, 2, 10, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (76, NULL, 1, 'L7553 - Petite Clear', 45.719999999999999, 16.931894, 21.995892000000001, 12.699999999999999, 25.399999999999999, 49.732945999999998, 4, 12, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (77, NULL, 1, 'L7560 - Clear Address', 66.049905999999993, 38.100000000000001, 63.5, 38.100000000000001, 7.199884, 15.875, 3, 7, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (78, NULL, 1, 'L7562 - Clear Address', 101.59999999999999, 33.869883999999999, 99.060000000000002, 33.869883999999999, 4.6700439999999999, 13.744956, 2, 8, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (79, NULL, 1, 'L7563 - Clear Address', 101.59999999999999, 38.100000000000001, 99.060000000000002, 38.100000000000001, 4.6700439999999999, 15.875, 2, 7, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (80, NULL, 1, 'L7565 - Clear Parcel', 101.59999999999999, 67.730115999999995, 99.060000000000002, 67.730115999999995, 4.6700439999999999, 13.759942000000001, 2, 4, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (81, NULL, 1, 'L7263 - Address', 101.59999999999999, 38.100000000000001, 99.060000000000002, 38.100000000000001, 4.6700439999999999, 15.875, 2, 7, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (82, NULL, 1, 'L7651 - Mini Address', 40.640000000000001, 21.169884, 38.100000000000001, 21.169884, 4.6700439999999999, 11.640058, 5, 13, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (83, NULL, 1, 'L7656 - 35mm Slide', 50.669952000000002, 12.699999999999999, 45.999907999999998, 11.109959999999999, 5.7701180000000001, 16.670020000000001, 4, 21, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (84, NULL, 1, 'L7665 - Mini Data Cartridge', 94.000066000000004, 21.120100000000001, 72.000110000000006, 21.150072000000002, 21.999956000000001, 22.29993, 2, 12, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (85, NULL, 1, 'L7667 - Standard Data Cartridge', 132.999988, 29.610050000000001, 132.999988, 29.610050000000001, 38.500050000000002, 15.979901999999999, 1, 9, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (86, NULL, 1, 'L7690 - Mini Address', 40.640000000000001, 21.169884, 38.100000000000001, 21.169884, 4.6700439999999999, 11.640058, 5, 13, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (87, NULL, 1, 'L7170 - Eurofolio', 133.99998600000001, 10.999978, 133.99998600000001, 10.999978, 37.999924, 17.225010000000001, 1, 24, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (88, NULL, 1, 'L7172 - Ring Binder', 102.540054, 29.999939999999999, 100.00005400000001, 29.999939999999999, 3.7299899999999999, 14.225016, 2, 9, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (89, NULL, 1, 'L7171 - Lever Arch', 59.999879999999997, 200.00010800000001, 59.999879999999997, 200.00010800000001, 29.224986000000001, 4.9999900000000004, 4, 1, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (90, NULL, 1, 'L7701 - Lever Arch', 61.999876, 192.000124, 61.999876, 192.000124, 25.224993999999999, 8.9999819999999993, 4, 1, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (91, NULL, 1, 'L7702 - Lever Arch', 38.999921999999998, 192.000124, 38.999921999999998, 192.000124, 12.724892000000001, 8.9999819999999993, 7, 1, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (92, NULL, 1, 'L7413 - Business Card', 90.000073999999998, 50.799999999999997, 90.000073999999998, 50.799999999999997, 14.999969999999999, 21.500084000000001, 2, 5, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (93, NULL, 1, 'L7414 - Business Card', 90.000073999999998, 51.999896, 90.000073999999998, 51.999896, 14.999969999999999, 18.50009, 2, 5, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (94, NULL, 1, 'L7415 - Business Card', 90.000073999999998, 51.999896, 90.000073999999998, 51.999896, 14.999969999999999, 18.50009, 2, 5, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (95, NULL, 1, 'L7417 - Business Card', 90.000073999999998, 53.999892000000003, 90.000073999999998, 53.999892000000003, 14.999969999999999, 13.5001, 2, 5, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (96, NULL, 1, 'L7419 - Business Card', 90.000073999999998, 53.999892000000003, 90.000073999999998, 53.999892000000003, 14.999969999999999, 13.5001, 2, 5, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (97, NULL, 1, 'L7418 - Name Badge', 86.000082000000006, 54.999890000000001, 86.000082000000006, 54.999890000000001, 18.999962, 38.500050000000002, 2, 4, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (98, NULL, 1, 'L7421 - Post Card', 139.37005400000001, 97.289873999999998, 139.37005400000001, 97.289873999999998, 9.4800419999999992, 7.7099159999999998, 2, 2, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (99, NULL, 1, 'L7901 - Organizer Page', 95.000063999999995, 169.99991399999999, 95.000063999999995, 169.99991399999999, 0, 0, 3, 1, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (100, NULL, 1, 'L7902 - Organizer Page', 148.50008399999999, 210.00008800000001, 148.50008399999999, 210.00008800000001, 0, 0, 2, 1, 14, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (101, NULL, 1, 'J8159 - Address', 66.540126000000001, 33.859977999999998, 64.000125999999995, 33.859977999999998, 6.4599820000000001, 13.785088, 3, 8, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (102, NULL, 1, 'J8160 - Address', 66.049905999999993, 38.100000000000001, 63.5, 38.100000000000001, 7.199884, 15.875, 3, 7, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (103, NULL, 1, 'J8161 - Address', 66.040000000000006, 46.559978000000001, 63.5, 46.559978000000001, 7.2100439999999999, 9.5450660000000003, 3, 6, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (104, NULL, 1, 'J8162 - Address', 101.59999999999999, 33.869883999999999, 99.060000000000002, 33.869883999999999, 4.6700439999999999, 13.744956, 2, 8, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (105, NULL, 1, 'J8163 - Address', 101.59999999999999, 38.100000000000001, 99.060000000000002, 38.100000000000001, 4.6700439999999999, 15.875, 2, 7, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (106, NULL, 1, 'J8164 - Address', 66.040000000000006, 71.967089999999999, 63.5, 71.967089999999999, 7.2100439999999999, 5.2900580000000001, 3, 4, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (107, NULL, 1, 'J8165 - Parcel', 101.59999999999999, 67.730115999999995, 99.060000000000002, 67.730115999999995, 4.6700439999999999, 13.759942000000001, 2, 4, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (108, NULL, 1, 'J8166 - Parcel', 101.59999999999999, 93.130116000000001, 99.060000000000002, 93.130116000000001, 4.6700439999999999, 9.5250000000000004, 2, 3, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (109, NULL, 1, 'J8167 - Shipping', 199.60005799999999, 289.04996799999998, 199.60005799999999, 289.04996799999998, 5.1998879999999996, 4.7000159999999997, 1, 1, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (110, NULL, 1, 'J8168 - Shipping', 199.60005799999999, 143.53006600000001, 199.60005799999999, 143.53006600000001, 4.7000159999999997, 5.6949339999999999, 1, 2, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (111, NULL, 1, 'J8169 - Parcel', 138.999976, 101.639878, 138.999976, 99.099878000000004, 10.219944, 4.629912, 2, 2, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (112, NULL, 1, 'J8173 - Shipping', 101.59999999999999, 56.999885999999996, 99.060000000000002, 56.999885999999996, 4.6700439999999999, 6.7249040000000004, 2, 5, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (113, NULL, 1, 'J8360 - Address', 66.049905999999993, 38.100000000000001, 63.5, 38.100000000000001, 7.199884, 15.875, 3, 7, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (114, NULL, 1, 'J8361 - Address', 66.040000000000006, 46.559978000000001, 63.5, 46.559978000000001, 7.2100439999999999, 9.5450660000000003, 3, 6, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (115, NULL, 1, 'J8362 - Address', 101.59999999999999, 33.869883999999999, 99.060000000000002, 33.869883999999999, 4.6700439999999999, 13.744956, 2, 8, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (116, NULL, 1, 'J8363 - Address', 101.59999999999999, 38.100000000000001, 99.060000000000002, 38.100000000000001, 4.6700439999999999, 15.875, 2, 7, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (117, NULL, 1, 'J8365 - Parcel', 101.59999999999999, 67.730115999999995, 99.060000000000002, 67.730115999999995, 4.6700439999999999, 13.759942000000001, 2, 4, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (118, NULL, 1, 'J8367 - Shipping', 199.60005799999999, 289.04996799999998, 199.60005799999999, 289.04996799999998, 5.1998879999999996, 4.7000159999999997, 1, 1, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (119, NULL, 1, 'J8368 - Shipping', 199.60005799999999, 143.53006600000001, 199.60005799999999, 143.53006600000001, 4.7000159999999997, 5.6949339999999999, 1, 2, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (120, NULL, 1, 'J8560 - Clear Address', 66.049905999999993, 38.100000000000001, 63.5, 38.100000000000001, 7.199884, 15.875, 3, 7, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (121, NULL, 1, 'J8562 - Clear Address', 101.59999999999999, 33.869883999999999, 99.060000000000002, 33.869883999999999, 4.6700439999999999, 13.744956, 2, 8, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (122, NULL, 1, 'J8563 - Clear Address', 101.59999999999999, 38.100000000000001, 99.060000000000002, 38.100000000000001, 4.6700439999999999, 15.875, 2, 7, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (123, NULL, 1, 'J8565 - Clear Parcel', 101.59999999999999, 67.730115999999995, 99.060000000000002, 67.730115999999995, 4.6700439999999999, 13.759942000000001, 2, 4, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (124, NULL, 1, 'J8651 - Mini Address', 40.640000000000001, 21.169884, 38.100000000000001, 21.169884, 4.6700439999999999, 11.640058, 5, 13, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (125, NULL, 1, 'J8657 - 35mm Slide', 50.669952000000002, 12.699999999999999, 45.999907999999998, 11.109959999999999, 31.329884, 61.120019999999997, 3, 14, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (126, NULL, 1, 'L2160 - Address', 66.799968000000007, 25.399999999999999, 66.799968000000007, 25.399999999999999, 20.569935999999998, 12.699999999999999, 1, 8, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (127, NULL, 1, 'L2162 - Address', 101.59999999999999, 33.779967999999997, 101.59999999999999, 33.781999999999996, 3.1749999999999998, 12.699999999999999, 1, 6, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (128, NULL, 1, 'L2163 - Parcel', 101.59999999999999, 50.799999999999997, 101.59999999999999, 50.799999999999997, 3.1749999999999998, 12.699999999999999, 1, 4, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (129, NULL, 1, 'L30063 - Address', 99.099878000000004, 63.5, 99.099878000000004, 38.100000000000001, 55.449978000000002, 23.500080000000001, 1, 2, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (130, NULL, 1, 'L30065 - Parcel', 99.099878000000004, 67.699889999999996, 99.099878000000004, 67.699889999999996, 5.8999119999999996, 40.399970000000003, 2, 1, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (131, NULL, 1, 'L300SL - 35mm Slide', 45.999907999999998, 33.839911999999998, 45.999907999999998, 11.109959999999999, 12.999974, 17.934940000000001, 4, 4, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (132, NULL, 1, 'J40063 - Address', 99.099878000000004, 63.5, 99.099878000000004, 38.100000000000001, 55.449978000000002, 23.500080000000001, 1, 2, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (133, NULL, 1, 'J40065 - Parcel', 99.099878000000004, 67.699889999999996, 99.099878000000004, 67.699889999999996, 5.8999119999999996, 40.399970000000003, 2, 1, 13, now(), now());
INSERT INTO label (identifier, users, manufacturer, name, space_x, space_y, size_x, size_y, offset_x, offset_y, columns, "rows", paper, date_of_creation, last_modified_date) VALUES (134, NULL, 1, 'J400SL - 35mm Slide', 45.999907999999998, 33.839911999999998, 45.999907999999998, 11.109959999999999, 12.999974, 17.934940000000001, 4, 4, 13, now(), now());


--
-- Data for Name: lens; Type: TABLE DATA; Schema: public; Owner: po_user
--



--
-- Data for Name: lens_type; Type: TABLE DATA; Schema: public; Owner: po_user
--



--
-- Data for Name: location; Type: TABLE DATA; Schema: public; Owner: po_user
--

INSERT INTO "location" (identifier, place, state, country, last_modified_date, users, city, latitude, longitude, private) VALUES (0, 'Unknown', NULL, NULL, NULL, NULL, NULL, NULL, NULL, false);


--
-- Data for Name: manufacturer; Type: TABLE DATA; Schema: public; Owner: po_user
--

INSERT INTO manufacturer (identifier, name, url, last_modified_date, last_modifying_users) VALUES (1, 'Avery', 'http://www.avery.com', now(), 1);
INSERT INTO manufacturer (identifier, name, url, last_modified_date, last_modifying_users) VALUES (2, 'Canon', 'http://www.canon.com', now(), 1);
INSERT INTO manufacturer (identifier, name, url, last_modified_date, last_modifying_users) VALUES (3, 'Fuji Film', 'http://www.fujifilm.com', now(), 1);
INSERT INTO manufacturer (identifier, name, url, last_modified_date, last_modifying_users) VALUES (4, 'Hasselblad', 'http://www.hasselblad.com', now(), 1);
INSERT INTO manufacturer (identifier, name, url, last_modified_date, last_modifying_users) VALUES (5, 'Kodak', 'http://www.kodak.com', now(), 1);
INSERT INTO manufacturer (identifier, name, url, last_modified_date, last_modifying_users) VALUES (6, 'Mamiya', 'http://www.mamiya.com', now(), 1);
INSERT INTO manufacturer (identifier, name, url, last_modified_date, last_modifying_users) VALUES (7, 'Minolta', 'http://www.minolta.com', now(), 1);
INSERT INTO manufacturer (identifier, name, url, last_modified_date, last_modifying_users) VALUES (8, 'Nikon', 'http://www.nikon-image.com', now(), 1);
INSERT INTO manufacturer (identifier, name, url, last_modified_date, last_modifying_users) VALUES (9, 'Olympus', 'http://www.olympus.com', now(), 1);
INSERT INTO manufacturer (identifier, name, url, last_modified_date, last_modifying_users) VALUES (10, 'Pentax', 'http://www.pentax.com', now(), 1);
INSERT INTO manufacturer (identifier, name, url, last_modified_date, last_modifying_users) VALUES (11, 'Rollei', 'http://www.rollei.de', now(), 1);
INSERT INTO manufacturer (identifier, name, url, last_modified_date, last_modifying_users) VALUES (12, 'Sinar', 'http://www.sinar.ch', now(), 1);
INSERT INTO manufacturer (identifier, name, url, last_modified_date, last_modifying_users) VALUES (13, 'Sony', 'http://www.sony.com', now(), 1);


--
-- Data for Name: orientations; Type: TABLE DATA; Schema: public; Owner: po_user
--

INSERT INTO orientations (identifier, name, im_transform) VALUES (1, 'Normal (0 deg)', ' ');
INSERT INTO orientations (identifier, name, im_transform) VALUES (2, 'Mirrored', ' -flop ');
INSERT INTO orientations (identifier, name, im_transform) VALUES (3, 'Upsidedown', ' -rotate 180 ');
INSERT INTO orientations (identifier, name, im_transform) VALUES (4, 'Upsidedown Mirrored', ' -flip ');
INSERT INTO orientations (identifier, name, im_transform) VALUES (5, '90 deg CW Mirrored', ' -flip -rotate 90 ');
INSERT INTO orientations (identifier, name, im_transform) VALUES (6, '90 deg CCW', ' -rotate 270 ');
INSERT INTO orientations (identifier, name, im_transform) VALUES (7, '90 deg CCW Mirrored', ' -flop -rotate 90 ');
INSERT INTO orientations (identifier, name, im_transform) VALUES (8, '90 deg CW', ' -rotate 90 ');


--
-- Data for Name: paper; Type: TABLE DATA; Schema: public; Owner: po_user
--

INSERT INTO paper (identifier, name, size_x, size_y, last_modified_date, last_modifying_users) VALUES (1, 'Letter    - (216 x 279 mm)', 612, 792, now(), 1);
INSERT INTO paper (identifier, name, size_x, size_y, last_modified_date, last_modifying_users) VALUES (2, 'Legal     - (216 x 356 mm)', 612, 1008, now(), 1);
INSERT INTO paper (identifier, name, size_x, size_y, last_modified_date, last_modifying_users) VALUES (3, 'Statement - (140 x 216 mm)', 396, 612, now(), 1);
INSERT INTO paper (identifier, name, size_x, size_y, last_modified_date, last_modifying_users) VALUES (4, 'Tabloid   - (279 x 432 mm)', 792, 1224, now(), 1);
INSERT INTO paper (identifier, name, size_x, size_y, last_modified_date, last_modifying_users) VALUES (5, 'Ledger    - (432 x 279 mm)', 1224, 792, now(), 1);
INSERT INTO paper (identifier, name, size_x, size_y, last_modified_date, last_modifying_users) VALUES (6, 'Folio     - (216 x 330 mm)', 612, 936, now(), 1);
INSERT INTO paper (identifier, name, size_x, size_y, last_modified_date, last_modifying_users) VALUES (7, 'Quarto    - (215 x 275 mm)', 610, 780, now(), 1);
INSERT INTO paper (identifier, name, size_x, size_y, last_modified_date, last_modifying_users) VALUES (8, 'Executive - (190 x 254 mm)', 540, 720, now(), 1);
INSERT INTO paper (identifier, name, size_x, size_y, last_modified_date, last_modifying_users) VALUES (9, 'A0        - (841 x 1189 mm)', 2384, 3370, now(), 1);
INSERT INTO paper (identifier, name, size_x, size_y, last_modified_date, last_modifying_users) VALUES (10, 'A1        - (594 x 841 mm)', 1684, 2384, now(), 1);
INSERT INTO paper (identifier, name, size_x, size_y, last_modified_date, last_modifying_users) VALUES (11, 'A2        - (420 x 594 mm)', 1191, 1684, now(), 1);
INSERT INTO paper (identifier, name, size_x, size_y, last_modified_date, last_modifying_users) VALUES (12, 'A3        - (297 x 420 mm)', 842, 1191, now(), 1);
INSERT INTO paper (identifier, name, size_x, size_y, last_modified_date, last_modifying_users) VALUES (13, 'A4        - (210 x 297 mm)', 595, 842, now(), 1);
INSERT INTO paper (identifier, name, size_x, size_y, last_modified_date, last_modifying_users) VALUES (14, 'A5        - (148 x 210 mm)', 420, 595, now(), 1);
INSERT INTO paper (identifier, name, size_x, size_y, last_modified_date, last_modifying_users) VALUES (15, 'A6        - (105 x 148 mm)', 297, 420, now(), 1);
INSERT INTO paper (identifier, name, size_x, size_y, last_modified_date, last_modifying_users) VALUES (16, 'A7        - (74 x 105 mm)', 210, 297, now(), 1);
INSERT INTO paper (identifier, name, size_x, size_y, last_modified_date, last_modifying_users) VALUES (17, 'A8        - (52 x 74 mm)', 148, 210, now(), 1);
INSERT INTO paper (identifier, name, size_x, size_y, last_modified_date, last_modifying_users) VALUES (18, 'A9        - (37 x 52 mm)', 105, 148, now(), 1);
INSERT INTO paper (identifier, name, size_x, size_y, last_modified_date, last_modifying_users) VALUES (19, 'A10       - (26 x 37 mm)', 73, 105, now(), 1);
INSERT INTO paper (identifier, name, size_x, size_y, last_modified_date, last_modifying_users) VALUES (20, 'B0        - (1030 x 1456 mm)', 2920, 4127, now(), 1);
INSERT INTO paper (identifier, name, size_x, size_y, last_modified_date, last_modifying_users) VALUES (21, 'B1        - (728 x 1030 mm)', 2064, 2920, now(), 1);
INSERT INTO paper (identifier, name, size_x, size_y, last_modified_date, last_modifying_users) VALUES (22, 'B2        - (515 x 728 mm)', 1460, 2064, now(), 1);
INSERT INTO paper (identifier, name, size_x, size_y, last_modified_date, last_modifying_users) VALUES (23, 'B3        - (364 x 515 mm)', 1032, 1460, now(), 1);
INSERT INTO paper (identifier, name, size_x, size_y, last_modified_date, last_modifying_users) VALUES (24, 'B4        - (257 x 364 mm)', 729, 1032, now(), 1);
INSERT INTO paper (identifier, name, size_x, size_y, last_modified_date, last_modifying_users) VALUES (25, 'B5        - (182 x 257 mm)', 516, 729, now(), 1);
INSERT INTO paper (identifier, name, size_x, size_y, last_modified_date, last_modifying_users) VALUES (26, 'B6        - (128 x 182 mm)', 363, 516, now(), 1);
INSERT INTO paper (identifier, name, size_x, size_y, last_modified_date, last_modifying_users) VALUES (27, 'B7        - (91 x 128 mm)', 258, 363, now(), 1);
INSERT INTO paper (identifier, name, size_x, size_y, last_modified_date, last_modifying_users) VALUES (28, 'B8        - (64 x 91 mm)', 181, 258, now(), 1);
INSERT INTO paper (identifier, name, size_x, size_y, last_modified_date, last_modifying_users) VALUES (29, 'B9        - (45 x 64 mm)', 127, 181, now(), 1);
INSERT INTO paper (identifier, name, size_x, size_y, last_modified_date, last_modifying_users) VALUES (30, 'B10       - (32 x 45 mm)', 91, 127, now(), 1);
INSERT INTO paper (identifier, name, size_x, size_y, last_modified_date, last_modifying_users) VALUES (31, 'C0        - (917 x 1297 mm)', 2599, 3676, now(), 1);
INSERT INTO paper (identifier, name, size_x, size_y, last_modified_date, last_modifying_users) VALUES (32, 'C1        - (648 x 917 mm)', 1837, 2599, now(), 1);
INSERT INTO paper (identifier, name, size_x, size_y, last_modified_date, last_modifying_users) VALUES (33, 'C2        - (458 x 648 mm)', 1298, 1837, now(), 1);
INSERT INTO paper (identifier, name, size_x, size_y, last_modified_date, last_modifying_users) VALUES (34, 'C3        - (324 x 457 mm)', 918, 1296, now(), 1);
INSERT INTO paper (identifier, name, size_x, size_y, last_modified_date, last_modifying_users) VALUES (35, 'C4        - (229 x 324 mm)', 649, 918, now(), 1);
INSERT INTO paper (identifier, name, size_x, size_y, last_modified_date, last_modifying_users) VALUES (36, 'C5        - (162 x 229 mm)', 459, 649, now(), 1);
INSERT INTO paper (identifier, name, size_x, size_y, last_modified_date, last_modifying_users) VALUES (37, 'C6        - (114 x 162 mm)', 323, 459, now(), 1);
INSERT INTO paper (identifier, name, size_x, size_y, last_modified_date, last_modifying_users) VALUES (38, 'C7        - (81 x 114 mm)', 230, 323, now(), 1);
INSERT INTO paper (identifier, name, size_x, size_y, last_modified_date, last_modifying_users) VALUES (39, 'ISOB0     - (1000 x 141 mm)', 2835, 4008, now(), 1);
INSERT INTO paper (identifier, name, size_x, size_y, last_modified_date, last_modifying_users) VALUES (40, 'ISOB1     - (707 x 1000 mm)', 2004, 2835, now(), 1);
INSERT INTO paper (identifier, name, size_x, size_y, last_modified_date, last_modifying_users) VALUES (41, 'ISOB2     - (500 x 707 mm)', 1417, 2004, now(), 1);
INSERT INTO paper (identifier, name, size_x, size_y, last_modified_date, last_modifying_users) VALUES (42, 'ISOB3     - (353 x 500 mm)', 1001, 1417, now(), 1);
INSERT INTO paper (identifier, name, size_x, size_y, last_modified_date, last_modifying_users) VALUES (43, 'ISOB4     - (250 x 353 mm)', 709, 1001, now(), 1);
INSERT INTO paper (identifier, name, size_x, size_y, last_modified_date, last_modifying_users) VALUES (44, 'ISOB5     - (176 x 250 mm)', 499, 709, now(), 1);
INSERT INTO paper (identifier, name, size_x, size_y, last_modified_date, last_modifying_users) VALUES (45, 'ISOB6     - (125 x 176 mm)', 354, 499, now(), 1);
INSERT INTO paper (identifier, name, size_x, size_y, last_modified_date, last_modifying_users) VALUES (46, 'ISOB7     - (88 x 125 mm)', 249, 354, now(), 1);
INSERT INTO paper (identifier, name, size_x, size_y, last_modified_date, last_modifying_users) VALUES (47, 'ISOB8     - (62 x 88 mm)', 176, 249, now(), 1);
INSERT INTO paper (identifier, name, size_x, size_y, last_modified_date, last_modifying_users) VALUES (48, 'ISOB9     - (44 x 62 mm)', 125, 176, now(), 1);
INSERT INTO paper (identifier, name, size_x, size_y, last_modified_date, last_modifying_users) VALUES (49, 'ISOB10    - (31 x 44 mm)', 88, 125, now(), 1);
INSERT INTO paper (identifier, name, size_x, size_y, last_modified_date, last_modifying_users) VALUES (50, '7x9       - (178 x 229 mm)', 504, 648, now(), 1);
INSERT INTO paper (identifier, name, size_x, size_y, last_modified_date, last_modifying_users) VALUES (51, '9x11      - (229 x 279 mm)', 648, 792, now(), 1);
INSERT INTO paper (identifier, name, size_x, size_y, last_modified_date, last_modifying_users) VALUES (52, '9x12      - (229 x 305 mm)', 648, 864, now(), 1);
INSERT INTO paper (identifier, name, size_x, size_y, last_modified_date, last_modifying_users) VALUES (53, '10x13     - (254 x 330 mm)', 720, 936, now(), 1);
INSERT INTO paper (identifier, name, size_x, size_y, last_modified_date, last_modifying_users) VALUES (54, '10x14     - (254 x 356 mm)', 720, 1008, now(), 1);


--
-- Data for Name: photo; Type: TABLE DATA; Schema: public; Owner: po_user
--



--
-- Data for Name: photo_dupe; Type: TABLE DATA; Schema: public; Owner: po_user
--



--
-- Data for Name: photo_keywords; Type: TABLE DATA; Schema: public; Owner: po_user
--



--
-- Data for Name: photo_tech; Type: TABLE DATA; Schema: public; Owner: po_user
--



--
-- Data for Name: photo_version; Type: TABLE DATA; Schema: public; Owner: po_user
--



--
-- Data for Name: rating; Type: TABLE DATA; Schema: public; Owner: po_user
--



--
-- Data for Name: scanner; Type: TABLE DATA; Schema: public; Owner: po_user
--



--
-- Data for Name: scanner_type; Type: TABLE DATA; Schema: public; Owner: po_user
--



--
-- Data for Name: shutter; Type: TABLE DATA; Schema: public; Owner: po_user
--

INSERT INTO shutter (identifier, value) VALUES (1, '60');
INSERT INTO shutter (identifier, value) VALUES (2, '30');
INSERT INTO shutter (identifier, value) VALUES (3, '25');
INSERT INTO shutter (identifier, value) VALUES (4, '20');
INSERT INTO shutter (identifier, value) VALUES (5, '15');
INSERT INTO shutter (identifier, value) VALUES (6, '13');
INSERT INTO shutter (identifier, value) VALUES (7, '10');
INSERT INTO shutter (identifier, value) VALUES (8, '8');
INSERT INTO shutter (identifier, value) VALUES (9, '6');
INSERT INTO shutter (identifier, value) VALUES (10, '5');
INSERT INTO shutter (identifier, value) VALUES (11, '4');
INSERT INTO shutter (identifier, value) VALUES (12, '3');
INSERT INTO shutter (identifier, value) VALUES (13, '2.5');
INSERT INTO shutter (identifier, value) VALUES (14, '2');
INSERT INTO shutter (identifier, value) VALUES (15, '1.6');
INSERT INTO shutter (identifier, value) VALUES (16, '1.3');
INSERT INTO shutter (identifier, value) VALUES (17, '1');
INSERT INTO shutter (identifier, value) VALUES (18, '1/1.3');
INSERT INTO shutter (identifier, value) VALUES (19, '1/1.6');
INSERT INTO shutter (identifier, value) VALUES (20, '1/2.0');
INSERT INTO shutter (identifier, value) VALUES (21, '1/2.5');
INSERT INTO shutter (identifier, value) VALUES (22, '1/3');
INSERT INTO shutter (identifier, value) VALUES (23, '1/4');
INSERT INTO shutter (identifier, value) VALUES (24, '1/5');
INSERT INTO shutter (identifier, value) VALUES (25, '1/6');
INSERT INTO shutter (identifier, value) VALUES (26, '1/8');
INSERT INTO shutter (identifier, value) VALUES (27, '1/10');
INSERT INTO shutter (identifier, value) VALUES (28, '1/13');
INSERT INTO shutter (identifier, value) VALUES (29, '1/15');
INSERT INTO shutter (identifier, value) VALUES (30, '1/20');
INSERT INTO shutter (identifier, value) VALUES (31, '1/25');
INSERT INTO shutter (identifier, value) VALUES (32, '1/30');
INSERT INTO shutter (identifier, value) VALUES (33, '1/40');
INSERT INTO shutter (identifier, value) VALUES (34, '1/50');
INSERT INTO shutter (identifier, value) VALUES (35, '1/60');
INSERT INTO shutter (identifier, value) VALUES (36, '1/80');
INSERT INTO shutter (identifier, value) VALUES (37, '1/100');
INSERT INTO shutter (identifier, value) VALUES (38, '1/125');
INSERT INTO shutter (identifier, value) VALUES (39, '1/160');
INSERT INTO shutter (identifier, value) VALUES (40, '1/200');
INSERT INTO shutter (identifier, value) VALUES (41, '1/250');
INSERT INTO shutter (identifier, value) VALUES (42, '1/320');
INSERT INTO shutter (identifier, value) VALUES (43, '1/400');
INSERT INTO shutter (identifier, value) VALUES (44, '1/500');
INSERT INTO shutter (identifier, value) VALUES (45, '1/640');
INSERT INTO shutter (identifier, value) VALUES (46, '1/800');
INSERT INTO shutter (identifier, value) VALUES (47, '1/1000');
INSERT INTO shutter (identifier, value) VALUES (48, '1/1250');
INSERT INTO shutter (identifier, value) VALUES (49, '1/1600');
INSERT INTO shutter (identifier, value) VALUES (50, '1/2000');
INSERT INTO shutter (identifier, value) VALUES (51, '1/2500');
INSERT INTO shutter (identifier, value) VALUES (52, '1/3200');
INSERT INTO shutter (identifier, value) VALUES (53, '1/4000');
INSERT INTO shutter (identifier, value) VALUES (54, '1/5000');
INSERT INTO shutter (identifier, value) VALUES (55, '1/6400');
INSERT INTO shutter (identifier, value) VALUES (56, '1/8000');


--
-- Data for Name: state; Type: TABLE DATA; Schema: public; Owner: po_user
--

INSERT INTO state (identifier, code, value) VALUES (1, 'AL', 'Alabama');
INSERT INTO state (identifier, code, value) VALUES (2, 'AK', 'Alaska');
INSERT INTO state (identifier, code, value) VALUES (3, 'AZ', 'Arizona');
INSERT INTO state (identifier, code, value) VALUES (4, 'AR', 'Arkansas');
INSERT INTO state (identifier, code, value) VALUES (5, 'CA', 'California');
INSERT INTO state (identifier, code, value) VALUES (6, 'CO', 'Colorado');
INSERT INTO state (identifier, code, value) VALUES (7, 'CT', 'Connecticut');
INSERT INTO state (identifier, code, value) VALUES (8, 'DE', 'Delaware');
INSERT INTO state (identifier, code, value) VALUES (9, 'DC', 'District of Columbia');
INSERT INTO state (identifier, code, value) VALUES (10, 'FL', 'Florida');
INSERT INTO state (identifier, code, value) VALUES (11, 'GA', 'Georgia');
INSERT INTO state (identifier, code, value) VALUES (12, 'HI', 'Hawaii');
INSERT INTO state (identifier, code, value) VALUES (13, 'ID', 'Idaho');
INSERT INTO state (identifier, code, value) VALUES (14, 'IL', 'Illinois');
INSERT INTO state (identifier, code, value) VALUES (15, 'IN', 'Indiana');
INSERT INTO state (identifier, code, value) VALUES (16, 'IA', 'Iowa');
INSERT INTO state (identifier, code, value) VALUES (17, 'KS', 'Kansas');
INSERT INTO state (identifier, code, value) VALUES (18, 'KY', 'Kentucky');
INSERT INTO state (identifier, code, value) VALUES (19, 'LA', 'Louisiana');
INSERT INTO state (identifier, code, value) VALUES (20, 'ME', 'Maine');
INSERT INTO state (identifier, code, value) VALUES (21, 'MD', 'Maryland');
INSERT INTO state (identifier, code, value) VALUES (22, 'MA', 'Massachusetts');
INSERT INTO state (identifier, code, value) VALUES (23, 'MI', 'Michigan');
INSERT INTO state (identifier, code, value) VALUES (24, 'MN', 'Minnesota');
INSERT INTO state (identifier, code, value) VALUES (25, 'MS', 'Mississippi');
INSERT INTO state (identifier, code, value) VALUES (26, 'MO', 'Missouri');
INSERT INTO state (identifier, code, value) VALUES (27, 'MT', 'Montana');
INSERT INTO state (identifier, code, value) VALUES (28, 'NE', 'Nebraska');
INSERT INTO state (identifier, code, value) VALUES (29, 'NV', 'Nevada');
INSERT INTO state (identifier, code, value) VALUES (30, 'NH', 'New Hampshire');
INSERT INTO state (identifier, code, value) VALUES (31, 'NJ', 'New Jersey');
INSERT INTO state (identifier, code, value) VALUES (32, 'NM', 'New Mexico');
INSERT INTO state (identifier, code, value) VALUES (33, 'NY', 'New York');
INSERT INTO state (identifier, code, value) VALUES (34, 'NC', 'North Carolina');
INSERT INTO state (identifier, code, value) VALUES (35, 'ND', 'North Dakota');
INSERT INTO state (identifier, code, value) VALUES (36, 'OH', 'Ohio');
INSERT INTO state (identifier, code, value) VALUES (37, 'OK', 'Oklahoma');
INSERT INTO state (identifier, code, value) VALUES (38, 'OR', 'Oregon');
INSERT INTO state (identifier, code, value) VALUES (39, 'PA', 'Pennsylvania');
INSERT INTO state (identifier, code, value) VALUES (40, 'RI', 'Rhode Island');
INSERT INTO state (identifier, code, value) VALUES (41, 'SC', 'South Carolina');
INSERT INTO state (identifier, code, value) VALUES (42, 'SD', 'South Dakota');
INSERT INTO state (identifier, code, value) VALUES (43, 'TN', 'Tennessee');
INSERT INTO state (identifier, code, value) VALUES (44, 'TX', 'Texas');
INSERT INTO state (identifier, code, value) VALUES (45, 'UT', 'Utah');
INSERT INTO state (identifier, code, value) VALUES (46, 'VT', 'Vermont');
INSERT INTO state (identifier, code, value) VALUES (47, 'VA', 'Virginia');
INSERT INTO state (identifier, code, value) VALUES (48, 'WA', 'Washington');
INSERT INTO state (identifier, code, value) VALUES (49, 'WV', 'West virginia');
INSERT INTO state (identifier, code, value) VALUES (50, 'WI', 'Wisconsin');
INSERT INTO state (identifier, code, value) VALUES (51, 'WY', 'Wyoming');
INSERT INTO state (identifier, code, value) VALUES (52, 'AS', 'American Samoa');
INSERT INTO state (identifier, code, value) VALUES (53, 'GU', 'Guam');
INSERT INTO state (identifier, code, value) VALUES (54, 'MP', 'Northern Mariana Islands');
INSERT INTO state (identifier, code, value) VALUES (55, 'PR', 'Puerto Rico');
INSERT INTO state (identifier, code, value) VALUES (56, 'VI', 'Virgin Islands');
INSERT INTO state (identifier, code, value) VALUES (57, 'FM', 'Federal States of Micronesia');
INSERT INTO state (identifier, code, value) VALUES (58, 'UM', 'United States Minor Outlying Islands');
INSERT INTO state (identifier, code, value) VALUES (59, '67', 'Johnston Atoll');
INSERT INTO state (identifier, code, value) VALUES (60, 'MH', 'Marshall Islands');
INSERT INTO state (identifier, code, value) VALUES (61, 'PW', 'Palau');
INSERT INTO state (identifier, code, value) VALUES (62, '', 'Midway Islands');
INSERT INTO state (identifier, code, value) VALUES (63, '', 'Navassa Island');
INSERT INTO state (identifier, code, value) VALUES (64, '', 'Wake Island');
INSERT INTO state (identifier, code, value) VALUES (65, '', 'Baker Island');
INSERT INTO state (identifier, code, value) VALUES (66, '', 'Howland Island');
INSERT INTO state (identifier, code, value) VALUES (67, '', 'Jarvis Island');
INSERT INTO state (identifier, code, value) VALUES (68, '', 'Kingman Reef');
INSERT INTO state (identifier, code, value) VALUES (69, '', 'Palmyra Atoll');


--
-- Data for Name: support; Type: TABLE DATA; Schema: public; Owner: po_user
--



--
-- Data for Name: support_type; Type: TABLE DATA; Schema: public; Owner: po_user
--



--
-- Data for Name: user_preferences; Type: TABLE DATA; Schema: public; Owner: po_user
--



--
-- Data for Name: user_type; Type: TABLE DATA; Schema: public; Owner: po_user
--

INSERT INTO user_type (identifier, value) VALUES (1, 'Disabled');
INSERT INTO user_type (identifier, value) VALUES (2, 'Client');
INSERT INTO user_type (identifier, value) VALUES (3, 'User');
INSERT INTO user_type (identifier, value) VALUES (4, 'Administrator');


--
-- Data for Name: users; Type: TABLE DATA; Schema: public; Owner: po_user
--

INSERT INTO users (identifier, first_name, last_name, company, member_since, "type", hide, username, "password", address1, address2, city, zipcode, state, country, phone, email, url) VALUES (1, 'admin', 'admin', '', now(), 4, false, 'admin', 'f6fdffe48c908deb0f4c3bd36c032e72', '', '', '', '', NULL, NULL, '', '', '');


--
-- Data for Name: version; Type: TABLE DATA; Schema: public; Owner: po_user
--

INSERT INTO version (identifier, description, major, minor, variation) VALUES (3, '2.35 customer release', 2, 35, 'A');


--
-- Name: access_type_pkey; Type: CONSTRAINT; Schema: public; Owner: po_user; Tablespace: 
--

ALTER TABLE ONLY access_type
    ADD CONSTRAINT access_type_pkey PRIMARY KEY (identifier);


--
-- Name: album_content_pkey; Type: CONSTRAINT; Schema: public; Owner: po_user; Tablespace: 
--

ALTER TABLE ONLY album_content
    ADD CONSTRAINT album_content_pkey PRIMARY KEY (identifier);


--
-- Name: album_pkey; Type: CONSTRAINT; Schema: public; Owner: po_user; Tablespace: 
--

ALTER TABLE ONLY album
    ADD CONSTRAINT album_pkey PRIMARY KEY (identifier);


--
-- Name: calendar_pkey; Type: CONSTRAINT; Schema: public; Owner: po_user; Tablespace: 
--

ALTER TABLE ONLY calendar
    ADD CONSTRAINT calendar_pkey PRIMARY KEY (identifier);


--
-- Name: camera_metering_pkey; Type: CONSTRAINT; Schema: public; Owner: po_user; Tablespace: 
--

ALTER TABLE ONLY camera_metering
    ADD CONSTRAINT camera_metering_pkey PRIMARY KEY (identifier);


--
-- Name: camera_pkey; Type: CONSTRAINT; Schema: public; Owner: po_user; Tablespace: 
--

ALTER TABLE ONLY camera
    ADD CONSTRAINT camera_pkey PRIMARY KEY (identifier);


--
-- Name: camera_program_pkey; Type: CONSTRAINT; Schema: public; Owner: po_user; Tablespace: 
--

ALTER TABLE ONLY camera_program
    ADD CONSTRAINT camera_program_pkey PRIMARY KEY (identifier);


--
-- Name: camera_type_pkey; Type: CONSTRAINT; Schema: public; Owner: po_user; Tablespace: 
--

ALTER TABLE ONLY camera_type
    ADD CONSTRAINT camera_type_pkey PRIMARY KEY (identifier);


--
-- Name: client_pkey; Type: CONSTRAINT; Schema: public; Owner: po_user; Tablespace: 
--

ALTER TABLE ONLY client
    ADD CONSTRAINT client_pkey PRIMARY KEY (identifier);


--
-- Name: client_status_pkey; Type: CONSTRAINT; Schema: public; Owner: po_user; Tablespace: 
--

ALTER TABLE ONLY client_status
    ADD CONSTRAINT client_status_pkey PRIMARY KEY (identifier);


--
-- Name: country_pkey; Type: CONSTRAINT; Schema: public; Owner: po_user; Tablespace: 
--

ALTER TABLE ONLY country
    ADD CONSTRAINT country_pkey PRIMARY KEY (identifier);


--
-- Name: ev_comp_pkey; Type: CONSTRAINT; Schema: public; Owner: po_user; Tablespace: 
--

ALTER TABLE ONLY ev_comp
    ADD CONSTRAINT ev_comp_pkey PRIMARY KEY (identifier);


--
-- Name: files_pkey; Type: CONSTRAINT; Schema: public; Owner: po_user; Tablespace: 
--

ALTER TABLE ONLY files
    ADD CONSTRAINT files_pkey PRIMARY KEY (identifier);


--
-- Name: film_format_pkey; Type: CONSTRAINT; Schema: public; Owner: po_user; Tablespace: 
--

ALTER TABLE ONLY film_format
    ADD CONSTRAINT film_format_pkey PRIMARY KEY (identifier);


--
-- Name: film_iso_pkey; Type: CONSTRAINT; Schema: public; Owner: po_user; Tablespace: 
--

ALTER TABLE ONLY film_iso
    ADD CONSTRAINT film_iso_pkey PRIMARY KEY (identifier);


--
-- Name: film_pkey; Type: CONSTRAINT; Schema: public; Owner: po_user; Tablespace: 
--

ALTER TABLE ONLY film
    ADD CONSTRAINT film_pkey PRIMARY KEY (identifier);


--
-- Name: film_type_pkey; Type: CONSTRAINT; Schema: public; Owner: po_user; Tablespace: 
--

ALTER TABLE ONLY film_type
    ADD CONSTRAINT film_type_pkey PRIMARY KEY (identifier);


--
-- Name: filter_pkey; Type: CONSTRAINT; Schema: public; Owner: po_user; Tablespace: 
--

ALTER TABLE ONLY filter
    ADD CONSTRAINT filter_pkey PRIMARY KEY (identifier);


--
-- Name: filter_type_pkey; Type: CONSTRAINT; Schema: public; Owner: po_user; Tablespace: 
--

ALTER TABLE ONLY filter_type
    ADD CONSTRAINT filter_type_pkey PRIMARY KEY (identifier);


--
-- Name: flash_mode_pkey; Type: CONSTRAINT; Schema: public; Owner: po_user; Tablespace: 
--

ALTER TABLE ONLY flash_mode
    ADD CONSTRAINT flash_mode_pkey PRIMARY KEY (identifier);


--
-- Name: flash_pkey; Type: CONSTRAINT; Schema: public; Owner: po_user; Tablespace: 
--

ALTER TABLE ONLY flash
    ADD CONSTRAINT flash_pkey PRIMARY KEY (identifier);


--
-- Name: flash_type_pkey; Type: CONSTRAINT; Schema: public; Owner: po_user; Tablespace: 
--

ALTER TABLE ONLY flash_type
    ADD CONSTRAINT flash_type_pkey PRIMARY KEY (identifier);


--
-- Name: folder_pkey; Type: CONSTRAINT; Schema: public; Owner: po_user; Tablespace: 
--

ALTER TABLE ONLY folder
    ADD CONSTRAINT folder_pkey PRIMARY KEY (identifier);


--
-- Name: label_pkey; Type: CONSTRAINT; Schema: public; Owner: po_user; Tablespace: 
--

ALTER TABLE ONLY label
    ADD CONSTRAINT label_pkey PRIMARY KEY (identifier);


--
-- Name: lens_pkey; Type: CONSTRAINT; Schema: public; Owner: po_user; Tablespace: 
--

ALTER TABLE ONLY lens
    ADD CONSTRAINT lens_pkey PRIMARY KEY (identifier);


--
-- Name: lens_type_pkey; Type: CONSTRAINT; Schema: public; Owner: po_user; Tablespace: 
--

ALTER TABLE ONLY lens_type
    ADD CONSTRAINT lens_type_pkey PRIMARY KEY (identifier);


--
-- Name: location_pkey; Type: CONSTRAINT; Schema: public; Owner: po_user; Tablespace: 
--

ALTER TABLE ONLY "location"
    ADD CONSTRAINT location_pkey PRIMARY KEY (identifier);


--
-- Name: manufacturer_pkey; Type: CONSTRAINT; Schema: public; Owner: po_user; Tablespace: 
--

ALTER TABLE ONLY manufacturer
    ADD CONSTRAINT manufacturer_pkey PRIMARY KEY (identifier);


--
-- Name: orientations_pkey; Type: CONSTRAINT; Schema: public; Owner: po_user; Tablespace: 
--

ALTER TABLE ONLY orientations
    ADD CONSTRAINT orientations_pkey PRIMARY KEY (identifier);


--
-- Name: paper_pkey; Type: CONSTRAINT; Schema: public; Owner: po_user; Tablespace: 
--

ALTER TABLE ONLY paper
    ADD CONSTRAINT paper_pkey PRIMARY KEY (identifier);


--
-- Name: photo_dupe_pkey; Type: CONSTRAINT; Schema: public; Owner: po_user; Tablespace: 
--

ALTER TABLE ONLY photo_dupe
    ADD CONSTRAINT photo_dupe_pkey PRIMARY KEY (identifier);


--
-- Name: photo_pkey; Type: CONSTRAINT; Schema: public; Owner: po_user; Tablespace: 
--

ALTER TABLE ONLY photo
    ADD CONSTRAINT photo_pkey PRIMARY KEY (identifier);


--
-- Name: photo_tech_pkey; Type: CONSTRAINT; Schema: public; Owner: po_user; Tablespace: 
--

ALTER TABLE ONLY photo_tech
    ADD CONSTRAINT photo_tech_pkey PRIMARY KEY (identifier);


--
-- Name: photo_version_pkey; Type: CONSTRAINT; Schema: public; Owner: po_user; Tablespace: 
--

ALTER TABLE ONLY photo_version
    ADD CONSTRAINT photo_version_pkey PRIMARY KEY (identifier);


--
-- Name: rating_pkey; Type: CONSTRAINT; Schema: public; Owner: po_user; Tablespace: 
--

ALTER TABLE ONLY rating
    ADD CONSTRAINT rating_pkey PRIMARY KEY (identifier);


--
-- Name: scanner_pkey; Type: CONSTRAINT; Schema: public; Owner: po_user; Tablespace: 
--

ALTER TABLE ONLY scanner
    ADD CONSTRAINT scanner_pkey PRIMARY KEY (identifier);


--
-- Name: scanner_type_pkey; Type: CONSTRAINT; Schema: public; Owner: po_user; Tablespace: 
--

ALTER TABLE ONLY scanner_type
    ADD CONSTRAINT scanner_type_pkey PRIMARY KEY (identifier);


--
-- Name: shutter_pkey; Type: CONSTRAINT; Schema: public; Owner: po_user; Tablespace: 
--

ALTER TABLE ONLY shutter
    ADD CONSTRAINT shutter_pkey PRIMARY KEY (identifier);


--
-- Name: state_pkey; Type: CONSTRAINT; Schema: public; Owner: po_user; Tablespace: 
--

ALTER TABLE ONLY state
    ADD CONSTRAINT state_pkey PRIMARY KEY (identifier);


--
-- Name: support_pkey; Type: CONSTRAINT; Schema: public; Owner: po_user; Tablespace: 
--

ALTER TABLE ONLY support
    ADD CONSTRAINT support_pkey PRIMARY KEY (identifier);


--
-- Name: support_type_pkey; Type: CONSTRAINT; Schema: public; Owner: po_user; Tablespace: 
--

ALTER TABLE ONLY support_type
    ADD CONSTRAINT support_type_pkey PRIMARY KEY (identifier);


--
-- Name: user_type_pkey; Type: CONSTRAINT; Schema: public; Owner: po_user; Tablespace: 
--

ALTER TABLE ONLY user_type
    ADD CONSTRAINT user_type_pkey PRIMARY KEY (identifier);


--
-- Name: users_pkey; Type: CONSTRAINT; Schema: public; Owner: po_user; Tablespace: 
--

ALTER TABLE ONLY users
    ADD CONSTRAINT users_pkey PRIMARY KEY (identifier);


--
-- Name: version_pkey; Type: CONSTRAINT; Schema: public; Owner: po_user; Tablespace: 
--

ALTER TABLE ONLY version
    ADD CONSTRAINT version_pkey PRIMARY KEY (identifier);


--
-- Name: album_content_album_idx; Type: INDEX; Schema: public; Owner: po_user; Tablespace: 
--

CREATE INDEX album_content_album_idx ON album_content USING btree (album);


--
-- Name: album_content_photo_idx; Type: INDEX; Schema: public; Owner: po_user; Tablespace: 
--

CREATE INDEX album_content_photo_idx ON album_content USING btree (photo);


--
-- Name: album_content_version_idx; Type: INDEX; Schema: public; Owner: po_user; Tablespace: 
--

CREATE INDEX album_content_version_idx ON album_content USING btree (version);


--
-- Name: album_event_idx; Type: INDEX; Schema: public; Owner: po_user; Tablespace: 
--

CREATE INDEX album_event_idx ON album USING btree (event);


--
-- Name: album_users_idx; Type: INDEX; Schema: public; Owner: po_user; Tablespace: 
--

CREATE INDEX album_users_idx ON album USING btree (users);


--
-- Name: calendar_client_idx; Type: INDEX; Schema: public; Owner: po_user; Tablespace: 
--

CREATE INDEX calendar_client_idx ON calendar USING btree (client);


--
-- Name: calendar_id_idx; Type: INDEX; Schema: public; Owner: po_user; Tablespace: 
--

CREATE INDEX calendar_id_idx ON calendar USING btree (identifier);


--
-- Name: calendar_user_idx; Type: INDEX; Schema: public; Owner: po_user; Tablespace: 
--

CREATE INDEX calendar_user_idx ON calendar USING btree (users);


--
-- Name: camera_type_idx; Type: INDEX; Schema: public; Owner: po_user; Tablespace: 
--

CREATE INDEX camera_type_idx ON camera USING btree ("type");


--
-- Name: camera_type_manufacturer_idx; Type: INDEX; Schema: public; Owner: po_user; Tablespace: 
--

CREATE INDEX camera_type_manufacturer_idx ON camera_type USING btree (manufacturer);


--
-- Name: camera_users_idx; Type: INDEX; Schema: public; Owner: po_user; Tablespace: 
--

CREATE INDEX camera_users_idx ON camera USING btree (users);


--
-- Name: client_client_idx; Type: INDEX; Schema: public; Owner: po_user; Tablespace: 
--

CREATE INDEX client_client_idx ON client USING btree (client);


--
-- Name: client_id_idx; Type: INDEX; Schema: public; Owner: po_user; Tablespace: 
--

CREATE INDEX client_id_idx ON client USING btree (identifier);


--
-- Name: client_users_idx; Type: INDEX; Schema: public; Owner: po_user; Tablespace: 
--

CREATE INDEX client_users_idx ON client USING btree (users);


--
-- Name: files_path_index; Type: INDEX; Schema: public; Owner: po_user; Tablespace: 
--

CREATE INDEX files_path_index ON files USING btree (path);


--
-- Name: files_version_idx; Type: INDEX; Schema: public; Owner: po_user; Tablespace: 
--

CREATE INDEX files_version_idx ON files USING btree (version);


--
-- Name: film_type_idx; Type: INDEX; Schema: public; Owner: po_user; Tablespace: 
--

CREATE INDEX film_type_idx ON film USING btree ("type");


--
-- Name: film_type_manufacturer_idx; Type: INDEX; Schema: public; Owner: po_user; Tablespace: 
--

CREATE INDEX film_type_manufacturer_idx ON film_type USING btree (manufacturer);


--
-- Name: film_users_idx; Type: INDEX; Schema: public; Owner: po_user; Tablespace: 
--

CREATE INDEX film_users_idx ON film USING btree (users);


--
-- Name: filter_type_idx; Type: INDEX; Schema: public; Owner: po_user; Tablespace: 
--

CREATE INDEX filter_type_idx ON filter USING btree ("type");


--
-- Name: filter_type_manufacturer_idx; Type: INDEX; Schema: public; Owner: po_user; Tablespace: 
--

CREATE INDEX filter_type_manufacturer_idx ON filter_type USING btree (manufacturer);


--
-- Name: filter_users_idx; Type: INDEX; Schema: public; Owner: po_user; Tablespace: 
--

CREATE INDEX filter_users_idx ON filter USING btree (users);


--
-- Name: flash_type_idx; Type: INDEX; Schema: public; Owner: po_user; Tablespace: 
--

CREATE INDEX flash_type_idx ON flash USING btree ("type");


--
-- Name: flash_type_manufacturer_idx; Type: INDEX; Schema: public; Owner: po_user; Tablespace: 
--

CREATE INDEX flash_type_manufacturer_idx ON flash_type USING btree (manufacturer);


--
-- Name: flash_users_idx; Type: INDEX; Schema: public; Owner: po_user; Tablespace: 
--

CREATE INDEX flash_users_idx ON flash USING btree (users);


--
-- Name: folder_event_idx; Type: INDEX; Schema: public; Owner: po_user; Tablespace: 
--

CREATE INDEX folder_event_idx ON folder USING btree (event);


--
-- Name: folder_parent_idx; Type: INDEX; Schema: public; Owner: po_user; Tablespace: 
--

CREATE INDEX folder_parent_idx ON folder USING btree (parent_folder);


--
-- Name: folder_users_idx; Type: INDEX; Schema: public; Owner: po_user; Tablespace: 
--

CREATE INDEX folder_users_idx ON folder USING btree (users);


--
-- Name: label_idx; Type: INDEX; Schema: public; Owner: po_user; Tablespace: 
--

CREATE INDEX label_idx ON label USING btree (identifier);


--
-- Name: lens_type_idx; Type: INDEX; Schema: public; Owner: po_user; Tablespace: 
--

CREATE INDEX lens_type_idx ON lens USING btree ("type");


--
-- Name: lens_type_manufacturer_idx; Type: INDEX; Schema: public; Owner: po_user; Tablespace: 
--

CREATE INDEX lens_type_manufacturer_idx ON lens_type USING btree (manufacturer);


--
-- Name: lens_users_idx; Type: INDEX; Schema: public; Owner: po_user; Tablespace: 
--

CREATE INDEX lens_users_idx ON lens USING btree (users);


--
-- Name: paper_idx; Type: INDEX; Schema: public; Owner: po_user; Tablespace: 
--

CREATE INDEX paper_idx ON paper USING btree (identifier);


--
-- Name: photo_dupe_key; Type: INDEX; Schema: public; Owner: po_user; Tablespace: 
--

CREATE INDEX photo_dupe_key ON photo_dupe USING btree ("key");


--
-- Name: photo_dupe_version_idx; Type: INDEX; Schema: public; Owner: po_user; Tablespace: 
--

CREATE INDEX photo_dupe_version_idx ON photo_dupe USING btree (version);


--
-- Name: photo_folder; Type: INDEX; Schema: public; Owner: po_user; Tablespace: 
--

CREATE INDEX photo_folder ON photo USING btree (folder);


--
-- Name: photo_keywords_photo; Type: INDEX; Schema: public; Owner: po_user; Tablespace: 
--

CREATE INDEX photo_keywords_photo ON photo_keywords USING btree (photo);


--
-- Name: photo_location_idx; Type: INDEX; Schema: public; Owner: po_user; Tablespace: 
--

CREATE INDEX photo_location_idx ON photo USING btree ("location");


--
-- Name: photo_tech_photo_idx; Type: INDEX; Schema: public; Owner: po_user; Tablespace: 
--

CREATE INDEX photo_tech_photo_idx ON photo_tech USING btree (photo);


--
-- Name: photo_users_idx; Type: INDEX; Schema: public; Owner: po_user; Tablespace: 
--

CREATE INDEX photo_users_idx ON photo USING btree (users);


--
-- Name: photo_version_photo_idx; Type: INDEX; Schema: public; Owner: po_user; Tablespace: 
--

CREATE INDEX photo_version_photo_idx ON photo_version USING btree (photo);


--
-- Name: rating_id_idx; Type: INDEX; Schema: public; Owner: po_user; Tablespace: 
--

CREATE INDEX rating_id_idx ON rating USING btree (identifier);


--
-- Name: rating_photo_idx; Type: INDEX; Schema: public; Owner: po_user; Tablespace: 
--

CREATE INDEX rating_photo_idx ON rating USING btree (photo);


--
-- Name: rating_users_idx; Type: INDEX; Schema: public; Owner: po_user; Tablespace: 
--

CREATE INDEX rating_users_idx ON rating USING btree (users);


--
-- Name: rating_version_idx; Type: INDEX; Schema: public; Owner: po_user; Tablespace: 
--

CREATE INDEX rating_version_idx ON rating USING btree (version);


--
-- Name: scanner_type_idx; Type: INDEX; Schema: public; Owner: po_user; Tablespace: 
--

CREATE INDEX scanner_type_idx ON scanner USING btree ("type");


--
-- Name: scanner_type_manufacturer_idx; Type: INDEX; Schema: public; Owner: po_user; Tablespace: 
--

CREATE INDEX scanner_type_manufacturer_idx ON scanner_type USING btree (manufacturer);


--
-- Name: scanner_users_idx; Type: INDEX; Schema: public; Owner: po_user; Tablespace: 
--

CREATE INDEX scanner_users_idx ON scanner USING btree (users);


--
-- Name: support_type_idx; Type: INDEX; Schema: public; Owner: po_user; Tablespace: 
--

CREATE INDEX support_type_idx ON support USING btree ("type");


--
-- Name: support_type_manufacturer_idx; Type: INDEX; Schema: public; Owner: po_user; Tablespace: 
--

CREATE INDEX support_type_manufacturer_idx ON support_type USING btree (manufacturer);


--
-- Name: support_users_idx; Type: INDEX; Schema: public; Owner: po_user; Tablespace: 
--

CREATE INDEX support_users_idx ON support USING btree (users);


--
-- Name: user_preferences_owner_idx; Type: INDEX; Schema: public; Owner: po_user; Tablespace: 
--

CREATE INDEX user_preferences_owner_idx ON user_preferences USING btree ("owner");


--
-- Name: album_content_trigger; Type: TRIGGER; Schema: public; Owner: po_user
--

CREATE TRIGGER album_content_trigger
    BEFORE INSERT OR DELETE ON album_content
    FOR EACH ROW
    EXECUTE PROCEDURE album_content_date_added_trig();


--
-- Name: album_content_update_trig; Type: TRIGGER; Schema: public; Owner: po_user
--

CREATE TRIGGER album_content_update_trig
    BEFORE INSERT OR DELETE OR UPDATE ON album_content
    FOR EACH ROW
    EXECUTE PROCEDURE date_changed_album_content();


--
-- Name: album_update_trig; Type: TRIGGER; Schema: public; Owner: po_user
--

CREATE TRIGGER album_update_trig
    BEFORE INSERT OR DELETE OR UPDATE ON album
    FOR EACH ROW
    EXECUTE PROCEDURE date_changed_album();


--
-- Name: calendar_date_added_trigger; Type: TRIGGER; Schema: public; Owner: po_user
--

CREATE TRIGGER calendar_date_added_trigger
    BEFORE INSERT ON calendar
    FOR EACH ROW
    EXECUTE PROCEDURE date_added_trig();


--
-- Name: client_date_added_trigger; Type: TRIGGER; Schema: public; Owner: po_user
--

CREATE TRIGGER client_date_added_trigger
    BEFORE INSERT ON client
    FOR EACH ROW
    EXECUTE PROCEDURE date_added_trig();


--
-- Name: date_changed_files_trig; Type: TRIGGER; Schema: public; Owner: po_user
--

CREATE TRIGGER date_changed_files_trig
    BEFORE INSERT OR UPDATE ON files
    FOR EACH ROW
    EXECUTE PROCEDURE date_changed_files();


--
-- Name: folder_update_trig; Type: TRIGGER; Schema: public; Owner: po_user
--

CREATE TRIGGER folder_update_trig
    BEFORE INSERT OR DELETE OR UPDATE ON folder
    FOR EACH ROW
    EXECUTE PROCEDURE date_changed_folder();


--
-- Name: photo_date_changed_trigger; Type: TRIGGER; Schema: public; Owner: po_user
--

CREATE TRIGGER photo_date_changed_trigger
    BEFORE INSERT OR DELETE OR UPDATE ON photo
    FOR EACH ROW
    EXECUTE PROCEDURE photo_date_changed_trig();


--
-- Name: photo_dupe_date_added_trigger; Type: TRIGGER; Schema: public; Owner: po_user
--

CREATE TRIGGER photo_dupe_date_added_trigger
    BEFORE INSERT ON photo_dupe
    FOR EACH ROW
    EXECUTE PROCEDURE date_added_trig();


--
-- Name: photo_keywords_date_changed_trigger; Type: TRIGGER; Schema: public; Owner: po_user
--

CREATE TRIGGER photo_keywords_date_changed_trigger
    BEFORE INSERT OR DELETE OR UPDATE ON photo_keywords
    FOR EACH ROW
    EXECUTE PROCEDURE photo_tech_date_changed_trig();


--
-- Name: photo_tech_date_changed_trigger; Type: TRIGGER; Schema: public; Owner: po_user
--

CREATE TRIGGER photo_tech_date_changed_trigger
    BEFORE INSERT OR UPDATE ON photo_tech
    FOR EACH ROW
    EXECUTE PROCEDURE photo_tech_date_changed_trig();


--
-- Name: photo_version_date_changed_trigger; Type: TRIGGER; Schema: public; Owner: po_user
--

CREATE TRIGGER photo_version_date_changed_trigger
    BEFORE INSERT OR UPDATE ON photo_version
    FOR EACH ROW
    EXECUTE PROCEDURE photo_version_date_changed_trig();


--
-- Name: rating_date_added_trigger; Type: TRIGGER; Schema: public; Owner: po_user
--

CREATE TRIGGER rating_date_added_trigger
    BEFORE INSERT ON rating
    FOR EACH ROW
    EXECUTE PROCEDURE date_added_trig();


--
-- Name: album_access_rights_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY album
    ADD CONSTRAINT album_access_rights_fkey FOREIGN KEY (access_rights) REFERENCES access_type(identifier);


--
-- Name: album_content_album_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY album_content
    ADD CONSTRAINT album_content_album_fkey FOREIGN KEY (album) REFERENCES album(identifier);


--
-- Name: album_content_photo_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY album_content
    ADD CONSTRAINT album_content_photo_fkey FOREIGN KEY (photo) REFERENCES photo(identifier);


--
-- Name: album_content_version_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY album_content
    ADD CONSTRAINT album_content_version_fkey FOREIGN KEY (version) REFERENCES photo_version(identifier);


--
-- Name: album_event_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY album
    ADD CONSTRAINT album_event_fkey FOREIGN KEY (event) REFERENCES calendar(identifier);


--
-- Name: album_parent_album_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY album
    ADD CONSTRAINT album_parent_album_fkey FOREIGN KEY (parent_album) REFERENCES album(identifier);


--
-- Name: album_users_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY album
    ADD CONSTRAINT album_users_fkey FOREIGN KEY (users) REFERENCES users(identifier);


--
-- Name: calendar_client_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY calendar
    ADD CONSTRAINT calendar_client_fkey FOREIGN KEY (client) REFERENCES users(identifier);


--
-- Name: calendar_users_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY calendar
    ADD CONSTRAINT calendar_users_fkey FOREIGN KEY (users) REFERENCES users(identifier);


--
-- Name: camera_access_rights_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY camera
    ADD CONSTRAINT camera_access_rights_fkey FOREIGN KEY (access_rights) REFERENCES access_type(identifier);


--
-- Name: camera_type_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY camera
    ADD CONSTRAINT camera_type_fkey FOREIGN KEY ("type") REFERENCES camera_type(identifier);


--
-- Name: camera_type_last_modifying_users_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY camera_type
    ADD CONSTRAINT camera_type_last_modifying_users_fkey FOREIGN KEY (last_modifying_users) REFERENCES users(identifier);


--
-- Name: camera_type_manufacturer_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY camera_type
    ADD CONSTRAINT camera_type_manufacturer_fkey FOREIGN KEY (manufacturer) REFERENCES manufacturer(identifier);


--
-- Name: camera_users_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY camera
    ADD CONSTRAINT camera_users_fkey FOREIGN KEY (users) REFERENCES users(identifier);


--
-- Name: client_client_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY client
    ADD CONSTRAINT client_client_fkey FOREIGN KEY (client) REFERENCES users(identifier);


--
-- Name: client_status_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY client
    ADD CONSTRAINT client_status_fkey FOREIGN KEY (status) REFERENCES client_status(identifier);


--
-- Name: client_users_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY client
    ADD CONSTRAINT client_users_fkey FOREIGN KEY (users) REFERENCES users(identifier);


--
-- Name: files_version_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY files
    ADD CONSTRAINT files_version_fkey FOREIGN KEY (version) REFERENCES photo_version(identifier);


--
-- Name: files_watermark_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY files
    ADD CONSTRAINT files_watermark_fkey FOREIGN KEY (watermark) REFERENCES photo_version(identifier);


--
-- Name: film_access_rights_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY film
    ADD CONSTRAINT film_access_rights_fkey FOREIGN KEY (access_rights) REFERENCES access_type(identifier);


--
-- Name: film_type_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY film
    ADD CONSTRAINT film_type_fkey FOREIGN KEY ("type") REFERENCES film_type(identifier);


--
-- Name: film_type_format_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY film_type
    ADD CONSTRAINT film_type_format_fkey FOREIGN KEY (format) REFERENCES film_format(identifier);


--
-- Name: film_type_iso_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY film_type
    ADD CONSTRAINT film_type_iso_fkey FOREIGN KEY (iso) REFERENCES film_iso(identifier);


--
-- Name: film_type_last_modifying_users_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY film_type
    ADD CONSTRAINT film_type_last_modifying_users_fkey FOREIGN KEY (last_modifying_users) REFERENCES users(identifier);


--
-- Name: film_type_manufacturer_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY film_type
    ADD CONSTRAINT film_type_manufacturer_fkey FOREIGN KEY (manufacturer) REFERENCES manufacturer(identifier);


--
-- Name: film_users_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY film
    ADD CONSTRAINT film_users_fkey FOREIGN KEY (users) REFERENCES users(identifier);


--
-- Name: filter_access_rights_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY filter
    ADD CONSTRAINT filter_access_rights_fkey FOREIGN KEY (access_rights) REFERENCES access_type(identifier);


--
-- Name: filter_type_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY filter
    ADD CONSTRAINT filter_type_fkey FOREIGN KEY ("type") REFERENCES filter_type(identifier);


--
-- Name: filter_type_last_modifying_users_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY filter_type
    ADD CONSTRAINT filter_type_last_modifying_users_fkey FOREIGN KEY (last_modifying_users) REFERENCES users(identifier);


--
-- Name: filter_type_manufacturer_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY filter_type
    ADD CONSTRAINT filter_type_manufacturer_fkey FOREIGN KEY (manufacturer) REFERENCES manufacturer(identifier);


--
-- Name: filter_users_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY filter
    ADD CONSTRAINT filter_users_fkey FOREIGN KEY (users) REFERENCES users(identifier);


--
-- Name: flash_access_rights_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY flash
    ADD CONSTRAINT flash_access_rights_fkey FOREIGN KEY (access_rights) REFERENCES access_type(identifier);


--
-- Name: flash_type_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY flash
    ADD CONSTRAINT flash_type_fkey FOREIGN KEY ("type") REFERENCES flash_type(identifier);


--
-- Name: flash_type_last_modifying_users_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY flash_type
    ADD CONSTRAINT flash_type_last_modifying_users_fkey FOREIGN KEY (last_modifying_users) REFERENCES users(identifier);


--
-- Name: flash_type_manufacturer_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY flash_type
    ADD CONSTRAINT flash_type_manufacturer_fkey FOREIGN KEY (manufacturer) REFERENCES manufacturer(identifier);


--
-- Name: flash_users_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY flash
    ADD CONSTRAINT flash_users_fkey FOREIGN KEY (users) REFERENCES users(identifier);


--
-- Name: folder_access_rights_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY folder
    ADD CONSTRAINT folder_access_rights_fkey FOREIGN KEY (access_rights) REFERENCES access_type(identifier);


--
-- Name: folder_event_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY folder
    ADD CONSTRAINT folder_event_fkey FOREIGN KEY (event) REFERENCES calendar(identifier);


--
-- Name: folder_parent_folder_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY folder
    ADD CONSTRAINT folder_parent_folder_fkey FOREIGN KEY (parent_folder) REFERENCES folder(identifier);


--
-- Name: folder_users_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY folder
    ADD CONSTRAINT folder_users_fkey FOREIGN KEY (users) REFERENCES users(identifier);


--
-- Name: label_manufacturer_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY label
    ADD CONSTRAINT label_manufacturer_fkey FOREIGN KEY (manufacturer) REFERENCES manufacturer(identifier);


--
-- Name: label_paper_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY label
    ADD CONSTRAINT label_paper_fkey FOREIGN KEY (paper) REFERENCES paper(identifier);


--
-- Name: label_users_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY label
    ADD CONSTRAINT label_users_fkey FOREIGN KEY (users) REFERENCES users(identifier);


--
-- Name: lens_access_rights_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY lens
    ADD CONSTRAINT lens_access_rights_fkey FOREIGN KEY (access_rights) REFERENCES access_type(identifier);


--
-- Name: lens_type_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY lens
    ADD CONSTRAINT lens_type_fkey FOREIGN KEY ("type") REFERENCES lens_type(identifier);


--
-- Name: lens_type_last_modifying_users_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY lens_type
    ADD CONSTRAINT lens_type_last_modifying_users_fkey FOREIGN KEY (last_modifying_users) REFERENCES users(identifier);


--
-- Name: lens_type_manufacturer_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY lens_type
    ADD CONSTRAINT lens_type_manufacturer_fkey FOREIGN KEY (manufacturer) REFERENCES manufacturer(identifier);


--
-- Name: lens_users_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY lens
    ADD CONSTRAINT lens_users_fkey FOREIGN KEY (users) REFERENCES users(identifier);


--
-- Name: location_country_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY "location"
    ADD CONSTRAINT location_country_fkey FOREIGN KEY (country) REFERENCES country(identifier);


--
-- Name: location_last_modifying_users_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY "location"
    ADD CONSTRAINT location_last_modifying_users_fkey FOREIGN KEY (users) REFERENCES users(identifier);


--
-- Name: location_state_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY "location"
    ADD CONSTRAINT location_state_fkey FOREIGN KEY (state) REFERENCES state(identifier);


--
-- Name: manufacturer_last_modifying_users_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY manufacturer
    ADD CONSTRAINT manufacturer_last_modifying_users_fkey FOREIGN KEY (last_modifying_users) REFERENCES users(identifier);


--
-- Name: paper_last_modifying_users_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY paper
    ADD CONSTRAINT paper_last_modifying_users_fkey FOREIGN KEY (last_modifying_users) REFERENCES users(identifier);


--
-- Name: photo_access_rights_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY photo
    ADD CONSTRAINT photo_access_rights_fkey FOREIGN KEY (access_rights) REFERENCES access_type(identifier);


--
-- Name: photo_dupe_photo_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY photo_dupe
    ADD CONSTRAINT photo_dupe_photo_fkey FOREIGN KEY (photo) REFERENCES photo(identifier);


--
-- Name: photo_dupe_version_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY photo_dupe
    ADD CONSTRAINT photo_dupe_version_fkey FOREIGN KEY (version) REFERENCES photo_version(identifier);


--
-- Name: photo_folder_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY photo
    ADD CONSTRAINT photo_folder_fkey FOREIGN KEY (folder) REFERENCES folder(identifier);


--
-- Name: photo_keywords_photo_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY photo_keywords
    ADD CONSTRAINT photo_keywords_photo_fkey FOREIGN KEY (photo) REFERENCES photo(identifier);


--
-- Name: photo_location_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY photo
    ADD CONSTRAINT photo_location_fkey FOREIGN KEY ("location") REFERENCES "location"(identifier);


--
-- Name: photo_tech_camera_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY photo_tech
    ADD CONSTRAINT photo_tech_camera_fkey FOREIGN KEY (camera) REFERENCES camera(identifier);


--
-- Name: photo_tech_camera_metering_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY photo_tech
    ADD CONSTRAINT photo_tech_camera_metering_fkey FOREIGN KEY (camera_metering) REFERENCES camera_metering(identifier);


--
-- Name: photo_tech_camera_program_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY photo_tech
    ADD CONSTRAINT photo_tech_camera_program_fkey FOREIGN KEY (camera_program) REFERENCES camera_program(identifier);


--
-- Name: photo_tech_ev_difference_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY photo_tech
    ADD CONSTRAINT photo_tech_ev_difference_fkey FOREIGN KEY (ev_difference) REFERENCES ev_comp(identifier);


--
-- Name: photo_tech_exposure_comp_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY photo_tech
    ADD CONSTRAINT photo_tech_exposure_comp_fkey FOREIGN KEY (exposure_comp) REFERENCES ev_comp(identifier);


--
-- Name: photo_tech_film_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY photo_tech
    ADD CONSTRAINT photo_tech_film_fkey FOREIGN KEY (film) REFERENCES film(identifier);


--
-- Name: photo_tech_filter_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY photo_tech
    ADD CONSTRAINT photo_tech_filter_fkey FOREIGN KEY (filter) REFERENCES filter(identifier);


--
-- Name: photo_tech_flash_comp_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY photo_tech
    ADD CONSTRAINT photo_tech_flash_comp_fkey FOREIGN KEY (flash_comp) REFERENCES ev_comp(identifier);


--
-- Name: photo_tech_flash_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY photo_tech
    ADD CONSTRAINT photo_tech_flash_fkey FOREIGN KEY (flash) REFERENCES flash(identifier);


--
-- Name: photo_tech_flash_mode_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY photo_tech
    ADD CONSTRAINT photo_tech_flash_mode_fkey FOREIGN KEY (flash_mode) REFERENCES flash_mode(identifier);


--
-- Name: photo_tech_iso_override_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY photo_tech
    ADD CONSTRAINT photo_tech_iso_override_fkey FOREIGN KEY (iso_override) REFERENCES film_iso(identifier);


--
-- Name: photo_tech_lens_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY photo_tech
    ADD CONSTRAINT photo_tech_lens_fkey FOREIGN KEY (lens) REFERENCES lens(identifier);


--
-- Name: photo_tech_photo_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY photo_tech
    ADD CONSTRAINT photo_tech_photo_fkey FOREIGN KEY (photo) REFERENCES photo(identifier);


--
-- Name: photo_tech_scanner_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY photo_tech
    ADD CONSTRAINT photo_tech_scanner_fkey FOREIGN KEY (scanner) REFERENCES scanner(identifier);


--
-- Name: photo_tech_shutter_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY photo_tech
    ADD CONSTRAINT photo_tech_shutter_fkey FOREIGN KEY (shutter) REFERENCES shutter(identifier);


--
-- Name: photo_tech_support_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY photo_tech
    ADD CONSTRAINT photo_tech_support_fkey FOREIGN KEY (support) REFERENCES support(identifier);


--
-- Name: photo_users_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY photo
    ADD CONSTRAINT photo_users_fkey FOREIGN KEY (users) REFERENCES users(identifier);


--
-- Name: photo_version_orientation_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY photo_version
    ADD CONSTRAINT photo_version_orientation_fkey FOREIGN KEY (orientation) REFERENCES orientations(identifier);


--
-- Name: photo_version_photo_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY photo_version
    ADD CONSTRAINT photo_version_photo_fkey FOREIGN KEY (photo) REFERENCES photo(identifier);


--
-- Name: rating_photo_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY rating
    ADD CONSTRAINT rating_photo_fkey FOREIGN KEY (photo) REFERENCES photo(identifier);


--
-- Name: rating_users_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY rating
    ADD CONSTRAINT rating_users_fkey FOREIGN KEY (users) REFERENCES users(identifier);


--
-- Name: rating_version_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY rating
    ADD CONSTRAINT rating_version_fkey FOREIGN KEY (version) REFERENCES photo_version(identifier);


--
-- Name: scanner_access_rights_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY scanner
    ADD CONSTRAINT scanner_access_rights_fkey FOREIGN KEY (access_rights) REFERENCES access_type(identifier);


--
-- Name: scanner_type_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY scanner
    ADD CONSTRAINT scanner_type_fkey FOREIGN KEY ("type") REFERENCES scanner_type(identifier);


--
-- Name: scanner_type_last_modifying_users_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY scanner_type
    ADD CONSTRAINT scanner_type_last_modifying_users_fkey FOREIGN KEY (last_modifying_users) REFERENCES users(identifier);


--
-- Name: scanner_type_manufacturer_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY scanner_type
    ADD CONSTRAINT scanner_type_manufacturer_fkey FOREIGN KEY (manufacturer) REFERENCES manufacturer(identifier);


--
-- Name: scanner_users_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY scanner
    ADD CONSTRAINT scanner_users_fkey FOREIGN KEY (users) REFERENCES users(identifier);


--
-- Name: support_access_rights_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY support
    ADD CONSTRAINT support_access_rights_fkey FOREIGN KEY (access_rights) REFERENCES access_type(identifier);


--
-- Name: support_type_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY support
    ADD CONSTRAINT support_type_fkey FOREIGN KEY ("type") REFERENCES support_type(identifier);


--
-- Name: support_type_last_modifying_users_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY support_type
    ADD CONSTRAINT support_type_last_modifying_users_fkey FOREIGN KEY (last_modifying_users) REFERENCES users(identifier);


--
-- Name: support_type_manufacturer_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY support_type
    ADD CONSTRAINT support_type_manufacturer_fkey FOREIGN KEY (manufacturer) REFERENCES manufacturer(identifier);


--
-- Name: support_users_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY support
    ADD CONSTRAINT support_users_fkey FOREIGN KEY (users) REFERENCES users(identifier);


--
-- Name: user_preferences_owner_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY user_preferences
    ADD CONSTRAINT user_preferences_owner_fkey FOREIGN KEY ("owner") REFERENCES users(identifier);


--
-- Name: users_country_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY users
    ADD CONSTRAINT users_country_fkey FOREIGN KEY (country) REFERENCES country(identifier);


--
-- Name: users_state_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY users
    ADD CONSTRAINT users_state_fkey FOREIGN KEY (state) REFERENCES state(identifier);


--
-- Name: users_type_fkey; Type: FK CONSTRAINT; Schema: public; Owner: po_user
--

ALTER TABLE ONLY users
    ADD CONSTRAINT users_type_fkey FOREIGN KEY ("type") REFERENCES user_type(identifier);


--
-- Name: public; Type: ACL; Schema: -; Owner: postgres
--

REVOKE ALL ON SCHEMA public FROM PUBLIC;
REVOKE ALL ON SCHEMA public FROM postgres;
GRANT ALL ON SCHEMA public TO postgres;
GRANT ALL ON SCHEMA public TO PUBLIC;


--
-- PostgreSQL database dump complete
--

